@@ -1,57 +1,51 @@
-"""Defines the templaters."""
+"""Defines the dbt templater.
 
-from collections import deque
-from contextlib import contextmanager
+NOTE: The dbt python package adds a significant overhead to import.
+This module is also loaded on every run of SQLFluff regardless of
+whether the dbt templater is selected in the configuration.
+
+The templater is however only _instantiated_ when selected, and as
+such, all imports of the dbt libraries are contained within the
+DbtTemplater class and so are only imported when necessary.
+"""
+
+import logging
 import os
 import os.path
-import logging
-from typing import List, Optional, Iterator, Tuple, Any, Dict, Deque
-
+from collections import deque
+from contextlib import contextmanager
 from dataclasses import dataclass
-
-from dbt.version import get_installed_version
-from dbt.config import read_user_config
-from dbt.config.runtime import RuntimeConfig as DbtRuntimeConfig
-from dbt.adapters.factory import register_adapter, get_adapter
-from dbt.compilation import Compiler as DbtCompiler
-from dbt.exceptions import (
-    CompilationException as DbtCompilationException,
-    FailedToConnectException as DbtFailedToConnectException,
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Deque,
+    Dict,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Union,
 )
-from dbt import flags
+
 from jinja2 import Environment
 from jinja2_simple_tags import StandaloneTag
 
-from sqlfluff.cli.formatters import OutputStreamFormatter
-from sqlfluff.core import FluffConfig
 from sqlfluff.core.cached_property import cached_property
-from sqlfluff.core.errors import SQLTemplaterError, SQLFluffSkipFile
-
+from sqlfluff.core.errors import SQLFluffSkipFile, SQLFluffUserError, SQLTemplaterError
 from sqlfluff.core.templaters.base import TemplatedFile, large_file_check
-
 from sqlfluff.core.templaters.jinja import JinjaTemplater
 
+if TYPE_CHECKING:  # pragma: no cover
+    from dbt.semver import VersionSpecifier
+
+    from sqlfluff.cli.formatters import OutputStreamFormatter
+    from sqlfluff.core import FluffConfig
+
 # Instantiate the templater logger
 templater_logger = logging.getLogger("sqlfluff.templater")
 
 
-DBT_VERSION = get_installed_version()
-DBT_VERSION_STRING = DBT_VERSION.to_version_string()
-DBT_VERSION_TUPLE = (int(DBT_VERSION.major), int(DBT_VERSION.minor))
-
-if DBT_VERSION_TUPLE >= (1, 0):
-    from dbt.flags import PROFILES_DIR
-else:
-    from dbt.config.profile import PROFILES_DIR
-
-if DBT_VERSION_TUPLE >= (1, 3):
-    COMPILED_SQL_ATTRIBUTE = "compiled_code"
-    RAW_SQL_ATTRIBUTE = "raw_code"
-else:
-    COMPILED_SQL_ATTRIBUTE = "compiled_sql"
-    RAW_SQL_ATTRIBUTE = "raw_sql"
-
-
 @dataclass
 class DbtConfigArgs:
     """Arguments to load dbt runtime config."""
@@ -60,8 +54,18 @@ class DbtConfigArgs:
     profiles_dir: Optional[str] = None
     profile: Optional[str] = None
     target: Optional[str] = None
+    threads: int = 1
     single_threaded: bool = False
-    vars: str = ""
+    # dict in 1.5.x onwards, json string before.
+    # NOTE: We always set this value when instantiating this
+    # class. If we rely on defaults, this should default to
+    # an empty string pre 1.5.x
+    vars: Optional[Union[Dict, str]] = None
+    # NOTE: The `which` argument here isn't covered in tests, but many
+    # dbt packages assume that it will have been set.
+    # https://github.com/sqlfluff/sqlfluff/issues/4861
+    # https://github.com/sqlfluff/sqlfluff/issues/4965
+    which: Optional[str] = "compile"
 
 
 class DbtTemplater(JinjaTemplater):
@@ -80,24 +84,79 @@ class DbtTemplater(JinjaTemplater):
         self._sequential_fails = 0
         super().__init__(**kwargs)
 
-    def config_pairs(self):  # pragma: no cover TODO?
+    def config_pairs(self):
         """Returns info about the given templater for output by the cli."""
         return [("templater", self.name), ("dbt", self.dbt_version)]
 
-    @property
+    @cached_property
+    def _dbt_version(self) -> "VersionSpecifier":
+        """Fetches the installed dbt version.
+
+        This is cached in the raw dbt format.
+
+        NOTE: We do this only on demand to reduce the amount of loading
+        required to discover the templater.
+        """
+        from dbt.version import get_installed_version
+
+        return get_installed_version()
+
+    @cached_property
     def dbt_version(self):
         """Gets the dbt version."""
-        return DBT_VERSION_STRING
+        return self._dbt_version.to_version_string()
 
-    @property
+    @cached_property
     def dbt_version_tuple(self):
-        """Gets the dbt version as a tuple on (major, minor)."""
-        return DBT_VERSION_TUPLE
+        """Gets the dbt version."""
+        return int(self._dbt_version.major), int(self._dbt_version.minor)
+
+    def try_silence_dbt_logs(self) -> None:
+        """Attempt to silence dbt logs.
+
+        During normal operation dbt is likely to log output such as:
+
+        .. code-block::
+
+           14:13:10  Registered adapter: snowflake=1.6.0
+
+        This is emitted by dbt directly to stdout/stderr, and so for us
+        to silence it (e.g. when outputting to json or yaml) we need to
+        reach into the internals of dbt and silence it directly.
+
+        https://github.com/sqlfluff/sqlfluff/issues/5054
+
+        NOTE: We wrap this in a try clause so that if the API changes
+        within dbt that we don't get a direct fail. This was tested on
+        dbt-code==1.6.0.
+        """
+        # First check whether we need to silence the logs. If a formatter
+        # is present then assume that it's not a problem
+        if not self.formatter:
+            try:
+                from dbt.events.functions import cleanup_event_logger
+
+                cleanup_event_logger()
+            except ImportError:
+                pass
 
     @cached_property
     def dbt_config(self):
         """Loads the dbt config."""
-        if self.dbt_version_tuple >= (1, 0):
+        from dbt import flags
+        from dbt.adapters.factory import register_adapter
+        from dbt.config import read_user_config
+        from dbt.config.runtime import RuntimeConfig as DbtRuntimeConfig
+
+        # Attempt to silence internal logging at this point.
+        # https://github.com/sqlfluff/sqlfluff/issues/5054
+        self.try_silence_dbt_logs()
+
+        if self.dbt_version_tuple >= (1, 5):
+            user_config = None
+            # 1.5.x+ this is a dict.
+            cli_vars = self._get_cli_vars()
+        else:
             # Here, we read flags.PROFILE_DIR directly, prior to calling
             # set_from_args(). Apparently, set_from_args() sets PROFILES_DIR
             # to a lowercase version of the value, and the profile wouldn't be
@@ -105,22 +164,27 @@ class DbtTemplater(JinjaTemplater):
             # was suggested and described here:
             # https://github.com/sqlfluff/sqlfluff/issues/2253#issuecomment-1018722979
             user_config = read_user_config(flags.PROFILES_DIR)
-            flags.set_from_args(
-                DbtConfigArgs(
-                    project_dir=self.project_dir,
-                    profiles_dir=self.profiles_dir,
-                    profile=self._get_profile(),
-                    vars=self._get_cli_vars(),
-                ),
-                user_config,
-            )
+            # Pre 1.5.x this is a string.
+            cli_vars = str(self._get_cli_vars())
+
+        flags.set_from_args(
+            DbtConfigArgs(
+                project_dir=self.project_dir,
+                profiles_dir=self.profiles_dir,
+                profile=self._get_profile(),
+                vars=cli_vars,
+                threads=1,
+            ),
+            user_config,
+        )
         self.dbt_config = DbtRuntimeConfig.from_args(
             DbtConfigArgs(
                 project_dir=self.project_dir,
                 profiles_dir=self.profiles_dir,
                 profile=self._get_profile(),
                 target=self._get_target(),
-                vars=self._get_cli_vars(),
+                vars=cli_vars,
+                threads=1,
             )
         )
         register_adapter(self.dbt_config)
@@ -129,12 +193,26 @@ class DbtTemplater(JinjaTemplater):
     @cached_property
     def dbt_compiler(self):
         """Loads the dbt compiler."""
+        from dbt.compilation import Compiler as DbtCompiler
+
         self.dbt_compiler = DbtCompiler(self.dbt_config)
         return self.dbt_compiler
 
     @cached_property
     def dbt_manifest(self):
         """Loads the dbt manifest."""
+        from dbt.exceptions import DbtProjectError
+
+        # NOTE: The uninstalled packages error only exists from around
+        # dbt 1.4 onwards. Before that we'll just get a slightly uglier
+        # error - not a breaking issue.
+        try:
+            from dbt.exceptions import UninstalledPackagesFoundError
+
+            summary_errors = (DbtProjectError, UninstalledPackagesFoundError)
+        except ImportError:
+            summary_errors = (DbtProjectError,)
+
         # Set dbt not to run tracking. We don't load
         # a full project and so some tracking routines
         # may fail.
@@ -150,10 +228,17 @@ class DbtTemplater(JinjaTemplater):
             # Changing cwd temporarily as dbt is not using project_dir to
             # read/write `target/partial_parse.msgpack`. This can be undone when
             # https://github.com/dbt-labs/dbt-core/issues/6055 is solved.
-            os.chdir(self.project_dir)
+            # For dbt 1.4+ this isn't necessary, but it is required for 1.3
+            # and before.
+            if self.dbt_version_tuple < (1, 4):
+                os.chdir(self.project_dir)
             self.dbt_manifest = ManifestLoader.get_full_manifest(self.dbt_config)
+        except summary_errors as err:  # pragma: no cover
+            raise SQLFluffUserError(f"{err.__class__.__name__}: {err}")
         finally:
-            os.chdir(old_cwd)
+            if self.dbt_version_tuple < (1, 4):
+                os.chdir(old_cwd)
+
         return self.dbt_manifest
 
     @cached_property
@@ -166,6 +251,8 @@ class DbtTemplater(JinjaTemplater):
 
         from dbt.graph.selector_methods import (
             MethodManager as DbtSelectorMethodManager,
+        )
+        from dbt.graph.selector_methods import (
             MethodName as DbtMethodName,
         )
 
@@ -186,18 +273,37 @@ class DbtTemplater(JinjaTemplater):
     def _get_profiles_dir(self):
         """Get the dbt profiles directory from the configuration.
 
-        The default is `~/.dbt` in 0.17 but we use the
-        PROFILES_DIR variable from the dbt library to
+        The default is `~/.dbt` but we use the
+        default_profiles_dir from the dbt library to
         support a change of default in the future, as well
         as to support the same overwriting mechanism as
         dbt (currently an environment variable).
         """
+        # Where default_profiles_dir is available, use it. For dbt 1.2 and
+        # earlier, it is not, so fall back to the flags option which should
+        # still be available in those versions.
+
+        from dbt import flags
+
+        # From dbt 1.3 onwards, the default_profiles_dir resolver is
+        # available. Before that version we use the flags module
+        try:
+            from dbt.cli.resolvers import default_profiles_dir
+        except ImportError:
+            default_profiles_dir = None
+
+        default_dir = (
+            default_profiles_dir()
+            if default_profiles_dir is not None
+            else flags.PROFILES_DIR
+        )
+
         dbt_profiles_dir = os.path.abspath(
             os.path.expanduser(
                 self.sqlfluff_config.get_section(
                     (self.templater_selector, self.name, "profiles_dir")
                 )
-                or PROFILES_DIR
+                or (os.getenv("DBT_PROFILES_DIR") or default_dir)
             )
         )
 
@@ -242,12 +348,12 @@ class DbtTemplater(JinjaTemplater):
             (self.templater_selector, self.name, "target")
         )
 
-    def _get_cli_vars(self) -> str:
+    def _get_cli_vars(self) -> dict:
         cli_vars = self.sqlfluff_config.get_section(
             (self.templater_selector, self.name, "context")
         )
 
-        return str(cli_vars) if cli_vars else "{}"
+        return cli_vars if cli_vars else {}
 
     def sequence_files(
         self, fnames: List[str], config=None, formatter=None
@@ -330,8 +436,8 @@ class DbtTemplater(JinjaTemplater):
         *,
         fname: str,
         in_str: Optional[str] = None,
-        config: Optional[FluffConfig] = None,
-        formatter: Optional[OutputStreamFormatter] = None,
+        config: Optional["FluffConfig"] = None,
+        formatter: Optional["OutputStreamFormatter"] = None,
     ):
         """Compile a dbt model and return the compiled SQL.
 
@@ -349,36 +455,50 @@ class DbtTemplater(JinjaTemplater):
         self.profiles_dir = self._get_profiles_dir()
         fname_absolute_path = os.path.abspath(fname)
 
+        try:
+            # These are the names in dbt-core 1.4.1+
+            # https://github.com/dbt-labs/dbt-core/pull/6539
+            from dbt.exceptions import CompilationError, FailedToConnectError
+        except ImportError:
+            # These are the historic names for older dbt-core versions
+            from dbt.exceptions import CompilationException as CompilationError
+            from dbt.exceptions import (
+                FailedToConnectException as FailedToConnectError,
+            )
+
         try:
             os.chdir(self.project_dir)
             processed_result = self._unsafe_process(fname_absolute_path, in_str, config)
             # Reset the fail counter
             self._sequential_fails = 0
             return processed_result
-        except DbtCompilationException as e:
-            # Increment the counter
-            self._sequential_fails += 1
-            if e.node:
-                return None, [
-                    SQLTemplaterError(
-                        f"dbt compilation error on file '{e.node.original_file_path}', "
-                        f"{e.msg}",
-                        # It's fatal if we're over the limit
-                        fatal=self._sequential_fails > self.sequential_fail_limit,
-                    )
-                ]
-            else:
-                raise  # pragma: no cover
-        except DbtFailedToConnectException as e:
+        except FailedToConnectError as e:
             return None, [
                 SQLTemplaterError(
                     "dbt tried to connect to the database and failed: you could use "
-                    "'execute' to skip the database calls. See"
+                    "'execute' to skip the database calls. See "
                     "https://docs.getdbt.com/reference/dbt-jinja-functions/execute/ "
                     f"Error: {e.msg}",
                     fatal=True,
                 )
             ]
+        except CompilationError as e:
+            # Increment the counter
+            self._sequential_fails += 1
+            if e.node:
+                _msg = (
+                    f"dbt compilation error on file '{e.node.original_file_path}'"
+                    f", {e.msg}"
+                )
+            else:
+                _msg = f"dbt compilation error: {e.msg}"
+            return None, [
+                SQLTemplaterError(
+                    _msg,
+                    # It's fatal if we're over the limit
+                    fatal=self._sequential_fails > self.sequential_fail_limit,
+                )
+            ]
         # If a SQLFluff error is raised, just pass it through
         except SQLTemplaterError as e:  # pragma: no cover
             return None, [e]
@@ -425,17 +545,12 @@ class DbtTemplater(JinjaTemplater):
             if os.path.abspath(macro.original_file_path) == abspath:
                 return "a macro"
 
-        if DBT_VERSION_TUPLE >= (1, 0):
-            # Scan disabled nodes.
-            for nodes in self.dbt_manifest.disabled.values():
-                for node in nodes:
-                    if os.path.abspath(node.original_file_path) == abspath:
-                        return "disabled"
-        else:
-            model_name = os.path.splitext(os.path.basename(fname))[0]
-            if self.dbt_manifest.find_disabled_by_name(name=model_name):
-                return "disabled"
-        return None
+        # Scan disabled nodes.
+        for nodes in self.dbt_manifest.disabled.values():
+            for node in nodes:
+                if os.path.abspath(node.original_file_path) == abspath:
+                    return "disabled"
+        return None  # pragma: no cover
 
     def _unsafe_process(self, fname, in_str=None, config=None):
         original_file_path = os.path.relpath(fname, start=os.getcwd())
@@ -450,11 +565,20 @@ class DbtTemplater(JinjaTemplater):
         # turn is used by our parent class' (JinjaTemplater) slice_file()
         # function.
         old_from_string = Environment.from_string
-        make_template = None
+        # Start with render_func undefined. We need to know whether it has been
+        # overwritten.
+        render_func: Optional[Callable[[str], str]] = None
+
+        if self.dbt_version_tuple >= (1, 3):
+            compiled_sql_attribute = "compiled_code"
+            raw_sql_attribute = "raw_code"
+        else:  # pragma: no cover
+            compiled_sql_attribute = "compiled_sql"
+            raw_sql_attribute = "raw_sql"
 
         def from_string(*args, **kwargs):
             """Replaces (via monkeypatch) the jinja2.Environment function."""
-            nonlocal make_template
+            nonlocal render_func
             # Is it processing the node corresponding to fname?
             globals = kwargs.get("globals")
             if globals:
@@ -462,17 +586,37 @@ class DbtTemplater(JinjaTemplater):
                 if model:
                     if model.get("original_file_path") == original_file_path:
                         # Yes. Capture the important arguments and create
-                        # a make_template() function.
+                        # a render_func() closure with overwrites the variable
+                        # from within _unsafe_process when from_string is run.
                         env = args[0]
                         globals = args[2] if len(args) >= 3 else kwargs["globals"]
 
-                        def make_template(in_str):
+                        # Overwrite the outer render_func
+                        def render_func(in_str):
                             env.add_extension(SnapshotExtension)
-                            return env.from_string(in_str, globals=globals)
+                            template = env.from_string(in_str, globals=globals)
+                            return template.render()
 
             return old_from_string(*args, **kwargs)
 
+        # NOTE: We need to inject the project root here in reaction to the
+        # breaking change upstream with dbt. Coverage works in 1.5.2, but
+        # appears to no longer be covered in 1.5.3.
+        # This change was backported and so exists in some versions
+        # but not others. When not present, no additional action is needed.
+        # https://github.com/dbt-labs/dbt-core/pull/7949
+        # On the 1.5.x branch this was between 1.5.1 and 1.5.2
+        try:
+            from dbt.task.contextvars import cv_project_root
+
+            cv_project_root.set(self.project_dir)  # pragma: no cover
+        except ImportError:
+            cv_project_root = None
+
+        # NOTE: _find_node will raise a compilation exception if the project
+        # fails to compile, and we catch that in the outer `.process()` method.
         node = self._find_node(fname, config)
+
         templater_logger.debug(
             "_find_node for path %r returned object of type %s.", fname, type(node)
         )
@@ -483,6 +627,15 @@ class DbtTemplater(JinjaTemplater):
             if v.config.materialized == "ephemeral"
             and not getattr(v, "compiled", False)
         )
+
+        try:
+            # These are the names in dbt-core 1.4.1+
+            # https://github.com/dbt-labs/dbt-core/pull/6539
+            from dbt.exceptions import UndefinedMacroError
+        except ImportError:
+            # These are the historic names for older dbt-core versions
+            from dbt.exceptions import UndefinedMacroException as UndefinedMacroError
+
         with self.connection():
             # Apply the monkeypatch.
             Environment.from_string = from_string
@@ -491,8 +644,15 @@ class DbtTemplater(JinjaTemplater):
                     node=node,
                     manifest=self.dbt_manifest,
                 )
-            except Exception as err:
-                templater_logger.exception(
+            except UndefinedMacroError as err:
+                # The explanation on the undefined macro error is already fairly
+                # explanatory, so just pass it straight through.
+                raise SQLTemplaterError(str(err))
+            except Exception as err:  # pragma: no cover
+                # NOTE: We use .error() here rather than .exception() because
+                # for most users, the trace which accompanies the latter isn't
+                # particularly helpful.
+                templater_logger.error(
                     "Fatal dbt compilation error on %s. This occurs most often "
                     "during incorrect sorting of ephemeral models before linting. "
                     "Please report this error on github at "
@@ -504,7 +664,11 @@ class DbtTemplater(JinjaTemplater):
                 raise SQLFluffSkipFile(  # pragma: no cover
                     f"Skipped file {fname} because dbt raised a fatal "
                     f"exception during compilation: {err!s}"
-                ) from err
+                )
+                # NOTE: We don't do a `raise ... from err` here because the
+                # full trace is not useful for most users. In debugging
+                # issues here it may be valuable to add the `from err` part
+                # after the above `raise` statement.
             finally:
                 # Undo the monkeypatch.
                 Environment.from_string = old_from_string
@@ -513,11 +677,11 @@ class DbtTemplater(JinjaTemplater):
                 # If injected SQL is present, it contains a better picture
                 # of what will actually hit the database (e.g. with tests).
                 # However it's not always present.
-                compiled_sql = node.injected_sql
+                compiled_sql = node.injected_sql  # pragma: no cover
             else:
-                compiled_sql = getattr(node, COMPILED_SQL_ATTRIBUTE)
+                compiled_sql = getattr(node, compiled_sql_attribute)
 
-            raw_sql = getattr(node, RAW_SQL_ATTRIBUTE)
+            raw_sql = getattr(node, raw_sql_attribute)
 
             if not compiled_sql:  # pragma: no cover
                 raise SQLTemplaterError(
@@ -548,7 +712,7 @@ class DbtTemplater(JinjaTemplater):
             # compiling. Unless fixed (like below), this will cause:
             #    1. Assertion errors in TemplatedFile, when it sanity checks the
             #       contents of the sliced_file array.
-            #    2. L009 linting errors when running "sqlfluff lint foo_bar.sql"
+            #    2. LT12 linting errors when running "sqlfluff lint foo_bar.sql"
             #       since the linter will use the compiled code with the newlines
             #       removed.
             #    3. "No newline at end of file" warnings in Git/GitHub since
@@ -564,8 +728,24 @@ class DbtTemplater(JinjaTemplater):
             #    3. Append the count from #1 above to compiled_sql. (In
             #       production, slice_file() does not usually use this string,
             #       but some test scenarios do.
-            setattr(node, RAW_SQL_ATTRIBUTE, source_dbt_sql)
-            compiled_sql = compiled_sql + "\n" * n_trailing_newlines
+            setattr(node, raw_sql_attribute, source_dbt_sql)
+
+            # So for files that have no templated elements in them, render_func
+            # will still be null at this point. If so, we replace it with a lambda
+            # which just directly returns the input , but _also_ reset the trailing
+            # newlines counter because they also won't have been stripped.
+            if render_func is None:
+                # NOTE: In this case, we shouldn't re-add newlines, because they
+                # were never taken away.
+                n_trailing_newlines = 0
+
+                # Overwrite the render_func placeholder.
+                def render_func(in_str):
+                    """A render function which just returns the input."""
+                    return in_str
+
+            # At this point assert that we _have_ a render_func
+            assert render_func is not None
 
             # TRICKY: dbt configures Jinja2 with keep_trailing_newline=False.
             # As documented (https://jinja.palletsprojects.com/en/3.0.x/api/),
@@ -576,9 +756,8 @@ class DbtTemplater(JinjaTemplater):
             # Below, we use "append_to_templated" to effectively "undo" this.
             raw_sliced, sliced_file, templated_sql = self.slice_file(
                 source_dbt_sql,
-                compiled_sql,
+                render_func=render_func,
                 config=config,
-                make_template=make_template,
                 append_to_templated="\n" if n_trailing_newlines else "",
             )
         # :HACK: If calling compile_node() compiled any ephemeral nodes,
@@ -605,23 +784,22 @@ class DbtTemplater(JinjaTemplater):
     @contextmanager
     def connection(self):
         """Context manager that manages a dbt connection, if needed."""
+        from dbt.adapters.factory import get_adapter
+
         # We have to register the connection in dbt >= 1.0.0 ourselves
         # In previous versions, we relied on the functionality removed in
         # https://github.com/dbt-labs/dbt-core/pull/4062.
-        if DBT_VERSION_TUPLE >= (1, 0):
-            adapter = self.adapters.get(self.project_dir)
-            if adapter is None:
-                adapter = get_adapter(self.dbt_config)
-                self.adapters[self.project_dir] = adapter
-                adapter.acquire_connection("master")
-                adapter.set_relations_cache(self.dbt_manifest)
+        adapter = self.adapters.get(self.project_dir)
+        if adapter is None:
+            adapter = get_adapter(self.dbt_config)
+            self.adapters[self.project_dir] = adapter
+            adapter.acquire_connection("master")
+            adapter.set_relations_cache(self.dbt_manifest)
 
-            yield
-            # :TRICKY: Once connected, we never disconnect. Making multiple
-            # connections during linting has proven to cause major performance
-            # issues.
-        else:
-            yield
+        yield
+        # :TRICKY: Once connected, we never disconnect. Making multiple
+        # connections during linting has proven to cause major performance
+        # issues.
 
 
 class SnapshotExtension(StandaloneTag):
