@@ -1,24 +1,27 @@
 """Tests for the dbt templater."""
 
-from copy import deepcopy
 import glob
-import os
+import json
 import logging
+import os
 import shutil
+from copy import deepcopy
 from pathlib import Path
 from unittest import mock
 
 import pytest
 
+from sqlfluff.cli.commands import lint
 from sqlfluff.core import FluffConfig, Lexer, Linter
 from sqlfluff.core.errors import SQLFluffSkipFile
-from sqlfluff_templater_dbt.templater import DBT_VERSION_TUPLE
+from sqlfluff.utils.testing.cli import invoke_assert_code
+from sqlfluff.utils.testing.logging import fluff_log_catcher
+from sqlfluff_templater_dbt.templater import DbtTemplater
 from test.fixtures.dbt.templater import (  # noqa: F401
     DBT_FLUFF_CONFIG,
     dbt_templater,
     project_dir,
 )
-from sqlfluff_templater_dbt.templater import DbtFailedToConnectException, DbtTemplater
 
 
 def test__templater_dbt_missing(dbt_templater, project_dir):  # noqa: F811
@@ -83,6 +86,8 @@ def test__templater_dbt_profiles_dir_expanded(dbt_templater):  # noqa: F811
         "ends_with_whitespace_stripping.sql",
         # Access dbt graph nodes
         "access_graph_nodes.sql",
+        # Call statements
+        "call_statement.sql",
     ],
 )
 def test__templater_dbt_templating_result(
@@ -106,36 +111,34 @@ def test_dbt_profiles_dir_env_var_uppercase(
 
 def _run_templater_and_verify_result(dbt_templater, project_dir, fname):  # noqa: F811
     path = Path(project_dir) / "models/my_new_project" / fname
+    config = FluffConfig(configs=DBT_FLUFF_CONFIG)
     templated_file, _ = dbt_templater.process(
         in_str=path.read_text(),
         fname=str(path),
-        config=FluffConfig(configs=DBT_FLUFF_CONFIG),
+        config=config,
     )
     template_output_folder_path = Path(
         "plugins/sqlfluff-templater-dbt/test/fixtures/dbt/templated_output/"
     )
     fixture_path = _get_fixture_path(template_output_folder_path, fname)
     assert str(templated_file) == fixture_path.read_text()
+    # Check we can lex the output too.
+    # https://github.com/sqlfluff/sqlfluff/issues/4013
+    lexer = Lexer(config=config)
+    _, lexing_violations = lexer.lex(templated_file)
+    assert not lexing_violations
 
 
 def _get_fixture_path(template_output_folder_path, fname):
     fixture_path: Path = template_output_folder_path / fname  # Default fixture location
-    # Is there a version-specific version of the fixture file?
-    if DBT_VERSION_TUPLE >= (1, 0):
-        dbt_version_specific_fixture_folder = "dbt_utils_0.8.0"
-    else:
-        dbt_version_specific_fixture_folder = None
-
-    if dbt_version_specific_fixture_folder:
-        # Maybe. Determine where it would exist.
-        version_specific_path = (
-            Path(template_output_folder_path)
-            / dbt_version_specific_fixture_folder
-            / fname
-        )
-        if version_specific_path.is_file():
-            # Ok, it exists. Use this path instead.
-            fixture_path = version_specific_path
+    dbt_version_specific_fixture_folder = "dbt_utils_0.8.0"
+    # Determine where it would exist.
+    version_specific_path = (
+        Path(template_output_folder_path) / dbt_version_specific_fixture_folder / fname
+    )
+    if version_specific_path.is_file():
+        # Ok, it exists. Use this path instead.
+        fixture_path = version_specific_path
     return fixture_path
 
 
@@ -210,10 +213,22 @@ def test__templater_dbt_slice_file_wrapped_test(
     raw_file, templated_file, result, dbt_templater, caplog  # noqa: F811
 ):
     """Test that wrapped queries are sliced safely using _check_for_wrapped()."""
+
+    def _render_func(in_str) -> str:
+        """Create a dummy render func.
+
+        Importantly one that does actually allow different content to be added.
+        """
+        # Find the raw location in the template for the test case.
+        loc = templated_file.find(raw_file)
+        # Replace the new content at the previous position.
+        # NOTE: Doing this allows the tracer logic to do what it needs to do.
+        return templated_file[:loc] + in_str + templated_file[loc + len(raw_file) :]
+
     with caplog.at_level(logging.DEBUG, logger="sqlfluff.templater"):
         _, resp, _ = dbt_templater.slice_file(
             raw_file,
-            templated_file,
+            render_func=_render_func,
         )
     assert resp == result
 
@@ -233,18 +248,25 @@ def test__templater_dbt_templating_test_lex(
 
     Handle any number of newlines.
     """
-    source_fpath = os.path.join(project_dir, fname)
-    with open(source_fpath, "r") as source_dbt_model:
-        source_dbt_sql = source_dbt_model.read()
-    n_trailing_newlines = len(source_dbt_sql) - len(source_dbt_sql.rstrip("\n"))
-    lexer = Lexer(config=FluffConfig(configs=DBT_FLUFF_CONFIG))
     path = Path(project_dir) / fname
-    templated_file, _ = dbt_templater.process(
-        in_str=path.read_text(),
-        fname=str(path),
-        config=FluffConfig(configs=DBT_FLUFF_CONFIG),
+    config = FluffConfig(configs=DBT_FLUFF_CONFIG)
+    source_dbt_sql = path.read_text()
+    # Count the newlines.
+    n_trailing_newlines = len(source_dbt_sql) - len(source_dbt_sql.rstrip("\n"))
+    print(
+        f"Loaded {path!r} (n_newlines: {n_trailing_newlines}): " f"{source_dbt_sql!r}",
     )
-    tokens, lex_vs = lexer.lex(templated_file)
+
+    templated_file, _ = dbt_templater.process(
+        in_str=source_dbt_sql,
+        fname=str(path),
+        config=config,
+    )
+
+    lexer = Lexer(config=config)
+    # Test that it successfully lexes.
+    _, _ = lexer.lex(templated_file)
+
     assert (
         templated_file.source_str
         == "select a\nfrom table_a" + "\n" * n_trailing_newlines
@@ -286,17 +308,30 @@ def test__templater_dbt_skips_file(
         "use_var.sql",
         "incremental.sql",
         "single_trailing_newline.sql",
-        "L034_test.sql",
+        "ST06_test.sql",
     ],
 )
 def test__dbt_templated_models_do_not_raise_lint_error(
-    project_dir, fname  # noqa: F811
+    project_dir, fname, caplog  # noqa: F811
 ):
     """Test that templated dbt models do not raise a linting error."""
-    lntr = Linter(config=FluffConfig(configs=DBT_FLUFF_CONFIG))
-    lnt = lntr.lint_path(
-        path=os.path.join(project_dir, "models/my_new_project/", fname)
-    )
+    linter = Linter(config=FluffConfig(configs=DBT_FLUFF_CONFIG))
+    # Log rules output.
+    with caplog.at_level(logging.DEBUG, logger="sqlfluff.rules"):
+        lnt = linter.lint_path(
+            path=os.path.join(project_dir, "models/my_new_project/", fname)
+        )
+    for linted_file in lnt.files:
+        # Log the rendered file to facilitate better debugging of the files.
+        print(f"## FILE: {linted_file.path}")
+        print("\n\n## RENDERED FILE:\n\n")
+        print(linted_file.templated_file.templated_str)
+        print("\n\n## PARSED TREE:\n\n")
+        print(linted_file.tree.stringify())
+        print("\n\n## VIOLATIONS:")
+        for idx, v in enumerate(linted_file.violations):
+            print(f"   {idx}:{v.get_info_dict()}")
+
     violations = lnt.check_tuples()
     assert len(violations) == 0
 
@@ -351,9 +386,24 @@ def test__templater_dbt_templating_absolute_path(
     [
         (
             "compiler_error.sql",
-            "dbt compilation error on file 'models/my_new_project/compiler_error.sql', "
-            "Unexpected end of template. "
-            "Jinja was looking for the following tags: 'endfor'",
+            "dbt compilation error on file 'models/my_new_project/compiler_error.sql'"
+            ", Unexpected end of template. Jinja was looking for the following tags:"
+            " 'endfor' or 'else'. The innermost block that needs to be closed is "
+            "'for'.\n  line 5\n    {{ col }}",
+        ),
+        (
+            "unknown_ref.sql",
+            # https://github.com/sqlfluff/sqlfluff/issues/3849
+            "Model 'model.my_new_project.unknown_ref' "
+            "(models/my_new_project/unknown_ref.sql) depends on a node named "
+            "'i_do_not_exist' which was not found",
+        ),
+        (
+            "unknown_macro.sql",
+            # https://github.com/sqlfluff/sqlfluff/issues/3849
+            "Compilation Error in model unknown_macro "
+            "(models/my_new_project/unknown_macro.sql)\n  'invalid_macro' is "
+            "undefined. This can happen when calling a macro that does not exist.",
         ),
     ],
 )
@@ -381,12 +431,9 @@ def test__templater_dbt_handle_exceptions(
         get_adapter(dbt_templater.dbt_config).connections.release()
     assert violations
     # NB: Replace slashes to deal with different platform paths being returned.
-    assert violations[0].desc().replace("\\", "/").startswith(exception_msg)
+    assert exception_msg in violations[0].desc().replace("\\", "/")
 
 
-@pytest.mark.skipif(
-    DBT_VERSION_TUPLE < (1, 0), reason="mocks a function that's only used in dbt >= 1.0"
-)
 @mock.patch("dbt.adapters.postgres.impl.PostgresAdapter.set_relations_cache")
 def test__templater_dbt_handle_database_connection_failure(
     set_relations_cache, project_dir, dbt_templater  # noqa: F811
@@ -394,6 +441,15 @@ def test__templater_dbt_handle_database_connection_failure(
     """Test the result of a failed database connection."""
     from dbt.adapters.factory import get_adapter
 
+    try:
+        from dbt.exceptions import (
+            FailedToConnectException as DbtFailedToConnectException,
+        )
+    except ImportError:
+        from dbt.exceptions import (
+            FailedToConnectError as DbtFailedToConnectException,
+        )
+
     # Clear the adapter cache to force this test to create a new connection.
     DbtTemplater.adapters.clear()
 
@@ -408,7 +464,7 @@ def test__templater_dbt_handle_database_connection_failure(
             project_dir, "models/my_new_project/exception_connect_database.sql"
         )
     )
-    dbt_fluff_config_fail = DBT_FLUFF_CONFIG.copy()
+    dbt_fluff_config_fail = deepcopy(DBT_FLUFF_CONFIG)
     dbt_fluff_config_fail["templater"]["dbt"][
         "profiles_dir"
     ] = "plugins/sqlfluff-templater-dbt/test/fixtures/dbt/profiles_yml_fail"
@@ -434,7 +490,7 @@ def test__templater_dbt_handle_database_connection_failure(
     )
 
 
-def test__project_dir_does_not_exist_error(dbt_templater, caplog):  # noqa: F811
+def test__project_dir_does_not_exist_error(dbt_templater):  # noqa: F811
     """Test an error is logged if the given dbt project directory doesn't exist."""
     dbt_templater.sqlfluff_config = FluffConfig(
         configs={
@@ -442,18 +498,11 @@ def test__project_dir_does_not_exist_error(dbt_templater, caplog):  # noqa: F811
             "templater": {"dbt": {"project_dir": "./non_existing_directory"}},
         }
     )
-    logger = logging.getLogger("sqlfluff")
-    original_propagate_value = logger.propagate
-    try:
-        logger.propagate = True
-        with caplog.at_level(logging.ERROR, logger="sqlfluff.templater"):
-            dbt_project_dir = dbt_templater._get_project_dir()
-        assert (
-            f"dbt_project_dir: {dbt_project_dir} could not be accessed. "
-            "Check it exists."
-        ) in caplog.text
-    finally:
-        logger.propagate = original_propagate_value
+    with fluff_log_catcher(logging.ERROR, "sqlfluff.templater") as caplog:
+        dbt_project_dir = dbt_templater._get_project_dir()
+    assert (
+        f"dbt_project_dir: {dbt_project_dir} could not be accessed. " "Check it exists."
+    ) in caplog.text
 
 
 @pytest.mark.parametrize(
@@ -482,3 +531,40 @@ def test__context_in_config_is_loaded(
 
     assert violations == []
     assert str(var_value) in processed.templated_str
+
+
+def test__dbt_log_supression():
+    """Test that when we try and parse in JSON format we get JSON.
+
+    This actually tests that we can successfully suppress unwanted
+    logging from dbt.
+    """
+    oldcwd = os.getcwd()
+    try:
+        os.chdir("plugins/sqlfluff-templater-dbt/test/fixtures/dbt")
+        result = invoke_assert_code(
+            ret_code=1,
+            args=[
+                lint,
+                [
+                    "--disable-progress-bar",
+                    "dbt_project/models/my_new_project/operator_errors.sql",
+                    "-f",
+                    "json",
+                ],
+            ],
+        )
+    finally:
+        os.chdir(oldcwd)
+    # Check that the full output parses as json
+    parsed = json.loads(result.output)
+    assert isinstance(parsed, list)
+    assert len(parsed) == 1
+    first_file = parsed[0]
+    assert isinstance(first_file, dict)
+    # NOTE: Path translation for linux/windows.
+    assert (
+        first_file["filepath"].replace("\\", "/")
+        == "dbt_project/models/my_new_project/operator_errors.sql"
+    )
+    assert len(first_file["violations"]) == 2
