@@ -1,52 +1,47 @@
 """Contains the CLI."""
 
-from itertools import chain
-import os
-import sys
 import json
 import logging
+import os
+import sys
 import time
+from itertools import chain
 from logging import LogRecord
-from typing import Callable, Tuple, Optional, cast
-
-import yaml
+from typing import Callable, Optional, Tuple
 
 import click
 
-# For the profiler
-import pstats
-from io import StringIO
-
 # To enable colour cross platform
 import colorama
+import yaml
 from tqdm import tqdm
-from sqlfluff.cli.autocomplete import shell_completion_enabled, dialect_shell_complete
 
-from sqlfluff.cli import EXIT_SUCCESS, EXIT_ERROR, EXIT_FAIL
+from sqlfluff.cli import EXIT_ERROR, EXIT_FAIL, EXIT_SUCCESS
+from sqlfluff.cli.autocomplete import dialect_shell_complete, shell_completion_enabled
 from sqlfluff.cli.click_deprecated_option import (
     DeprecatedOption,
     DeprecatedOptionsCommand,
 )
 from sqlfluff.cli.formatters import (
-    format_linting_result_header,
     OutputStreamFormatter,
+    format_linting_result_header,
 )
-from sqlfluff.cli.helpers import get_package_version
-from sqlfluff.cli.outputstream import make_output_stream, OutputStream
+from sqlfluff.cli.helpers import LazySequence, get_package_version
+from sqlfluff.cli.outputstream import OutputStream, make_output_stream
 
 # Import from sqlfluff core.
 from sqlfluff.core import (
-    Linter,
     FluffConfig,
+    Linter,
+    SQLFluffUserError,
     SQLLintError,
     SQLTemplaterError,
-    SQLFluffUserError,
-    dialect_selector,
     dialect_readout,
+    dialect_selector,
 )
 from sqlfluff.core.config import progress_bar_configuration
-
-from sqlfluff.core.enums import FormatType, Color
+from sqlfluff.core.enums import Color, FormatType
+from sqlfluff.core.linter import LintingResult
 from sqlfluff.core.plugin.host import get_plugin_manager
 
 
@@ -137,13 +132,13 @@ def set_logging_level(
 class PathAndUserErrorHandler:
     """Make an API call but with error handling for the CLI."""
 
-    def __init__(self, formatter):
+    def __init__(self, formatter) -> None:
         self.formatter = formatter
 
-    def __enter__(self):
+    def __enter__(self) -> "PathAndUserErrorHandler":
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
         if exc_type is SQLFluffUserError:
             click.echo(
                 "\nUser Error: "
@@ -212,12 +207,15 @@ def core_options(f: Callable) -> Callable:
         default=None,
         help="The templater to use (default=jinja)",
         type=click.Choice(
-            [
-                templater.name
-                for templater in chain.from_iterable(
-                    get_plugin_manager().hook.get_templaters()
-                )
-            ]
+            # Use LazySequence so that we don't load templaters until required.
+            LazySequence(
+                lambda: [
+                    templater.name
+                    for templater in chain.from_iterable(
+                        get_plugin_manager().hook.get_templaters()
+                    )
+                ]
+            )
         ),
     )(f)
     f = click.option(
@@ -226,10 +224,10 @@ def core_options(f: Callable) -> Callable:
         default=None,
         help=(
             "Narrow the search to only specific rules. For example "
-            "specifying `--rules L001` will only search for rule `L001` (Unnecessary "
+            "specifying `--rules LT01` will only search for rule `LT01` (Unnecessary "
             "trailing whitespace). Multiple rules can be specified with commas e.g. "
-            "`--rules L001,L002` will specify only looking for violations of rule "
-            "`L001` and rule `L002`."
+            "`--rules LT01,LT02` will specify only looking for violations of rule "
+            "`LT01` and rule `LT02`."
         ),
     )(f)
     f = click.option(
@@ -238,12 +236,12 @@ def core_options(f: Callable) -> Callable:
         default=None,
         help=(
             "Exclude specific rules. For example "
-            "specifying `--exclude-rules L001` will remove rule `L001` (Unnecessary "
+            "specifying `--exclude-rules LT01` will remove rule `LT01` (Unnecessary "
             "trailing whitespace) from the set of considered rules. This could either "
             "be the allowlist, or the general set if there is no specific allowlist. "
             "Multiple rules can be specified with commas e.g. "
-            "`--exclude-rules L001,L002` will exclude violations of rule "
-            "`L001` and rule `L002`."
+            "`--exclude-rules LT01,LT02` will exclude violations of rule "
+            "`LT01` and rule `LT02`."
         ),
     )(f)
     f = click.option(
@@ -308,6 +306,59 @@ def core_options(f: Callable) -> Callable:
         default=None,
         help="Set this flag to ignore inline noqa comments.",
     )(f)
+    f = click.option(
+        "--library-path",
+        default=None,
+        help=(
+            "Override the `library_path` value from the [sqlfluff:templater:jinja]"
+            " configuration value. Set this to 'none' to disable entirely."
+            " This overrides any values set by users in configuration files or"
+            " inline directives."
+        ),
+    )(f)
+    return f
+
+
+def lint_options(f: Callable) -> Callable:
+    """Add lint operation options to commands via a decorator.
+
+    These are cli commands that do linting, i.e. `lint` and `fix`.
+    """
+    f = click.option(
+        "-p",
+        "--processes",
+        type=int,
+        default=None,
+        help=(
+            "The number of parallel processes to run. Positive numbers work as "
+            "expected. Zero and negative numbers will work as number_of_cpus - "
+            "number. e.g  -1 means all cpus except one. 0 means all cpus."
+        ),
+    )(f)
+    f = click.option(
+        "--disable_progress_bar",
+        "--disable-progress-bar",
+        is_flag=True,
+        help="Disables progress bars.",
+        cls=DeprecatedOption,
+        deprecated=["--disable_progress_bar"],
+    )(f)
+    f = click.option(
+        "--persist-timing",
+        default=None,
+        help=(
+            "A filename to persist the timing information for a linting run to "
+            "in csv format for external analysis. NOTE: This feature should be "
+            "treated as beta, and the format of the csv file may change in "
+            "future releases without warning."
+        ),
+    )(f)
+    f = click.option(
+        "--warn-unused-ignores",
+        is_flag=True,
+        default=False,
+        help="Warn about unneeded '-- noqa:' comments.",
+    )(f)
     return f
 
 
@@ -341,11 +392,26 @@ def get_config(
                 )
             )
             sys.exit(EXIT_ERROR)
+
     from_root_kwargs = {}
     if "require_dialect" in kwargs:
         from_root_kwargs["require_dialect"] = kwargs.pop("require_dialect")
+
+    library_path = kwargs.pop("library_path", None)
+
+    if not kwargs.get("warn_unused_ignores", True):
+        # If it's present AND True, then keep it, otherwise remove this so
+        # that we default to the root config.
+        del kwargs["warn_unused_ignores"]
+
     # Instantiate a config object (filtering out the nulls)
     overrides = {k: kwargs[k] for k in kwargs if kwargs[k] is not None}
+    if library_path is not None:
+        # Check for a null value
+        if library_path.lower() == "none":
+            library_path = None  # Set an explicit None value.
+        # Set the global override
+        overrides["library_path"] = library_path
     try:
         return FluffConfig.from_root(
             extra_config_path=extra_config_path,
@@ -390,13 +456,13 @@ def get_linter_and_formatter(
     context_settings={"help_option_names": ["-h", "--help"]},
     epilog="""\b\bExamples:\n
   sqlfluff lint --dialect postgres .\n
-  sqlfluff lint --dialect postgres --rules L042 .\n
-  sqlfluff fix --dialect sqlite --rules L041,L042 src/queries\n
+  sqlfluff lint --dialect postgres --rules ST05 .\n
+  sqlfluff fix --dialect sqlite --rules LT10,ST05 src/queries\n
   sqlfluff parse --dialect sqlite --templater jinja src/queries/common.sql
 """,
 )
 @click.version_option()
-def cli():
+def cli() -> None:
     """SQLFluff is a modular SQL linter for humans."""  # noqa D403
 
 
@@ -421,7 +487,20 @@ def rules(**kwargs) -> None:
     """Show the current rules in use."""
     c = get_config(**kwargs, dialect="ansi")
     lnt, formatter = get_linter_and_formatter(c)
-    click.echo(formatter.format_rules(lnt), color=c.get("color"))
+    try:
+        click.echo(formatter.format_rules(lnt), color=c.get("color"))
+    # No cover for clause covering poorly formatted rules.
+    # Without creating a poorly formed plugin, these are hard to
+    # test.
+    except (SQLFluffUserError, AssertionError) as err:  # pragma: no cover
+        click.echo(
+            OutputStreamFormatter.colorize_helper(
+                c.get("color"),
+                f"Error loading rules: {str(err)}",
+                color=Color.red,
+            )
+        )
+        sys.exit(EXIT_ERROR)
 
 
 @cli.command()
@@ -433,7 +512,7 @@ def dialects(**kwargs) -> None:
     click.echo(formatter.format_dialects(dialect_readout), color=c.get("color"))
 
 
-def dump_file_payload(filename: Optional[str], payload: str):
+def dump_file_payload(filename: Optional[str], payload: str) -> None:
     """Write the output file content to stdout or file."""
     # If there's a file specified to write to, write to it.
     if filename:
@@ -447,6 +526,7 @@ def dump_file_payload(filename: Optional[str], payload: str):
 @cli.command(cls=DeprecatedOptionsCommand)
 @common_options
 @core_options
+@lint_options
 @click.option(
     "-f",
     "--format",
@@ -485,35 +565,6 @@ def dump_file_payload(filename: Optional[str], payload: str):
     is_flag=True,
     help="Perform the operation regardless of .sqlfluffignore configurations",
 )
-@click.option(
-    "-p",
-    "--processes",
-    type=int,
-    default=None,
-    help=(
-        "The number of parallel processes to run. Positive numbers work as "
-        "expected. Zero and negative numbers will work as number_of_cpus - "
-        "number. e.g  -1 means all cpus except one. 0 means all cpus."
-    ),
-)
-@click.option(
-    "--disable_progress_bar",
-    "--disable-progress-bar",
-    is_flag=True,
-    help="Disables progress bars.",
-    cls=DeprecatedOption,
-    deprecated=["--disable_progress_bar"],
-)
-@click.option(
-    "--persist-timing",
-    default=None,
-    help=(
-        "A filename to persist the timing information for a linting run to "
-        "in csv format for external analysis. NOTE: This feature should be "
-        "treated as beta, and the format of the csv file may change in "
-        "future releases without warning."
-    ),
-)
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def lint(
     paths: Tuple[str],
@@ -526,9 +577,9 @@ def lint(
     bench: bool = False,
     processes: Optional[int] = None,
     disable_progress_bar: Optional[bool] = False,
+    persist_timing: Optional[str] = None,
     extra_config_path: Optional[str] = None,
     ignore_local_config: bool = False,
-    persist_timing: Optional[str] = None,
     **kwargs,
 ) -> None:
     """Lint SQL files via passing a list of files or using stdin.
@@ -594,6 +645,8 @@ def lint(
         file_output = json.dumps(result.as_records())
     elif format == FormatType.yaml.value:
         file_output = yaml.dump(result.as_records(), sort_keys=False)
+    elif format == FormatType.none.value:
+        file_output = ""
     elif format == FormatType.github_annotation.value:
         if annotation_level == "error":
             annotation_level = "failure"
@@ -635,13 +688,15 @@ def lint(
                 line += f"col={violation['line_pos']}"
                 line += "::"
                 line += f"{violation['code']}: {violation['description']}"
+                if violation["name"]:
+                    line += f" [{violation['name']}]"
 
                 github_result_native.append(line)
 
         file_output = "\n".join(github_result_native)
 
     if file_output:
-        dump_file_payload(write_output, cast(str, file_output))
+        dump_file_payload(write_output, file_output)
 
     if persist_timing:
         result.persist_timing_records(persist_timing)
@@ -653,22 +708,32 @@ def lint(
         timing_summary = result.timing_summary()
         for step in timing_summary:
             click.echo(f"=== {step} ===")
-            click.echo(formatter.cli_table(timing_summary[step].items()))
+            click.echo(
+                formatter.cli_table(timing_summary[step].items(), cols=3, col_width=20)
+            )
 
     if not nofail:
         if not non_human_output:
             formatter.completion_message()
-        sys.exit(result.stats()["exit code"])
+        sys.exit(result.stats(EXIT_FAIL, EXIT_SUCCESS)["exit code"])
     else:
         sys.exit(EXIT_SUCCESS)
 
 
-def do_fixes(lnt, result, formatter=None, **kwargs):
+def do_fixes(
+    result: LintingResult,
+    formatter: Optional[OutputStreamFormatter] = None,
+    fixed_file_suffix: str = "",
+) -> bool:
     """Actually do the fixes."""
-    click.echo("Persisting Changes...")
-    res = result.persist_changes(formatter=formatter, **kwargs)
+    if formatter and formatter.verbosity >= 0:
+        click.echo("Persisting Changes...")
+    res = result.persist_changes(
+        formatter=formatter, fixed_file_suffix=fixed_file_suffix
+    )
     if all(res.values()):
-        click.echo("Done. Please check your files to confirm.")
+        if formatter and formatter.verbosity >= 0:
+            click.echo("Done. Please check your files to confirm.")
         return True
     # If some failed then return false
     click.echo(
@@ -680,16 +745,190 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
     return False  # pragma: no cover
 
 
+def _stdin_fix(linter: Linter, formatter, fix_even_unparsable: bool) -> None:
+    """Handle fixing from stdin."""
+    exit_code = EXIT_SUCCESS
+    stdin = sys.stdin.read()
+
+    result = linter.lint_string_wrapped(stdin, fname="stdin", fix=True)
+    templater_error = result.num_violations(types=SQLTemplaterError) > 0
+    unfixable_error = result.num_violations(types=SQLLintError, fixable=False) > 0
+    if not fix_even_unparsable:
+        exit_code = formatter.handle_files_with_tmp_or_prs_errors(result)
+
+    if result.num_violations(types=SQLLintError, fixable=True) > 0:
+        stdout = result.paths[0].files[0].fix_string()[0]
+    else:
+        stdout = stdin
+
+    if templater_error:
+        click.echo(
+            formatter.colorize(
+                "Fix aborted due to unparsable template variables.",
+                Color.red,
+            ),
+            err=True,
+        )
+        click.echo(
+            formatter.colorize(
+                "Use --FIX-EVEN-UNPARSABLE' to attempt to fix the SQL anyway.",
+                Color.red,
+            ),
+            err=True,
+        )
+
+    if unfixable_error:
+        click.echo(
+            formatter.colorize("Unfixable violations detected.", Color.red),
+            err=True,
+        )
+
+    click.echo(stdout, nl=False)
+    sys.exit(EXIT_FAIL if templater_error or unfixable_error else exit_code)
+
+
+def _paths_fix(
+    linter: Linter,
+    formatter,
+    paths,
+    processes,
+    fix_even_unparsable,
+    force,
+    fixed_suffix,
+    bench,
+    show_lint_violations,
+    warn_force: bool = True,
+    persist_timing: Optional[str] = None,
+) -> None:
+    """Handle fixing from paths."""
+    # Lint the paths (not with the fix argument at this stage), outputting as we go.
+    if formatter.verbosity >= 0:
+        click.echo("==== finding fixable violations ====")
+    exit_code = EXIT_SUCCESS
+
+    if force and warn_force and formatter.verbosity >= 0:
+        click.echo(
+            f"{formatter.colorize('FORCE MODE', Color.red)}: " "Attempting fixes..."
+        )
+
+    with PathAndUserErrorHandler(formatter):
+        result: LintingResult = linter.lint_paths(
+            paths,
+            fix=True,
+            ignore_non_existent_files=False,
+            processes=processes,
+            # If --force is set, then apply the changes as we go rather
+            # than waiting until the end.
+            apply_fixes=force,
+            fixed_file_suffix=fixed_suffix,
+            fix_even_unparsable=fix_even_unparsable,
+        )
+
+    if not fix_even_unparsable:
+        exit_code = formatter.handle_files_with_tmp_or_prs_errors(result)
+
+    # NB: We filter to linting violations here, because they're
+    # the only ones which can be potentially fixed.
+    num_fixable = result.num_violations(types=SQLLintError, fixable=True)
+
+    if num_fixable > 0:
+        if not force and formatter.verbosity >= 0:
+            click.echo("==== fixing violations ====")
+
+        click.echo(f"{num_fixable} " "fixable linting violations found")
+
+        if not force:
+            click.echo(
+                "Are you sure you wish to attempt to fix these? [Y/n] ", nl=False
+            )
+            c = click.getchar().lower()
+            click.echo("...")
+            if c in ("y", "\r", "\n"):
+                if formatter.verbosity >= 0:
+                    click.echo("Attempting fixes...")
+                success = do_fixes(
+                    result,
+                    formatter,
+                    fixed_file_suffix=fixed_suffix,
+                )
+                if not success:
+                    sys.exit(EXIT_FAIL)  # pragma: no cover
+                else:
+                    formatter.completion_message()
+            elif c == "n":
+                click.echo("Aborting...")
+                exit_code = EXIT_FAIL
+            else:  # pragma: no cover
+                click.echo("Invalid input, please enter 'Y' or 'N'")
+                click.echo("Aborting...")
+                exit_code = EXIT_FAIL
+    else:
+        if formatter.verbosity >= 0:
+            click.echo("==== no fixable linting violations found ====")
+            formatter.completion_message()
+
+    error_types = [
+        (
+            dict(types=SQLLintError, fixable=False),
+            "  [{} unfixable linting violations found]",
+            EXIT_FAIL,
+        ),
+    ]
+    for num_violations_kwargs, message_format, error_level in error_types:
+        num_violations = result.num_violations(**num_violations_kwargs)
+        if num_violations > 0 and formatter.verbosity >= 0:
+            click.echo(message_format.format(num_violations))
+            exit_code = max(exit_code, error_level)
+
+    if bench:
+        click.echo("==== overall timings ====")
+        click.echo(formatter.cli_table([("Clock time", result.total_time)]))
+        timing_summary = result.timing_summary()
+        for step in timing_summary:
+            click.echo(f"=== {step} ===")
+            click.echo(
+                formatter.cli_table(timing_summary[step].items(), cols=3, col_width=20)
+            )
+
+    if show_lint_violations:
+        click.echo("==== lint for unfixable violations ====")
+        all_results = result.violation_dict(**num_violations_kwargs)
+        sorted_files = sorted(all_results.keys())
+        for file in sorted_files:
+            violations = all_results.get(file, [])
+            click.echo(formatter.format_filename(file, success=(not violations)))
+            for violation in violations:
+                click.echo(formatter.format_violation(violation))
+
+    if persist_timing:
+        result.persist_timing_records(persist_timing)
+
+    sys.exit(exit_code)
+
+
 @cli.command(cls=DeprecatedOptionsCommand)
 @common_options
 @core_options
+@lint_options
 @click.option(
     "-f",
     "--force",
     is_flag=True,
     help=(
-        "skip the confirmation prompt and go straight to applying "
-        "fixes. **Use this with caution.**"
+        "Skip the confirmation prompt and go straight to applying "
+        "fixes. Fixes will also be applied file by file, during the "
+        "linting process, rather than waiting until all files are "
+        "linted before fixing. **Use this with caution.**"
+    ),
+)
+@click.option(
+    "-q",
+    "--quiet",
+    is_flag=True,
+    help=(
+        "Reduces the amount of output to stdout to a minimal level. "
+        "This is effectively the opposite of -v. NOTE: It will only "
+        "take effect if -f/--force is also set."
     ),
 )
 @click.option(
@@ -698,25 +937,6 @@ def do_fixes(lnt, result, formatter=None, **kwargs):
     default=None,
     help="An optional suffix to add to fixed files.",
 )
-@click.option(
-    "-p",
-    "--processes",
-    type=int,
-    default=None,
-    help=(
-        "The number of parallel processes to run. Positive numbers work as "
-        "expected. Zero and negative numbers will work as number_of_cpus - "
-        "number. e.g  -1 means all cpus except one. 0 means all cpus."
-    ),
-)
-@click.option(
-    "--disable_progress_bar",
-    "--disable-progress-bar",
-    is_flag=True,
-    help="Disables progress bars.",
-    cls=DeprecatedOption,
-    deprecated=["--disable_progress_bar"],
-)
 @click.option(
     "--FIX-EVEN-UNPARSABLE",
     is_flag=True,
@@ -740,10 +960,12 @@ def fix(
     force: bool,
     paths: Tuple[str],
     bench: bool = False,
+    quiet: bool = False,
     fixed_suffix: str = "",
     logger: Optional[logging.Logger] = None,
     processes: Optional[int] = None,
     disable_progress_bar: Optional[bool] = False,
+    persist_timing: Optional[str] = None,
     extra_config_path: Optional[str] = None,
     ignore_local_config: bool = False,
     show_lint_violations: bool = False,
@@ -758,6 +980,13 @@ def fix(
     """
     # some quick checks
     fixing_stdin = ("-",) == paths
+    if quiet:
+        if kwargs["verbose"]:
+            click.echo(
+                "ERROR: The --quiet flag can only be used if --verbose is not set.",
+            )
+            sys.exit(EXIT_ERROR)
+        kwargs["verbose"] = -1
 
     config = get_config(
         extra_config_path, ignore_local_config, require_dialect=False, **kwargs
@@ -771,7 +1000,103 @@ def fix(
     verbose = config.get("verbose")
     progress_bar_configuration.disable_progress_bar = disable_progress_bar
 
-    exit_code = EXIT_SUCCESS
+    formatter.dispatch_config(lnt)
+
+    # Set up logging.
+    set_logging_level(
+        verbosity=verbose,
+        formatter=formatter,
+        logger=logger,
+        stderr_output=fixing_stdin,
+    )
+
+    # handle stdin case. should output formatted sql to stdout and nothing else.
+    if fixing_stdin:
+        _stdin_fix(lnt, formatter, fix_even_unparsable)
+    else:
+        _paths_fix(
+            lnt,
+            formatter,
+            paths,
+            processes,
+            fix_even_unparsable,
+            force,
+            fixed_suffix,
+            bench,
+            show_lint_violations,
+            persist_timing=persist_timing,
+        )
+
+
+@cli.command(name="format", cls=DeprecatedOptionsCommand)
+@common_options
+@core_options
+@lint_options
+@click.option(
+    "-x",
+    "--fixed-suffix",
+    default=None,
+    help="An optional suffix to add to fixed files.",
+)
+@click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
+def cli_format(
+    paths: Tuple[str],
+    bench: bool = False,
+    fixed_suffix: str = "",
+    logger: Optional[logging.Logger] = None,
+    processes: Optional[int] = None,
+    disable_progress_bar: Optional[bool] = False,
+    persist_timing: Optional[str] = None,
+    extra_config_path: Optional[str] = None,
+    ignore_local_config: bool = False,
+    **kwargs,
+) -> None:
+    """Autoformat SQL files.
+
+    This effectively force applies `sqlfluff fix` with a known subset of fairly
+    stable rules. Enabled rules are ignored, but rule exclusions (via CLI) or
+    config are still respected.
+
+    PATH is the path to a sql file or directory to lint. This can be either a
+    file ('path/to/file.sql'), a path ('directory/of/sql/files'), a single ('-')
+    character to indicate reading from *stdin* or a dot/blank ('.'/' ') which will
+    be interpreted like passing the current working directory as a path argument.
+    """
+    # some quick checks
+    fixing_stdin = ("-",) == paths
+
+    if kwargs.get("rules"):
+        click.echo(
+            "Specifying rules is not supported for sqlfluff format.",
+        )
+        sys.exit(EXIT_ERROR)
+
+    # Override rules for sqlfluff format
+    kwargs["rules"] = (
+        # All of the capitalisation rules
+        "capitalisation,"
+        # All of the layout rules
+        "layout,"
+        # Safe rules from other groups
+        "ambiguous.union,"
+        "convention.not_equal,"
+        "convention.coalesce,"
+        "convention.select_trailing_comma,"
+        "convention.is_null,"
+        "jinja.padding,"
+        "structure.distinct,"
+    )
+
+    config = get_config(
+        extra_config_path, ignore_local_config, require_dialect=False, **kwargs
+    )
+    output_stream = make_output_stream(
+        config, None, os.devnull if fixing_stdin else None
+    )
+    lnt, formatter = get_linter_and_formatter(config, output_stream)
+
+    verbose = config.get("verbose")
+    progress_bar_configuration.disable_progress_bar = disable_progress_bar
 
     formatter.dispatch_config(lnt)
 
@@ -785,141 +1110,21 @@ def fix(
 
     # handle stdin case. should output formatted sql to stdout and nothing else.
     if fixing_stdin:
-        stdin = sys.stdin.read()
-
-        result = lnt.lint_string_wrapped(stdin, fname="stdin", fix=True)
-        templater_error = result.num_violations(types=SQLTemplaterError) > 0
-        unfixable_error = result.num_violations(types=SQLLintError, fixable=False) > 0
-        if not fix_even_unparsable:
-            exit_code = formatter.handle_files_with_tmp_or_prs_errors(result)
-
-        if result.num_violations(types=SQLLintError, fixable=True) > 0:
-            stdout = result.paths[0].files[0].fix_string()[0]
-        else:
-            stdout = stdin
-
-        if templater_error:
-            click.echo(
-                formatter.colorize(
-                    "Fix aborted due to unparsable template variables.",
-                    Color.red,
-                ),
-                err=True,
-            )
-            click.echo(
-                formatter.colorize(
-                    "Use --FIX-EVEN-UNPARSABLE' to attempt to fix the SQL anyway.",
-                    Color.red,
-                ),
-                err=True,
-            )
-
-        if unfixable_error:
-            click.echo(
-                formatter.colorize("Unfixable violations detected.", Color.red),
-                err=True,
-            )
-
-        click.echo(stdout, nl=False)
-        sys.exit(EXIT_FAIL if templater_error or unfixable_error else exit_code)
-
-    # Lint the paths (not with the fix argument at this stage), outputting as we go.
-    click.echo("==== finding fixable violations ====")
-
-    with PathAndUserErrorHandler(formatter):
-        result = lnt.lint_paths(
-            paths,
-            fix=True,
-            ignore_non_existent_files=False,
-            processes=processes,
-        )
-
-    if not fix_even_unparsable:
-        exit_code = formatter.handle_files_with_tmp_or_prs_errors(result)
-
-    # NB: We filter to linting violations here, because they're
-    # the only ones which can be potentially fixed.
-    if result.num_violations(types=SQLLintError, fixable=True) > 0:
-        click.echo("==== fixing violations ====")
-        click.echo(
-            f"{result.num_violations(types=SQLLintError, fixable=True)} fixable "
-            "linting violations found"
-        )
-        if force:
-            click.echo(
-                f"{formatter.colorize('FORCE MODE', Color.red)}: Attempting fixes..."
-            )
-            success = do_fixes(
-                lnt,
-                result,
-                formatter,
-                types=SQLLintError,
-                fixed_file_suffix=fixed_suffix,
-            )
-            if not success:
-                sys.exit(EXIT_FAIL)  # pragma: no cover
-        else:
-            click.echo(
-                "Are you sure you wish to attempt to fix these? [Y/n] ", nl=False
-            )
-            c = click.getchar().lower()
-            click.echo("...")
-            if c in ("y", "\r", "\n"):
-                click.echo("Attempting fixes...")
-                success = do_fixes(
-                    lnt,
-                    result,
-                    formatter,
-                    types=SQLLintError,
-                    fixed_file_suffix=fixed_suffix,
-                )
-                if not success:
-                    sys.exit(EXIT_FAIL)  # pragma: no cover
-                else:
-                    formatter.completion_message()
-            elif c == "n":
-                click.echo("Aborting...")
-                exit_code = EXIT_FAIL
-            else:  # pragma: no cover
-                click.echo("Invalid input, please enter 'Y' or 'N'")
-                click.echo("Aborting...")
-                exit_code = EXIT_FAIL
+        _stdin_fix(lnt, formatter, fix_even_unparsable=False)
     else:
-        click.echo("==== no fixable linting violations found ====")
-        formatter.completion_message()
-
-    error_types = [
-        (
-            dict(types=SQLLintError, fixable=False),
-            "  [{} unfixable linting violations found]",
-            EXIT_FAIL,
-        ),
-    ]
-    for num_violations_kwargs, message_format, error_level in error_types:
-        num_violations = result.num_violations(**num_violations_kwargs)
-        if num_violations > 0:
-            click.echo(message_format.format(num_violations))
-            exit_code = max(exit_code, error_level)
-
-    if bench:
-        click.echo("==== overall timings ====")
-        click.echo(formatter.cli_table([("Clock time", result.total_time)]))
-        timing_summary = result.timing_summary()
-        for step in timing_summary:
-            click.echo(f"=== {step} ===")
-            click.echo(formatter.cli_table(timing_summary[step].items()))
-
-    if show_lint_violations:
-        click.echo("==== lint for unfixable violations ====")
-        all_results = result.violation_dict(**num_violations_kwargs)
-        sorted_files = sorted(all_results.keys())
-        for file in sorted_files:
-            violations = all_results.get(file, [])
-            click.echo(formatter.format_filename(file, success=(not violations)))
-            for violation in violations:
-                click.echo(formatter.format_violation(violation))
-
-    sys.exit(exit_code)
+        _paths_fix(
+            lnt,
+            formatter,
+            paths,
+            processes,
+            fix_even_unparsable=False,
+            force=True,  # Always force in format mode.
+            fixed_suffix=fixed_suffix,
+            bench=bench,
+            show_lint_violations=False,
+            warn_force=False,  # don't warn about being in force mode.
+            persist_timing=persist_timing,
+        )
 
 
 def quoted_presenter(dumper, data):
@@ -934,9 +1139,6 @@ def quoted_presenter(dumper, data):
 @common_options
 @core_options
 @click.argument("path", nargs=1, type=click.Path(allow_dash=True))
-@click.option(
-    "--recurse", default=0, help="The depth to recursively parse to (0 for unlimited)"
-)
 @click.option(
     "-c",
     "--code-only",
@@ -961,6 +1163,7 @@ def quoted_presenter(dumper, data):
             FormatType.human.value,
             FormatType.json.value,
             FormatType.yaml.value,
+            FormatType.none.value,
         ],
         case_sensitive=False,
     ),
@@ -975,7 +1178,12 @@ def quoted_presenter(dumper, data):
     ),
 )
 @click.option(
-    "--profiler", is_flag=True, help="Set this flag to engage the python profiler."
+    "--parse-statistics",
+    is_flag=True,
+    help=(
+        "Set this flag to enabled detailed debugging readout "
+        "on the use of terminators in the parser."
+    ),
 )
 @click.option(
     "--nofail",
@@ -991,12 +1199,12 @@ def parse(
     include_meta: bool,
     format: str,
     write_output: Optional[str],
-    profiler: bool,
     bench: bool,
     nofail: bool,
     logger: Optional[logging.Logger] = None,
     extra_config_path: Optional[str] = None,
     ignore_local_config: bool = False,
+    parse_statistics: bool = False,
     **kwargs,
 ) -> None:
     """Parse SQL files and just spit out the result.
@@ -1015,7 +1223,6 @@ def parse(
     output_stream = make_output_stream(c, format, write_output)
     lnt, formatter = get_linter_and_formatter(c, output_stream)
     verbose = c.get("verbose")
-    recurse = c.get("recurse")
 
     progress_bar_configuration.disable_progress_bar = True
 
@@ -1029,18 +1236,6 @@ def parse(
         stderr_output=non_human_output,
     )
 
-    # TODO: do this better
-
-    if profiler:
-        # Set up the profiler if required
-        try:
-            import cProfile
-        except ImportError:  # pragma: no cover
-            click.echo("The cProfiler is not available on your platform.")
-            sys.exit(EXIT_ERROR)
-        pr = cProfile.Profile()
-        pr.enable()
-
     t0 = time.monotonic()
 
     # handle stdin if specified via lone '-'
@@ -1050,8 +1245,8 @@ def parse(
                 lnt.parse_string(
                     sys.stdin.read(),
                     "stdin",
-                    recurse=recurse,
                     config=lnt.config,
+                    parse_statistics=parse_statistics,
                 ),
             ]
         else:
@@ -1059,7 +1254,7 @@ def parse(
             parsed_strings = list(
                 lnt.parse_path(
                     path=path,
-                    recurse=recurse,
+                    parse_statistics=parse_statistics,
                 )
             )
 
@@ -1091,17 +1286,11 @@ def parse(
             file_output = yaml.dump(parsed_strings_dict, sort_keys=False)
         elif format == FormatType.json.value:
             file_output = json.dumps(parsed_strings_dict)
+        elif format == FormatType.none.value:
+            file_output = ""
 
         # Dump the output to stdout or to file as appropriate.
         dump_file_payload(write_output, file_output)
-    if profiler:
-        pr.disable()
-        profiler_buffer = StringIO()
-        ps = pstats.Stats(pr, stream=profiler_buffer).sort_stats("cumulative")
-        ps.print_stats()
-        click.echo("==== profiler stats ====")
-        # Only print the first 50 lines of it
-        click.echo("\n".join(profiler_buffer.getvalue().split("\n")[:50]))
 
     if violations_count > 0 and not nofail:
         sys.exit(EXIT_FAIL)  # pragma: no cover
@@ -1159,7 +1348,7 @@ def render(
             fname = path
 
     # Get file specific config
-    file_config.process_raw_file_for_config(raw_sql)
+    file_config.process_raw_file_for_config(raw_sql, fname)
     rendered = lnt.render_string(raw_sql, fname, file_config, "utf8")
 
     if rendered.templater_violations:
