@@ -24,8 +24,11 @@ _dialect_lookup = {
     "athena": ("dialect_athena", "athena_dialect"),
     "bigquery": ("dialect_bigquery", "bigquery_dialect"),
     "clickhouse": ("dialect_clickhouse", "clickhouse_dialect"),
+    "databricks": ("dialect_databricks", "databricks_dialect"),
     "db2": ("dialect_db2", "db2_dialect"),
+    "duckdb": ("dialect_duckdb", "duckdb_dialect"),
     "exasol": ("dialect_exasol", "exasol_dialect"),
+    "greenplum": ("dialect_greenplum", "greenplum_dialect"),
     "hive": ("dialect_hive", "hive_dialect"),
     "materialize": ("dialect_materialize", "materialize_dialect"),
     "mysql": ("dialect_mysql", "mysql_dialect"),
@@ -34,11 +37,11 @@ _dialect_lookup = {
     "redshift": ("dialect_redshift", "redshift_dialect"),
     "snowflake": ("dialect_snowflake", "snowflake_dialect"),
     "soql": ("dialect_soql", "soql_dialect"),
+    "sparksql": ("dialect_sparksql", "sparksql_dialect"),
     "sqlite": ("dialect_sqlite", "sqlite_dialect"),
     "teradata": ("dialect_teradata", "teradata_dialect"),
+    "trino": ("dialect_trino", "trino_dialect"),
     "tsql": ("dialect_tsql", "tsql_dialect"),
-    "sparksql": ("dialect_sparksql", "sparksql_dialect"),
-    "databricks": ("dialect_sparksql", "sparksql_dialect"),
 }
 
 _legacy_dialects = {
@@ -58,6 +61,8 @@ def load_raw_dialect(label: str, base_module: str = "sqlfluff.dialects") -> Dial
     """Dynamically load a dialect."""
     if label in _legacy_dialects:
         raise SQLFluffUserError(_legacy_dialects[label])
+    elif label not in _dialect_lookup:
+        raise KeyError("Unknown dialect")
     module_name, name = _dialect_lookup[label]
     module = import_module(f"{base_module}.{module_name}")
     result: Dialect = getattr(module, name)
