@@ -1,19 +1,18 @@
 """Defines the base dialect class."""
 
-from typing import Set, Union, Type
+import sys
+from typing import Any, Dict, List, Optional, Set, Type, Union, cast
 
 from sqlfluff.core.parser import (
+    BaseSegment,
     KeywordSegment,
     SegmentGenerator,
-    BaseSegment,
     StringParser,
 )
-from sqlfluff.core.parser.grammar.base import BaseGrammar
+from sqlfluff.core.parser.grammar.base import BaseGrammar, Nothing
+from sqlfluff.core.parser.lexer import LexerType
 from sqlfluff.core.parser.matchable import Matchable
-
-DialectElementType = Union[Type[BaseSegment], Matchable, SegmentGenerator]
-# NOTE: Post expansion, no generators remain
-ExpandedDialectElementType = Union[Type[BaseSegment], Matchable]
+from sqlfluff.core.parser.types import BracketPairTuple, DialectElementType
 
 
 class Dialect:
@@ -28,13 +27,13 @@ class Dialect:
 
     def __init__(
         self,
-        name,
-        lexer_matchers=None,
-        library=None,
-        sets=None,
-        inherits_from=None,
-        root_segment_name=None,
-    ):
+        name: str,
+        root_segment_name: str,
+        lexer_matchers: Optional[List[LexerType]] = None,
+        library: Optional[Dict[str, DialectElementType]] = None,
+        sets: Optional[Dict[str, Set[Union[str, BracketPairTuple]]]] = None,
+        inherits_from: Optional[str] = None,
+    ) -> None:
         self._library = library or {}
         self.name = name
         self.lexer_matchers = lexer_matchers
@@ -43,7 +42,7 @@ class Dialect:
         self.inherits_from = inherits_from
         self.root_segment_name = root_segment_name
 
-    def __repr__(self):  # pragma: no cover
+    def __repr__(self) -> str:  # pragma: no cover
         return f"<Dialect: {self.name}>"
 
     def expand(self) -> "Dialect":
@@ -68,27 +67,27 @@ class Dialect:
         expanded_copy = self.copy_as(name=self.name)
         # Expand any callable elements of the dialect.
         for key in expanded_copy._library:
-            if isinstance(expanded_copy._library[key], SegmentGenerator):
+            seg_gen = expanded_copy._library[key]
+            if isinstance(seg_gen, SegmentGenerator):
                 # If the element is callable, call it passing the current
                 # dialect and store the result in its place.
                 # Use the .replace() method for its error handling.
-                expanded_copy.replace(
-                    **{key: expanded_copy._library[key].expand(expanded_copy)}
-                )
+                expanded_copy.replace(**{key: seg_gen.expand(expanded_copy)})
         # Expand any keyword sets.
         for keyword_set in [
             "unreserved_keywords",
             "reserved_keywords",
         ]:  # e.g. reserved_keywords, (JOIN, ...)
             # Make sure the values are available as KeywordSegments
-            for kw in expanded_copy.sets(keyword_set):
+            keyword_sets = expanded_copy.sets(keyword_set)
+            for kw in keyword_sets:
                 n = kw.capitalize() + "KeywordSegment"
                 if n not in expanded_copy._library:
                     expanded_copy._library[n] = StringParser(kw.lower(), KeywordSegment)
         expanded_copy.expanded = True
         return expanded_copy
 
-    def sets(self, label) -> Set:
+    def sets(self, label: str) -> Set[str]:
         """Allows access to sets belonging to this dialect.
 
         These sets belong to the dialect and are copied for sub
@@ -96,11 +95,35 @@ class Dialect:
         dialect objects to create some of the bulk-produced rules.
 
         """
+        assert label not in (
+            "bracket_pairs",
+            "angle_bracket_pairs",
+        ), f"Use `bracket_sets` to retrieve {label} set."
+
         if label not in self._sets:
             self._sets[label] = set()
-        return self._sets[label]
+        return cast(Set[str], self._sets[label])
 
-    def copy_as(self, name):
+    def bracket_sets(self, label: str) -> Set[BracketPairTuple]:
+        """Allows access to bracket sets belonging to this dialect."""
+        assert label in (
+            "bracket_pairs",
+            "angle_bracket_pairs",
+        ), "Invalid bracket set. Consider using `sets` instead."
+
+        if label not in self._sets:
+            self._sets[label] = set()
+        return cast(Set[BracketPairTuple], self._sets[label])
+
+    def update_keywords_set_from_multiline_string(
+        self, set_label: str, values: str
+    ) -> None:
+        """Special function to update a keywords set from a multi-line string."""
+        self.sets(set_label).update(
+            [n.strip().upper() for n in values.strip().split("\n")]
+        )
+
+    def copy_as(self, name: str) -> "Dialect":
         """Copy this dialect and create a new one with a different name.
 
         This is the primary method for inheritance, after which, the
@@ -117,6 +140,8 @@ class Dialect:
         for label in self._sets:
             new_sets[label] = self._sets[label].copy()
 
+        assert self.lexer_matchers
+
         return self.__class__(
             name=name,
             library=self._library.copy(),
@@ -126,7 +151,7 @@ class Dialect:
             root_segment_name=self.root_segment_name,
         )
 
-    def add(self, **kwargs: DialectElementType):
+    def add(self, **kwargs: DialectElementType) -> None:
         """Add a segment to the dialect directly.
 
         This is the alternative to the decorator route, most useful for segments
@@ -143,7 +168,7 @@ class Dialect:
                 raise ValueError(f"{n!r} is already registered in {self!r}")
             self._library[n] = kwargs[n]
 
-    def replace(self, **kwargs: DialectElementType):
+    def replace(self, **kwargs: DialectElementType) -> None:
         """Override a segment on the dialect directly.
 
         Usage is very similar to add, but elements specified must already exist.
@@ -151,12 +176,13 @@ class Dialect:
         for n in kwargs:
             if n not in self._library:  # pragma: no cover
                 raise ValueError(f"{n!r} is not already registered in {self!r}")
-            cls = kwargs[n]
-            if self._library[n] is cls:
+            replacement = kwargs[n]
+            # If trying to replace with same, just skip.
+            if self._library[n] is replacement:
                 continue
-            elif self._library[n] == cls:
-                # Check for replacement with a new but identical class.
-                # This would be a sign of redundant definitions in the dialect.
+            # Check for replacement with a new but identical class.
+            # This would be a sign of redundant definitions in the dialect.
+            elif self._library[n] == replacement:
                 raise ValueError(
                     f"Attempted unnecessary identical redefinition of {n!r} in {self!r}"
                 )  # pragma: no cover
@@ -164,18 +190,31 @@ class Dialect:
             # To replace a segment, the replacement must either be a
             # subclass of the original, *or* it must have the same
             # public methods and/or fields as it.
-            base_dir = set(dir(self._library[n]))
+            # NOTE: Other replacements aren't validated.
             subclass = False
-            if isinstance(self._library[n], type) and isinstance(cls, type):
-                subclass = issubclass(cls, self._library[n])
+            if isinstance(self._library[n], type) and not isinstance(
+                # NOTE: The exception here is we _are_ allowed to replace a
+                # segment with a `Nothing()` grammar, which shows that a segment
+                # has been disabled.
+                replacement,
+                Nothing,
+            ):
+                assert isinstance(
+                    replacement, type
+                ), f"Cannot replace {n!r} with {replacement}"
+                old_seg = cast(Type["BaseSegment"], self._library[n])
+                new_seg = cast(Type["BaseSegment"], replacement)
+                assert issubclass(old_seg, BaseSegment)
+                assert issubclass(new_seg, BaseSegment)
+                subclass = issubclass(new_seg, old_seg)
                 if not subclass:
-                    if self._library[n].type != cls.type:
+                    if old_seg.type != new_seg.type:
                         raise ValueError(  # pragma: no cover
                             f"Cannot replace {n!r} because 'type' property does not "
-                            f"match: {cls.type} != {self._library[n].type}"
+                            f"match: {new_seg.type} != {old_seg.type}"
                         )
-
-                    cls_dir = set(dir(cls))
+                    base_dir = set(dir(self._library[n]))
+                    cls_dir = set(dir(new_seg))
                     missing = set(
                         n for n in base_dir.difference(cls_dir) if not n.startswith("_")
                     )
@@ -185,39 +224,17 @@ class Dialect:
                             f"is missing these from base: {', '.join(missing)}"
                         )
 
-            if subclass:
-                # If the segment class we're replacing defines these fields, the
-                # replacement must override either:
-                # - NONE of them or
-                # - ALL of them
-                # Overriding a subset of them is not necessarily wrong, but it's
-                # error-prone, hence this policy.
-                grammars = {"match_grammar", "parse_grammar"}
-                # TRICKY: The explicit use of __dict__ on the classes is
-                # deliberate. We are concerned with whether a class itself does
-                # or does not define a thing, IGNORING INHERITED VALUES.
-                if grammars.intersection(set(self._library[n].__dict__)) == grammars:
-                    overrides = grammars.intersection(set(cls.__dict__))
-                    if overrides and overrides != grammars:
-                        for grammar in grammars:
-                            if (
-                                grammar in self._library[n].__dict__
-                                and grammar not in cls.__dict__
-                            ):
-                                raise ValueError(
-                                    f"Cannot replace {n!r} because it needs "
-                                    f"to define '{grammar}'"
-                                )
-            self._library[n] = cls
+            self._library[n] = replacement
 
-    def add_update_segments(self, module_dct):
+    def add_update_segments(self, module_dct: Dict[str, Any]) -> None:
         """Scans module dictionary, adding or replacing segment definitions."""
         for k, v in module_dct.items():
             if isinstance(v, type) and issubclass(v, BaseSegment):
                 if k not in self._library:
                     self.add(**{k: v})
                 else:
-                    self.replace(**{k: v})
+                    non_seg_v = cast(Union[Matchable, SegmentGenerator], v)
+                    self.replace(**{k: non_seg_v})
 
     def get_grammar(self, name: str) -> BaseGrammar:
         """Allow access to grammars pre-expansion.
@@ -227,11 +244,12 @@ class Dialect:
         """
         if name not in self._library:  # pragma: no cover
             raise ValueError(f"Element {name} not found in dialect.")
-        if not isinstance(self._library[name], BaseGrammar):  # pragma: no cover
+        grammar = self._library[name]
+        if not isinstance(grammar, BaseGrammar):  # pragma: no cover
             raise TypeError(
                 f"Attempted to fetch non grammar [{name}] with get_grammar."
             )
-        return self._library[name]
+        return grammar
 
     def get_segment(self, name: str) -> Type["BaseSegment"]:
         """Allow access to segments pre-expansion.
@@ -241,13 +259,17 @@ class Dialect:
         """
         if name not in self._library:  # pragma: no cover
             raise ValueError(f"Element {name} not found in dialect.")
-        if not issubclass(self._library[name], BaseSegment):  # pragma: no cover
-            raise TypeError(
-                f"Attempted to fetch non segment [{name}] with get_segment."
-            )
-        return self._library[name]
+        segment = cast(Type["BaseSegment"], self._library[name])
 
-    def ref(self, name: str) -> ExpandedDialectElementType:
+        if issubclass(segment, BaseSegment):
+            return segment
+        else:  # pragma: no cover
+            raise TypeError(
+                f"Attempted to fetch non segment [{name}] "
+                f"with get_segment - type{type(segment)}"
+            )
+
+    def ref(self, name: str) -> Matchable:
         """Return an object which acts as a late binding reference to the element named.
 
         NB: This requires the dialect to be expanded, and only returns Matchables
@@ -260,6 +282,7 @@ class Dialect:
         if name in self._library:
             res = self._library[name]
             if res:
+                assert not isinstance(res, SegmentGenerator)
                 return res
             else:  # pragma: no cover
                 raise ValueError(
@@ -267,24 +290,38 @@ class Dialect:
                         name, self.name
                     )
                 )
-        else:  # pragma: no cover
-            if name.endswith("KeywordSegment"):
-                keyword_tip = (
-                    " Perhaps specify the keyword? "
-                    "https://github.com/sqlfluff/sqlfluff/wiki/Contributing-Dialect-Changes#keywords"  # noqa E501
+        elif name.endswith("KeywordSegment"):  # pragma: no cover
+            keyword = name[0:-14]
+            keyword_tip = (
+                "\n\nThe syntax in the query is not (yet?) supported. Try to"
+                " narrow down your query to a minimal, reproducible case and"
+                " raise an issue on GitHub.\n\n"
+                "Or, even better, see this guide on how to help contribute"
+                " keyword and/or dialect updates:\n"
+                "https://github.com/sqlfluff/sqlfluff/wiki/Contributing-Dialect-Changes#keywords"  # noqa E501
+            )
+            # Keyword errors are common so avoid printing the whole, scary,
+            # traceback as not that useful and confusing to people.
+            sys.tracebacklimit = 0
+            raise RuntimeError(
+                (
+                    "Grammar refers to the "
+                    "{!r} keyword which was not found in the {} dialect.{}".format(
+                        keyword, self.name, keyword_tip
+                    )
                 )
-            else:
-                keyword_tip = ""
+            )
+        else:  # pragma: no cover
             raise RuntimeError(
                 (
                     "Grammar refers to "
-                    "{!r} which was not found in the {} dialect.{}".format(
-                        name, self.name, keyword_tip
+                    "{!r} which was not found in the {} dialect.".format(
+                        name, self.name
                     )
                 )
             )
 
-    def set_lexer_matchers(self, lexer_matchers):
+    def set_lexer_matchers(self, lexer_matchers: List[LexerType]) -> None:
         """Set the lexer struct for the dialect.
 
         This is what is used for base dialects. For derived dialects
@@ -294,14 +331,14 @@ class Dialect:
         """
         self.lexer_matchers = lexer_matchers
 
-    def get_lexer_matchers(self):
+    def get_lexer_matchers(self) -> List[LexerType]:
         """Fetch the lexer struct for this dialect."""
         if self.lexer_matchers:
             return self.lexer_matchers
         else:  # pragma: no cover
             raise ValueError(f"Lexing struct has not been set for dialect {self}")
 
-    def patch_lexer_matchers(self, lexer_patch):
+    def patch_lexer_matchers(self, lexer_patch: List[LexerType]) -> None:
         """Patch an existing lexer struct.
 
         Used to edit the lexer of a sub-dialect.
@@ -321,7 +358,7 @@ class Dialect:
         # Overwrite with the buffer once we're done
         self.lexer_matchers = buff
 
-    def insert_lexer_matchers(self, lexer_patch, before):
+    def insert_lexer_matchers(self, lexer_patch: List[LexerType], before: str) -> None:
         """Insert new records into an existing lexer struct.
 
         Used to edit the lexer of a sub-dialect. The patch is
@@ -348,6 +385,6 @@ class Dialect:
         # Overwrite with the buffer once we're done
         self.lexer_matchers = buff
 
-    def get_root_segment(self):
+    def get_root_segment(self) -> Union[Type[BaseSegment], Matchable]:
         """Get the root segment of the dialect."""
         return self.ref(self.root_segment_name)
