@@ -4,23 +4,13 @@ This stores the idea of a collection of linted files at a single start path
 
 """
 
-from typing import (
-    Any,
-    Dict,
-    List,
-    Optional,
-    Union,
-    overload,
-)
+from typing import Any, Dict, List, Optional, Union, overload
+
 from typing_extensions import Literal
 
-
-from sqlfluff.core.errors import (
-    CheckTuple,
-)
-from sqlfluff.core.parser.segments.base import BaseSegment
-
+from sqlfluff.core.errors import CheckTuple
 from sqlfluff.core.linter.linted_file import LintedFile
+from sqlfluff.core.parser.segments.base import BaseSegment
 
 
 class LintedDir:
@@ -39,23 +29,16 @@ class LintedDir:
         self.files.append(file)
 
     @overload
-    def check_tuples(
-        self, by_path: Literal[False]
-    ) -> List[CheckTuple]:  # pragma: no cover
+    def check_tuples(self, by_path: Literal[False]) -> List[CheckTuple]:
         """Return a List of CheckTuples when by_path is False."""
-        ...
 
     @overload
-    def check_tuples(
-        self, by_path: Literal[True]
-    ) -> Dict[str, List[CheckTuple]]:  # pragma: no cover
+    def check_tuples(self, by_path: Literal[True]) -> Dict[str, List[CheckTuple]]:
         """Return a Dict of paths and CheckTuples when by_path is True."""
-        ...
 
     @overload
-    def check_tuples(self, by_path: bool = False):  # pragma: no cover
+    def check_tuples(self, by_path: bool = False):
         """Default overload method."""
-        ...
 
     def check_tuples(
         self, by_path=False, raise_on_non_linting_violations=True
@@ -106,7 +89,7 @@ class LintedDir:
         )
 
     def persist_changes(
-        self, formatter: Any = None, fixed_file_suffix: str = "", **kwargs
+        self, formatter: Any = None, fixed_file_suffix: str = ""
     ) -> Dict[str, Union[bool, str]]:
         """Persist changes to files in the given path.
 
@@ -115,15 +98,9 @@ class LintedDir:
         # Run all the fixes for all the files and return a dict
         buffer: Dict[str, Union[bool, str]] = {}
         for file in self.files:
-            if file.num_violations(fixable=True, **kwargs) > 0:
-                buffer[file.path] = file.persist_tree(suffix=fixed_file_suffix)
-                result = buffer[file.path]
-            else:  # pragma: no cover TODO?
-                buffer[file.path] = True
-                result = "SKIP"
-
-            if formatter:
-                formatter.dispatch_persist_filename(filename=file.path, result=result)
+            buffer[file.path] = file.persist_tree(
+                suffix=fixed_file_suffix, formatter=formatter
+            )
         return buffer
 
     @property
@@ -133,4 +110,8 @@ class LintedDir:
             raise ValueError(
                 ".tree() cannot be called when a LintedDir contains more than one file."
             )
+        elif not self.files:  # pragma: no cover
+            raise ValueError(
+                "LintedDir has no parsed files. There is probably a parsing error."
+            )
         return self.files[0].tree
