@@ -1,10 +1,10 @@
 """Defines the linter class."""
 
-import fnmatch
+import logging
 import os
 import time
-import logging
 from typing import (
+    TYPE_CHECKING,
     Any,
     Iterable,
     Iterator,
@@ -21,39 +21,37 @@ import pathspec
 import regex
 from tqdm import tqdm
 
+from sqlfluff.core.config import ConfigLoader, FluffConfig, progress_bar_configuration
 from sqlfluff.core.errors import (
     SQLBaseError,
+    SQLFluffSkipFile,
+    SQLFluffUserError,
     SQLLexError,
     SQLLintError,
     SQLParseError,
-    SQLFluffSkipFile,
-    SQLFluffUserError,
 )
-from sqlfluff.core.parser import Lexer, Parser, RegexLexer
 from sqlfluff.core.file_helpers import get_encoding
-from sqlfluff.core.templaters import TemplatedFile
-from sqlfluff.core.rules import get_ruleset
-from sqlfluff.core.rules.doc_decorators import is_fix_compatible
-from sqlfluff.core.config import FluffConfig, ConfigLoader, progress_bar_configuration
-
-# Classes needed only for type checking
-from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
-from sqlfluff.core.parser.segments.meta import MetaSegment
-from sqlfluff.core.parser.segments.raw import RawSegment
-from sqlfluff.core.rules import BaseRule
-
-from sqlfluff.core.linter.common import (
-    RuleTuple,
-    ParsedString,
-    NoQaDirective,
-    RenderedFile,
-)
-from sqlfluff.core.linter.linted_file import LintedFile
+from sqlfluff.core.linter.common import ParsedString, RenderedFile, RuleTuple
 from sqlfluff.core.linter.linted_dir import LintedDir
+from sqlfluff.core.linter.linted_file import (
+    TMP_PRS_ERROR_TYPES,
+    FileTimings,
+    LintedFile,
+)
 from sqlfluff.core.linter.linting_result import LintingResult
+from sqlfluff.core.parser import Lexer, Parser
+from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
+from sqlfluff.core.rules import BaseRule, RulePack, get_ruleset
+from sqlfluff.core.rules.fix import apply_fixes, compute_anchor_edit_info
+from sqlfluff.core.rules.noqa import IgnoreMask
+
+if TYPE_CHECKING:  # pragma: no cover
+    from sqlfluff.core.parser.segments.meta import MetaSegment
+    from sqlfluff.core.templaters import TemplatedFile
 
 
 WalkableType = Iterable[Tuple[str, Optional[List[str]], List[str]]]
+RuleTimingsType = List[Tuple[str, str, float]]
 
 # Instantiate the linter logger
 linter_logger: logging.Logger = logging.getLogger("sqlfluff.linter")
@@ -93,19 +91,22 @@ class Linter:
         # Store references to user rule classes
         self.user_rules = user_rules or []
 
-    def get_ruleset(self, config: Optional[FluffConfig] = None) -> List[BaseRule]:
+    def get_rulepack(self, config: Optional[FluffConfig] = None) -> RulePack:
         """Get hold of a set of rules."""
         rs = get_ruleset()
         # Register any user rules
         for rule in self.user_rules:
             rs.register(rule)
         cfg = config or self.config
-        return rs.get_rulelist(config=cfg)
+        return rs.get_rulepack(config=cfg)
 
     def rule_tuples(self) -> List[RuleTuple]:
         """A simple pass through to access the rule tuples of the rule set."""
-        rs = self.get_ruleset()
-        return [RuleTuple(rule.code, rule.description) for rule in rs]
+        rs = self.get_rulepack()
+        return [
+            RuleTuple(rule.code, rule.name, rule.description, rule.groups, rule.aliases)
+            for rule in rs.rules
+        ]
 
     # #### Static methods
     # These are the building blocks of the linting process.
@@ -133,7 +134,7 @@ class Linter:
         with open(fname, encoding=encoding, errors="backslashreplace") as target_file:
             raw_file = target_file.read()
         # Scan the raw file for config commands.
-        file_config.process_raw_file_for_config(raw_file)
+        file_config.process_raw_file_for_config(raw_file, fname)
         # Return the raw file and config
         return raw_file, file_config, encoding
 
@@ -144,7 +145,7 @@ class Linter:
 
     @staticmethod
     def _lex_templated_file(
-        templated_file: TemplatedFile, config: FluffConfig
+        templated_file: "TemplatedFile", config: FluffConfig
     ) -> Tuple[Optional[Sequence[BaseSegment]], List[SQLLexError], FluffConfig]:
         """Lex a templated file.
 
@@ -185,7 +186,7 @@ class Linter:
                 getattr(elem, "indent_val", 0)
                 for elem in cast(Tuple[BaseSegment, ...], tokens)
             )
-            if indent_balance != 0:
+            if indent_balance != 0:  # pragma: no cover
                 linter_logger.debug(
                     "Indent balance test failed for %r. Template indents will not be "
                     "linted for this file.",
@@ -199,11 +200,11 @@ class Linter:
         new_tokens = []
         for token in cast(Tuple[BaseSegment, ...], tokens):
             if token.is_meta:
-                token = cast(MetaSegment, token)
+                token = cast("MetaSegment", token)
                 if token.indent_val != 0:
                     # Don't allow it if we're not linting templating block indents.
                     if not templating_blocks_indent:
-                        continue
+                        continue  # pragma: no cover
             new_tokens.append(token)
 
         # Return new buffer
@@ -213,121 +214,53 @@ class Linter:
     def _parse_tokens(
         tokens: Sequence[BaseSegment],
         config: FluffConfig,
-        recurse: bool = True,
         fname: Optional[str] = None,
+        parse_statistics: bool = False,
     ) -> Tuple[Optional[BaseSegment], List[SQLParseError]]:
         parser = Parser(config=config)
         violations = []
         # Parse the file and log any problems
         try:
             parsed: Optional[BaseSegment] = parser.parse(
-                tokens,
-                recurse=recurse,
+                # Regardless of how the sequence was passed in, we should
+                # coerce it to a tuple here, before we head deeper into
+                # the parsing process.
+                tuple(tokens),
                 fname=fname,
+                parse_statistics=parse_statistics,
             )
         except SQLParseError as err:
             linter_logger.info("PARSING FAILED! : %s", err)
             violations.append(err)
             return None, violations
 
-        if parsed:
-            linter_logger.info("\n###\n#\n# {}\n#\n###".format("Parsed Tree:"))
-            linter_logger.info("\n" + parsed.stringify())
-            # We may succeed parsing, but still have unparsable segments. Extract them
-            # here.
-            for unparsable in parsed.iter_unparsables():
-                # No exception has been raised explicitly, but we still create one here
-                # so that we can use the common interface
-                violations.append(
-                    SQLParseError(
-                        "Line {0[0]}, Position {0[1]}: Found unparsable section: "
-                        "{1!r}".format(
-                            unparsable.pos_marker.working_loc,
-                            unparsable.raw
-                            if len(unparsable.raw) < 40
-                            else unparsable.raw[:40] + "...",
-                        ),
-                        segment=unparsable,
-                    )
+        if parsed is None:  # pragma: no cover
+            return None, violations
+
+        linter_logger.info("\n###\n#\n# {}\n#\n###".format("Parsed Tree:"))
+        linter_logger.info("\n" + parsed.stringify())
+        # We may succeed parsing, but still have unparsable segments. Extract them
+        # here.
+        for unparsable in parsed.iter_unparsables():
+            # No exception has been raised explicitly, but we still create one here
+            # so that we can use the common interface
+            assert unparsable.pos_marker
+            violations.append(
+                SQLParseError(
+                    "Line {0[0]}, Position {0[1]}: Found unparsable section: "
+                    "{1!r}".format(
+                        unparsable.pos_marker.working_loc,
+                        unparsable.raw
+                        if len(unparsable.raw) < 40
+                        else unparsable.raw[:40] + "...",
+                    ),
+                    segment=unparsable,
                 )
-                linter_logger.info("Found unparsable segment...")
-                linter_logger.info(unparsable.stringify())
+            )
+            linter_logger.info("Found unparsable segment...")
+            linter_logger.info(unparsable.stringify())
         return parsed, violations
 
-    @staticmethod
-    def parse_noqa(
-        comment: str,
-        line_no: int,
-        rule_codes: List[str],
-    ):
-        """Extract ignore mask entries from a comment string."""
-        # Also trim any whitespace afterward
-
-        # Comment lines can also have noqa e.g.
-        # --dafhsdkfwdiruweksdkjdaffldfsdlfjksd -- noqa: L016
-        # Therefore extract last possible inline ignore.
-        comment = [c.strip() for c in comment.split("--")][-1]
-
-        if comment.startswith("noqa"):
-            # This is an ignore identifier
-            comment_remainder = comment[4:]
-            if comment_remainder:
-                if not comment_remainder.startswith(":"):
-                    return SQLParseError(
-                        "Malformed 'noqa' section. Expected 'noqa: <rule>[,...]",
-                        line_no=line_no,
-                    )
-                comment_remainder = comment_remainder[1:].strip()
-                if comment_remainder:
-                    action: Optional[str]
-                    if "=" in comment_remainder:
-                        action, rule_part = comment_remainder.split("=", 1)
-                        if action not in {"disable", "enable"}:  # pragma: no cover
-                            return SQLParseError(
-                                "Malformed 'noqa' section. "
-                                "Expected 'noqa: enable=<rule>[,...] | all' "
-                                "or 'noqa: disable=<rule>[,...] | all",
-                                line_no=line_no,
-                            )
-                    else:
-                        action = None
-                        rule_part = comment_remainder
-                        if rule_part in {"disable", "enable"}:
-                            return SQLParseError(
-                                "Malformed 'noqa' section. "
-                                "Expected 'noqa: enable=<rule>[,...] | all' "
-                                "or 'noqa: disable=<rule>[,...] | all",
-                                line_no=line_no,
-                            )
-                    rules: Optional[Tuple[str, ...]]
-                    if rule_part != "all":
-                        # Rules can be globs therefore we compare to the rule_set to
-                        # expand the globs.
-                        unexpanded_rules = tuple(
-                            r.strip() for r in rule_part.split(",")
-                        )
-                        expanded_rules = []
-                        for r in unexpanded_rules:
-                            expanded_rule = [
-                                x
-                                for x in fnmatch.filter(rule_codes, r)
-                                if x not in expanded_rules
-                            ]
-                            if expanded_rule:
-                                expanded_rules.extend(expanded_rule)
-                            elif r not in expanded_rules:
-                                # We were unable to expand the glob.
-                                # Therefore assume the user is referencing
-                                # a special error type (e.g. PRS, LXR, or TMP)
-                                # and add this to the list of rules to ignore.
-                                expanded_rules.append(r)
-                        rules = tuple(expanded_rules)
-                    else:
-                        rules = None
-                    return NoQaDirective(line_no, rules, action)
-            return NoQaDirective(line_no, None, None)
-        return None
-
     @staticmethod
     def remove_templated_errors(
         linting_errors: List[SQLBaseError],
@@ -337,6 +270,7 @@ class Linter:
         result: List[SQLBaseError] = []
         for e in linting_errors:
             if isinstance(e, SQLLintError):
+                assert e.segment.pos_marker
                 if (
                     # Is it in a literal section?
                     e.segment.pos_marker.is_literal()
@@ -351,13 +285,13 @@ class Linter:
         return result
 
     @staticmethod
-    def _report_conflicting_fixes_same_anchor(message: str):  # pragma: no cover
+    def _report_conflicting_fixes_same_anchor(message: str) -> None:  # pragma: no cover
         # This function exists primarily in order to let us monkeypatch it at
         # runtime (replacing it with a function that raises an exception).
         linter_logger.critical(message)
 
     @staticmethod
-    def _warn_unfixable(code: str):
+    def _warn_unfixable(code: str) -> None:
         linter_logger.warning(
             f"One fix for {code} not applied, it would re-cause the same error."
         )
@@ -369,13 +303,13 @@ class Linter:
     def parse_rendered(
         cls,
         rendered: RenderedFile,
-        recurse: bool = True,
+        parse_statistics: bool = False,
     ) -> ParsedString:
         """Parse a rendered file."""
         t0 = time.monotonic()
         violations = cast(List[SQLBaseError], rendered.templater_violations)
         tokens: Optional[Sequence[BaseSegment]]
-        if rendered.templated_file:
+        if rendered.templated_file is not None:
             tokens, lvs, config = cls._lex_templated_file(
                 rendered.templated_file, rendered.config
             )
@@ -390,8 +324,8 @@ class Linter:
             parsed, pvs = cls._parse_tokens(
                 tokens,
                 rendered.config,
-                recurse=recurse,
                 fname=rendered.fname,
+                parse_statistics=parse_statistics,
             )
             violations += pvs
         else:
@@ -412,80 +346,17 @@ class Linter:
             rendered.source_str,
         )
 
-    @classmethod
-    def extract_ignore_from_comment(
-        cls,
-        comment: RawSegment,
-        rule_codes: List[str],
-    ):
-        """Extract ignore mask entries from a comment segment."""
-        # Also trim any whitespace afterward
-        comment_content = comment.raw_trimmed().strip()
-        comment_line, _ = comment.pos_marker.source_position()
-        result = cls.parse_noqa(comment_content, comment_line, rule_codes)
-        if isinstance(result, SQLParseError):
-            result.segment = comment
-        return result
-
-    @classmethod
-    def extract_ignore_mask_tree(
-        cls,
-        tree: BaseSegment,
-        rule_codes: List[str],
-    ) -> Tuple[List[NoQaDirective], List[SQLBaseError]]:
-        """Look for inline ignore comments and return NoQaDirectives."""
-        ignore_buff: List[NoQaDirective] = []
-        violations: List[SQLBaseError] = []
-        for comment in tree.recursive_crawl("comment"):
-            if comment.is_type("inline_comment"):
-                ignore_entry = cls.extract_ignore_from_comment(comment, rule_codes)
-                if isinstance(ignore_entry, SQLParseError):
-                    violations.append(ignore_entry)
-                elif ignore_entry:
-                    ignore_buff.append(ignore_entry)
-        if ignore_buff:
-            linter_logger.info("Parsed noqa directives from file: %r", ignore_buff)
-        return ignore_buff, violations
-
-    @classmethod
-    def extract_ignore_mask_source(
-        cls,
-        source: str,
-        inline_comment_regex: RegexLexer,
-        rule_codes: List[str],
-    ) -> Tuple[List[NoQaDirective], List[SQLBaseError]]:
-        """Look for inline ignore comments and return NoQaDirectives.
-
-        Very similar to extract_ignore_mask_tree(), but can be run on raw source
-        (i.e. does not require the code to have parsed successfully).
-        """
-        ignore_buff: List[NoQaDirective] = []
-        violations: List[SQLBaseError] = []
-        for idx, line in enumerate(source.split("\n")):
-            match = inline_comment_regex.search(line) if line else None
-            if match:
-                ignore_entry = cls.parse_noqa(
-                    line[match[0] : match[1]], idx + 1, rule_codes
-                )
-                if isinstance(ignore_entry, SQLParseError):
-                    violations.append(ignore_entry)  # pragma: no cover
-                elif ignore_entry:
-                    ignore_buff.append(ignore_entry)
-        if ignore_buff:
-            linter_logger.info("Parsed noqa directives from file: %r", ignore_buff)
-        return ignore_buff, violations
-
     @classmethod
     def lint_fix_parsed(
         cls,
         tree: BaseSegment,
         config: FluffConfig,
-        rule_set: List[BaseRule],
+        rule_pack: RulePack,
         fix: bool = False,
         fname: Optional[str] = None,
-        templated_file: Optional[TemplatedFile] = None,
+        templated_file: Optional["TemplatedFile"] = None,
         formatter: Any = None,
-    ) -> Tuple[BaseSegment, List[SQLBaseError], List[NoQaDirective]]:
+    ) -> Tuple[BaseSegment, List[SQLBaseError], Optional[IgnoreMask], RuleTimingsType]:
         """Lint and optionally fix a tree object."""
         # Keep track of the linting errors on the very first linter pass. The
         # list of issues output by "lint" and "fix" only includes issues present
@@ -495,7 +366,9 @@ class Linter:
         # A placeholder for the fixes we had on the previous loop
         last_fixes = None
         # Keep a set of previous versions to catch infinite loops.
-        previous_versions: Set[Tuple[str, Tuple[SourceFix, ...]]] = {(tree.raw, ())}
+        previous_versions: Set[Tuple[str, Tuple["SourceFix", ...]]] = {(tree.raw, ())}
+        # Keep a buffer for recording rule timings.
+        rule_timings: RuleTimingsType = []
 
         # If we are fixing then we want to loop up to the runaway_limit, otherwise just
         # once for linting.
@@ -503,15 +376,14 @@ class Linter:
 
         # Dispatch the output for the lint header
         if formatter:
-            formatter.dispatch_lint_header(fname)
+            formatter.dispatch_lint_header(fname, sorted(rule_pack.codes()))
 
         # Look for comment segments which might indicate lines to ignore.
         if not config.get("disable_noqa"):
-            rule_codes = [r.code for r in rule_set]
-            ignore_buff, ivs = cls.extract_ignore_mask_tree(tree, rule_codes)
+            ignore_mask, ivs = IgnoreMask.from_tree(tree, rule_pack.reference_map)
             initial_linting_errors += ivs
         else:
-            ignore_buff = []
+            ignore_mask = None
 
         save_tree = tree
         # There are two phases of rule running.
@@ -528,26 +400,27 @@ class Linter:
         for phase in phases:
             if len(phases) > 1:
                 rules_this_phase = [
-                    rule for rule in rule_set if rule.lint_phase == phase
+                    rule for rule in rule_pack.rules if rule.lint_phase == phase
                 ]
             else:
-                rules_this_phase = rule_set
+                rules_this_phase = rule_pack.rules
             for loop in range(loop_limit if phase == "main" else 2):
 
-                def is_first_linter_pass():
+                def is_first_linter_pass() -> bool:
                     return phase == phases[0] and loop == 0
 
                 # Additional newlines are to assist in scanning linting loops
                 # during debugging.
                 linter_logger.info(
-                    f"\n\nEntering linter phase {phase}, loop {loop+1}/{loop_limit}\n"
+                    f"\n\nEntering linter phase {phase}, "
+                    f"loop {loop + 1}/{loop_limit}\n"
                 )
                 changed = False
 
                 if is_first_linter_pass():
                     # In order to compute initial_linting_errors correctly, need
                     # to run all rules on the first loop of the main phase.
-                    rules_this_phase = rule_set
+                    rules_this_phase = rule_pack.rules
                 progress_bar_crawler = tqdm(
                     rules_this_phase,
                     desc="lint by rules",
@@ -564,11 +437,12 @@ class Linter:
                     if (
                         fix
                         and not is_first_linter_pass()
-                        and not is_fix_compatible(crawler)
+                        and not crawler.is_fix_compatible
                     ):
                         continue
 
                     progress_bar_crawler.set_description(f"rule {crawler.code}")
+                    t0 = time.monotonic()
 
                     # fixes should be a dict {} with keys edit, delete, create
                     # delete is just a list of segments to delete
@@ -580,7 +454,7 @@ class Linter:
                         dialect=config.get("dialect_obj"),
                         fix=fix,
                         templated_file=templated_file,
-                        ignore_mask=ignore_buff,
+                        ignore_mask=ignore_mask,
                         fname=fname,
                         config=config,
                     )
@@ -590,7 +464,7 @@ class Linter:
                     if fix and fixes:
                         linter_logger.info(f"Applying Fixes [{crawler.code}]: {fixes}")
                         # Do some sanity checks on the fixes before applying.
-                        anchor_info = BaseSegment.compute_anchor_edit_info(fixes)
+                        anchor_info = compute_anchor_edit_info(fixes)
                         if any(
                             not info.is_valid for info in anchor_info.values()
                         ):  # pragma: no cover
@@ -598,8 +472,13 @@ class Linter:
                                 f"Rule {crawler.code} returned conflicting "
                                 "fixes with the same anchor. This is only "
                                 "supported for create_before+create_after, so "
-                                f"the fixes will not be applied. {fixes!r}"
+                                "the fixes will not be applied. "
                             )
+                            for uuid, info in anchor_info.items():
+                                if not info.is_valid:
+                                    message += f"\n{uuid}:"
+                                    for _fix in info.fixes:
+                                        message += f"\n    {_fix}"
                             cls._report_conflicting_fixes_same_anchor(message)
                             for lint_result in linting_errors:
                                 lint_result.fixes = []
@@ -613,8 +492,11 @@ class Linter:
                             # This is the happy path. We have fixes, now we want to
                             # apply them.
                             last_fixes = fixes
-                            new_tree, _, _ = tree.apply_fixes(
-                                config.get("dialect_obj"), crawler.code, anchor_info
+                            new_tree, _, _, _valid = apply_fixes(
+                                tree,
+                                config.get("dialect_obj"),
+                                crawler.code,
+                                anchor_info,
                             )
                             # Check for infinite loops. We use a combination of the
                             # fixed templated file and the list of source fixes to
@@ -623,7 +505,16 @@ class Linter:
                                 new_tree.raw,
                                 tuple(new_tree.source_fixes),
                             )
-                            if loop_check_tuple not in previous_versions:
+                            if not _valid:
+                                # The fixes result in an invalid file. Don't apply
+                                # the fix and skip onward. Show a warning.
+                                linter_logger.warning(
+                                    f"Fixes for {crawler.code} not applied, as it "
+                                    "would result in an unparsable file. Please "
+                                    "report this as a bug with a minimal query "
+                                    "which demonstrates this warning."
+                                )
+                            elif loop_check_tuple not in previous_versions:
                                 # We've not seen this version of the file so
                                 # far. Continue.
                                 tree = new_tree
@@ -636,6 +527,11 @@ class Linter:
                                 # we want to stop.
                                 cls._warn_unfixable(crawler.code)
 
+                    # Record rule timing
+                    rule_timings.append(
+                        (crawler.code, crawler.name, time.monotonic() - t0)
+                    )
+
                 if fix and not changed:
                     # We did not change the file. Either the file is clean (no
                     # fixes), or any fixes which are present will take us back
@@ -671,22 +567,25 @@ class Linter:
                     # Reason: When the linter hits the loop limit, the file is often
                     # messy, e.g. some of the fixes were applied repeatedly, possibly
                     # other weird things. We don't want the user to see this junk!
-                    return save_tree, initial_linting_errors, ignore_buff
+                    return save_tree, initial_linting_errors, ignore_mask, rule_timings
 
         if config.get("ignore_templated_areas", default=True):
             initial_linting_errors = cls.remove_templated_errors(initial_linting_errors)
 
-        return tree, initial_linting_errors, ignore_buff
+        linter_logger.info("\n###\n#\n# {}\n#\n###".format("Fixed Tree:"))
+        linter_logger.info("\n" + tree.stringify())
+
+        return tree, initial_linting_errors, ignore_mask, rule_timings
 
     @classmethod
     def lint_parsed(
         cls,
         parsed: ParsedString,
-        rule_set: List[BaseRule],
+        rule_pack: RulePack,
         fix: bool = False,
         formatter: Any = None,
         encoding: str = "utf8",
-    ):
+    ) -> LintedFile:
         """Lint a ParsedString and return a LintedFile."""
         violations = parsed.violations
         time_dict = parsed.time_dict
@@ -694,10 +593,15 @@ class Linter:
         if parsed.tree:
             t0 = time.monotonic()
             linter_logger.info("LINTING (%s)", parsed.fname)
-            tree, initial_linting_errors, ignore_buff = cls.lint_fix_parsed(
+            (
+                tree,
+                initial_linting_errors,
+                ignore_mask,
+                rule_timings,
+            ) = cls.lint_fix_parsed(
                 parsed.tree,
                 config=parsed.config,
-                rule_set=rule_set,
+                rule_pack=rule_pack,
                 fix=fix,
                 fname=parsed.fname,
                 templated_file=parsed.templated_file,
@@ -712,20 +616,21 @@ class Linter:
         else:
             # If no parsed tree, set to None
             tree = None
-            ignore_buff = []
+            ignore_mask = None
+            rule_timings = []
             if not parsed.config.get("disable_noqa"):
                 # Templating and/or parsing have failed. Look for "noqa"
                 # comments (the normal path for identifying these comments
                 # requires access to the parse tree, and because of the failure,
                 # we don't have a parse tree).
-                ignore_buff, ignore_violations = cls.extract_ignore_mask_source(
+                ignore_mask, ignore_violations = IgnoreMask.from_source(
                     parsed.source_str,
                     [
                         lm
                         for lm in parsed.config.get("dialect_obj").lexer_matchers
                         if lm.name == "inline_comment"
                     ][0],
-                    [r.code for r in rule_set],
+                    rule_pack.reference_map,
                 )
                 violations += ignore_violations
 
@@ -738,9 +643,9 @@ class Linter:
             parsed.fname,
             # Deduplicate violations
             LintedFile.deduplicate_in_source_space(violations),
-            time_dict,
+            FileTimings(time_dict, rule_timings),
             tree,
-            ignore_mask=ignore_buff,
+            ignore_mask=ignore_mask,
             templated_file=parsed.templated_file,
             encoding=encoding,
         )
@@ -748,7 +653,10 @@ class Linter:
         # This is the main command line output from linting.
         if formatter:
             formatter.dispatch_file_violations(
-                parsed.fname, linted_file, only_fixable=fix
+                parsed.fname,
+                linted_file,
+                only_fixable=fix,
+                warn_unused_ignores=parsed.config.get("warn_unused_ignores"),
             )
 
         # Safety flag for unset dialects
@@ -764,7 +672,7 @@ class Linter:
     def lint_rendered(
         cls,
         rendered: RenderedFile,
-        rule_set: List[BaseRule],
+        rule_pack: RulePack,
         fix: bool = False,
         formatter: Any = None,
     ) -> LintedFile:
@@ -772,7 +680,7 @@ class Linter:
         parsed = cls.parse_rendered(rendered)
         return cls.lint_parsed(
             parsed,
-            rule_set=rule_set,
+            rule_pack=rule_pack,
             fix=fix,
             formatter=formatter,
             encoding=rendered.encoding,
@@ -821,7 +729,7 @@ class Linter:
             templated_file = None
             templater_violations = []
 
-        if not templated_file:
+        if templated_file is None:
             linter_logger.info("TEMPLATING FAILED: %s", templater_violations)
 
         # Record time
@@ -848,9 +756,9 @@ class Linter:
         self,
         in_str: str,
         fname: str = "<string>",
-        recurse: bool = True,
         config: Optional[FluffConfig] = None,
         encoding: str = "utf-8",
+        parse_statistics: bool = False,
     ) -> ParsedString:
         """Parse a string."""
         violations: List[SQLBaseError] = []
@@ -863,7 +771,7 @@ class Linter:
         config = config or self.config
 
         # Scan the raw file for config commands.
-        config.process_raw_file_for_config(in_str)
+        config.process_raw_file_for_config(in_str, fname)
         rendered = self.render_string(in_str, fname, config, encoding)
         violations += rendered.templater_violations
 
@@ -871,22 +779,22 @@ class Linter:
         if self.formatter:
             self.formatter.dispatch_parse_header(fname)
 
-        return self.parse_rendered(rendered, recurse=recurse)
+        return self.parse_rendered(rendered, parse_statistics=parse_statistics)
 
     def fix(
         self,
         tree: BaseSegment,
         config: Optional[FluffConfig] = None,
         fname: Optional[str] = None,
-        templated_file: Optional[TemplatedFile] = None,
+        templated_file: Optional["TemplatedFile"] = None,
     ) -> Tuple[BaseSegment, List[SQLBaseError]]:
         """Return the fixed tree and violations from lintfix when we're fixing."""
         config = config or self.config
-        rule_set = self.get_ruleset(config=config)
-        fixed_tree, violations, _ = self.lint_fix_parsed(
+        rule_pack = self.get_rulepack(config=config)
+        fixed_tree, violations, _, _ = self.lint_fix_parsed(
             tree,
             config,
-            rule_set,
+            rule_pack,
             fix=True,
             fname=fname,
             templated_file=templated_file,
@@ -899,15 +807,15 @@ class Linter:
         tree: BaseSegment,
         config: Optional[FluffConfig] = None,
         fname: Optional[str] = None,
-        templated_file: Optional[TemplatedFile] = None,
+        templated_file: Optional["TemplatedFile"] = None,
     ) -> List[SQLBaseError]:
         """Return just the violations from lintfix when we're only linting."""
         config = config or self.config
-        rule_set = self.get_ruleset(config=config)
-        _, violations, _ = self.lint_fix_parsed(
+        rule_pack = self.get_rulepack(config=config)
+        _, violations, _, _ = self.lint_fix_parsed(
             tree,
             config,
-            rule_set,
+            rule_pack,
             fix=False,
             fname=fname,
             templated_file=templated_file,
@@ -938,11 +846,11 @@ class Linter:
             config=config,
         )
         # Get rules as appropriate
-        rule_set = self.get_ruleset(config=config)
+        rule_pack = self.get_rulepack(config=config)
         # Lint the file and return the LintedFile
         return self.lint_parsed(
             parsed,
-            rule_set,
+            rule_pack,
             fix=fix,
             formatter=self.formatter,
             encoding=encoding,
@@ -1046,7 +954,12 @@ class Linter:
             for ignore_base, ignore_spec in ignores.items():
                 abs_ignore_base = os.path.abspath(ignore_base)
                 if abs_fpath.startswith(
-                    abs_ignore_base + os.sep
+                    abs_ignore_base
+                    + (
+                        ""
+                        if os.path.dirname(abs_ignore_base) == abs_ignore_base
+                        else os.sep
+                    )
                 ) and ignore_spec.match_file(
                     os.path.relpath(abs_fpath, abs_ignore_base)
                 ):
@@ -1114,6 +1027,9 @@ class Linter:
         ignore_non_existent_files: bool = False,
         ignore_files: bool = True,
         processes: Optional[int] = None,
+        apply_fixes: bool = False,
+        fixed_file_suffix: str = "",
+        fix_even_unparsable: bool = False,
     ) -> LintingResult:
         """Lint an iterable of paths."""
         # If no paths specified - assume local
@@ -1138,6 +1054,10 @@ class Linter:
         files_count = len(expanded_paths)
         if processes is None:
             processes = self.config.get("processes", default=1)
+        # Hard set processes to 1 if only 1 file is queued.
+        # The overhead will never be worth it with one file.
+        if files_count == 1:
+            processes = 1
 
         # to avoid circular import
         from sqlfluff.core.linter.runner import get_runner
@@ -1169,6 +1089,18 @@ class Linter:
                 linter_logger.error("Fatal linting error. Halting further linting.")
                 break
 
+            # If we're applying fixes, then do that here.
+            if apply_fixes:
+                num_tmp_prs_errors = linted_file.num_violations(
+                    types=TMP_PRS_ERROR_TYPES,
+                    filter_ignore=False,
+                    filter_warning=False,
+                )
+                if fix_even_unparsable or num_tmp_prs_errors == 0:
+                    linted_file.persist_tree(
+                        suffix=fixed_file_suffix, formatter=self.formatter
+                    )
+
             # Progress bar for files is rendered only when there is more than one file.
             # Additionally, as it's updated after each loop, we need to get file name
             # from the next loop. This is why `enumerate` starts with `1` and there
@@ -1183,7 +1115,7 @@ class Linter:
     def parse_path(
         self,
         path: str,
-        recurse: bool = True,
+        parse_statistics: bool = False,
     ) -> Iterator[ParsedString]:
         """Parse a path of sql files.
 
@@ -1204,7 +1136,7 @@ class Linter:
             yield self.parse_string(
                 raw_file,
                 fname=fname,
-                recurse=recurse,
                 config=config,
                 encoding=encoding,
+                parse_statistics=parse_statistics,
             )
