@@ -1,12 +1,11 @@
 """Conditional Grammar."""
 
-from sqlfluff.core.parser.segments import Indent
-from sqlfluff.core.parser.match_result import MatchResult
-from sqlfluff.core.parser.match_wrapper import match_wrapper
+from typing import Sequence, Type, Union
 
-from sqlfluff.core.parser.grammar.base import (
-    BaseGrammar,
-)
+from sqlfluff.core.parser.context import ParseContext
+from sqlfluff.core.parser.grammar.base import BaseGrammar
+from sqlfluff.core.parser.match_result import MatchResult
+from sqlfluff.core.parser.segments import BaseSegment, Indent
 
 
 class Conditional(BaseGrammar):
@@ -26,7 +25,7 @@ class Conditional(BaseGrammar):
             are met for this grammar to be enabled.
 
     Example:
-        .. code-block::
+        .. code-block:: python
 
             Conditional(Dedent, config_type="indent", indented_joins=False)
 
@@ -41,33 +40,45 @@ class Conditional(BaseGrammar):
     | "indentation" keys are currently set up.
     """
 
-    def __init__(self, *args, config_type: str = "indentation", **rules):
-        if not all(issubclass(arg, Indent) for arg in args):
-            raise ValueError(
-                "Conditional is only designed to work with Indent segments."
-            )
-        if len(args) != 1:
-            raise ValueError(
-                "Conditional is only designed to work with a single element."
-            )
-        if not config_type:
+    def __init__(
+        self,
+        meta: Type[Indent],
+        config_type: str = "indentation",
+        **rules: Union[str, bool]
+    ):
+        """Initialize a new instance of the class.
+
+        This method initializes an instance of the class with the provided
+        arguments.
+
+        Args:
+            meta (Type[Indent]): The meta argument.
+            config_type (str, optional): The config_type argument. Defaults to
+                "indentation".
+            **rules (Union[str, bool]): The rules argument.
+        """
+        assert issubclass(
+            meta, Indent
+        ), "Conditional is only designed to work with Indent/Dedent segments."
+        self._meta = meta
+        if not config_type:  # pragma: no cover
             raise ValueError("Conditional config_type must be set.")
-        elif config_type not in ("indentation"):
+        elif config_type not in ("indentation"):  # pragma: no cover
             raise ValueError(
                 "Only 'indentation' is supported as a Conditional config_type."
             )
-        if not rules:
+        if not rules:  # pragma: no cover
             raise ValueError("Conditional requires rules to be set.")
         self._config_type = config_type
         self._config_rules = rules
-        super().__init__(*args)
+        super().__init__()
 
-    def is_enabled(self, parse_context):
+    def is_enabled(self, parse_context: ParseContext) -> bool:
         """Evaluate conditionals and return whether enabled."""
         # NOTE: Because only "indentation" is the only current config_type
         # supported, this code is much simpler that would be required in
         # future if multiple options are available.
-        if self._config_type != "indentation":
+        if self._config_type != "indentation":  # pragma: no cover
             raise ValueError(
                 "Only 'indentation' is supported as a Conditional config_type."
             )
@@ -81,12 +92,16 @@ class Conditional(BaseGrammar):
                 return False
         return True
 
-    @match_wrapper()
-    def match(self, segments, parse_context):
-        """Evaluate conditionals and return content."""
+    def match(
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
+        """If enabled, return a single insert of the new segment."""
         if not self.is_enabled(parse_context):
-            return MatchResult.from_unmatched(segments)
+            return MatchResult.empty_at(idx)
 
-        # Instantiate the new element and return
-        new_seg = self._elements[0]()
-        return MatchResult((new_seg,), segments)
+        return MatchResult(
+            matched_slice=slice(idx, idx), insert_segments=((idx, self._meta),)
+        )
