@@ -2,18 +2,28 @@
 
 import ast
 from string import Formatter
-from typing import Iterable, Dict, Tuple, List, Iterator, Optional, NamedTuple
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    NamedTuple,
+    Optional,
+    Tuple,
+)
 
 from sqlfluff.core.errors import SQLTemplaterError
-from sqlfluff.core.string_helpers import findall
-
+from sqlfluff.core.helpers.slice import offset_slice, zero_slice
+from sqlfluff.core.helpers.string import findall
 from sqlfluff.core.templaters.base import (
+    RawFileSlice,
     RawTemplater,
     TemplatedFile,
-    templater_logger,
-    RawFileSlice,
     TemplatedFileSlice,
     large_file_check,
+    templater_logger,
 )
 
 
@@ -54,9 +64,9 @@ class IntermediateFileSlice(NamedTuple):
                 # Assume it's a literal, check the literal actually matches.
                 templated_len = len(focus.raw)
                 if target_end == "head":
-                    check_slice = slice(
+                    check_slice = offset_slice(
                         main_templated_slice.start,
-                        main_templated_slice.start + templated_len,
+                        templated_len,
                     )
                 else:
                     check_slice = slice(
@@ -124,7 +134,7 @@ class IntermediateFileSlice(NamedTuple):
         # Return
         return head_buffer, new_slice, tail_buffer
 
-    def try_simple(self):
+    def try_simple(self) -> TemplatedFileSlice:
         """Try to turn this intermediate slice into a simple slice."""
         # Yield anything simple
         if len(self.slice_buffer) == 1:
@@ -136,7 +146,7 @@ class IntermediateFileSlice(NamedTuple):
         else:
             raise ValueError("IntermediateFileSlice is not simple!")
 
-    def coalesce(self):
+    def coalesce(self) -> TemplatedFileSlice:
         """Coalesce this whole slice into a single one. Brutally."""
         return TemplatedFileSlice(
             PythonTemplater._coalesce_types(self.slice_buffer),
@@ -159,12 +169,12 @@ class PythonTemplater(RawTemplater):
 
     name = "python"
 
-    def __init__(self, override_context=None, **kwargs):
+    def __init__(self, override_context=None, **kwargs) -> None:
         self.default_context = dict(test_value="__test__")
         self.override_context = override_context or {}
 
     @staticmethod
-    def infer_type(s):
+    def infer_type(s) -> Any:
         """Infer a python type from a string and convert.
 
         Given a string value, convert it to a more specific built-in Python type
@@ -177,7 +187,22 @@ class PythonTemplater(RawTemplater):
             return s
 
     def get_context(self, fname=None, config=None, **kw) -> Dict:
-        """Get the templating context from the config."""
+        """Get the templating context from the config.
+
+        This function retrieves the templating context from the config by
+        loading the config and updating the live_context dictionary with the
+        loaded_context and other predefined context dictionaries. It then goes
+        through the loaded_context dictionary and infers the types of the values
+        before returning the live_context dictionary.
+
+        Args:
+            fname (str, optional): The file name.
+            config (dict, optional): The config dictionary.
+            **kw: Additional keyword arguments.
+
+        Returns:
+            dict: The templating context.
+        """
         # TODO: The config loading should be done outside the templater code. Here
         # is a silly place.
         if config:
@@ -201,7 +226,7 @@ class PythonTemplater(RawTemplater):
     @large_file_check
     def process(
         self, *, in_str: str, fname: str, config=None, formatter=None
-    ) -> Tuple[Optional[TemplatedFile], list]:
+    ) -> Tuple[Optional[TemplatedFile], List]:
         """Process a string and return a TemplatedFile.
 
         Note that the arguments are enforced as keywords
@@ -222,16 +247,23 @@ class PythonTemplater(RawTemplater):
 
         """
         live_context = self.get_context(fname=fname, config=config)
-        try:
-            new_str = in_str.format(**live_context)
-        except KeyError as err:
-            # TODO: Add a url here so people can get more help.
-            raise SQLTemplaterError(
-                "Failure in Python templating: {}. Have you configured your "
-                "variables?".format(err)
-            )
+
+        def render_func(raw_str: str) -> str:
+            """Render the string using the captured live_context."""
+            try:
+                rendered_str = raw_str.format(**live_context)
+            except KeyError as err:
+                raise SQLTemplaterError(
+                    "Failure in Python templating: {}. Have you configured your "
+                    "variables? https://docs.sqlfluff.com/en/stable/"
+                    "configuration.html#templating-configuration".format(err)
+                )
+            return rendered_str
+
         raw_sliced, sliced_file, new_str = self.slice_file(
-            in_str, new_str, config=config
+            in_str,
+            render_func=render_func,
+            config=config,
         )
         return (
             TemplatedFile(
@@ -245,11 +277,16 @@ class PythonTemplater(RawTemplater):
         )
 
     def slice_file(
-        self, raw_str: str, templated_str: str, config=None, **kwargs
+        self, raw_str: str, render_func: Callable[[str], str], config=None, **kwargs
     ) -> Tuple[List[RawFileSlice], List[TemplatedFileSlice], str]:
         """Slice the file to determine regions where we can fix."""
         templater_logger.info("Slicing File Template")
         templater_logger.debug("    Raw String: %r", raw_str)
+        # Render the templated string.
+        # NOTE: This seems excessive in this simple example, but for other templating
+        # engines we need more control over the rendering so may need to call this
+        # method more than once.
+        templated_str = render_func(raw_str)
         templater_logger.debug("    Templated String: %r", templated_str)
         # Slice the raw file
         raw_sliced = list(self._slice_template(raw_str))
@@ -322,7 +359,8 @@ class PythonTemplater(RawTemplater):
     ) -> Tuple[List[TemplatedFileSlice], str]:
         """Identify a wrapped query (e.g. dbt test) and handle it.
 
-        If unwrap_wrapped is true, we trim the wrapping from the templated file.
+        If unwrap_wrapped is true, we trim the wrapping from the templated
+        file.
         If unwrap_wrapped is false, we add a slice at start and end.
         """
         if not slices:
@@ -367,7 +405,7 @@ class PythonTemplater(RawTemplater):
             slices.append(
                 TemplatedFileSlice(
                     "templated",
-                    slice(last_slice.source_slice.stop, last_slice.source_slice.stop),
+                    zero_slice(last_slice.source_slice.stop),
                     slice(last_slice.templated_slice.stop, len(templated_str)),
                 )
             )
@@ -505,14 +543,13 @@ class PythonTemplater(RawTemplater):
                 idx = None
                 yield IntermediateFileSlice(
                     "invariant",
-                    slice(
+                    offset_slice(
                         raw_file_slice.source_idx,
-                        raw_file_slice.source_idx + len(raw_file_slice.raw),
+                        len(raw_file_slice.raw),
                     ),
-                    slice(
+                    offset_slice(
                         templated_occurrences[raw_file_slice.raw][0],
-                        templated_occurrences[raw_file_slice.raw][0]
-                        + len(raw_file_slice.raw),
+                        len(raw_file_slice.raw),
                     ),
                     [
                         RawFileSlice(
@@ -1013,8 +1050,8 @@ class PythonTemplater(RawTemplater):
                 # Yield the literal
                 owu_literal_slice = TemplatedFileSlice(
                     "literal",
-                    slice(raw_idx, raw_idx + raw_len),
-                    slice(template_idx, template_idx + raw_len),
+                    offset_slice(raw_idx, raw_len),
+                    offset_slice(template_idx, raw_len),
                 )
                 templater_logger.debug(
                     "    Yielding Unique: %r, %s",
