@@ -16,7 +16,6 @@ CONNECT
 CONNECTION
 CONSTRAINT
 CREATE
-CROSS
 CURRENT
 CURRENT_DATE
 CURRENT_TIME
@@ -24,6 +23,7 @@ CURRENT_TIMESTAMP
 CURRENT_USER
 DELETE
 DISTINCT
+DO
 DROP
 ELSE
 EXISTS
@@ -44,7 +44,6 @@ INSERT_ONLY
 INTERSECT
 INTO
 IS
-ISSUE
 JOIN
 LATERAL
 LEFT
@@ -55,6 +54,7 @@ MATCH_RECOGNIZE
 MINUS
 NATURAL
 NOT
+NOTIFY
 NULL
 NULL_IF
 OF
@@ -101,6 +101,7 @@ ADD
 ADMIN
 AFTER
 ALLOWED_IP_LIST
+ALLOWED_VALUES
 ALLOW_DUPLICATE
 ALLOW_OVERLAPPING_EXECUTION
 API
@@ -134,6 +135,7 @@ AZURE
 AZURE_EVENT_GRID
 AZURE_EVENT_GRID_TOPIC_ENDPOINT
 AZURE_SAS_TOKEN
+AZURE_STORAGE_QUEUE
 AZURE_STORAGE_QUEUE_PRIMARY_URI
 AZURE_TENANT_ID
 BASE64
@@ -146,6 +148,7 @@ BINARY_FORMAT
 BINDING
 BLOCK
 BLOCKED_IP_LIST
+BODY
 BROTLI
 BZ2
 CACHE
@@ -154,6 +157,7 @@ CALLED
 CALLER
 CASCADE
 CASE
+CASES
 CASE_INSENSITIVE
 CASE_SENSITIVE
 CHAIN
@@ -176,6 +180,8 @@ CONTINUE
 COPY
 COPY_OPTIONS
 CREDENTIALS
+CREDIT_QUOTA
+CROSS
 CSV
 CUBE
 CURRENT_ACCOUNT
@@ -191,6 +197,7 @@ CURRENT_STATEMENT
 CURRENT_TRANSACTION
 CURRENT_VERSION
 CURRENT_WAREHOUSE
+CURSOR
 CYCLE
 DATA
 DATABASE
@@ -199,6 +206,7 @@ DATA_FORMAT
 DATA_RETENTION_TIME_IN_DAYS
 DATE
 DATE_FORMAT
+DAILY
 DAYS_TO_EXPIRY
 DEFAULT
 DEFAULT_DDL_COLLATION
@@ -233,6 +241,7 @@ ENABLE_OCTAL
 ENCODING
 ENCRYPTION
 END
+END_TIMESTAMP
 ENFORCED
 ENFORCE_LENGTH
 ENUM
@@ -260,11 +269,13 @@ FILTER
 FINAL
 FIRST
 FIRST_NAME
+FOR
 FORCE
 FOREIGN
 FORMAT
 FORMATS
 FORMAT_NAME
+FREQUENCY
 FUNCTION
 FUNCTIONS
 FUTURE
@@ -278,6 +289,7 @@ GRANTED
 GRANTS
 GROUPING
 GZIP
+HANDLER
 HEADER
 HEADERS
 HEX
@@ -288,8 +300,10 @@ IF
 IGNORE
 IGNORE_UTF8_ERRORS
 IMMEDIATE
+IMMEDIATELY
 IMMUTABLE
 IMPORT
+IMPORTS
 IMPORTED
 INCLUDE_QUERY_ID
 INDEX
@@ -300,6 +314,8 @@ INPUT
 INTEGRATION
 INTEGRATIONS
 INTERVAL
+ISSUE
+JAVA
 JAVASCRIPT
 JSON
 KEY
@@ -310,6 +326,7 @@ LAST
 LAST_NAME
 LAST_QUERY_ID
 LAST_TRANSACTION
+LET
 LIMIT
 LIST
 LISTING
@@ -348,12 +365,14 @@ MODEL
 MODIFIED_AFTER
 MODIFY
 MONITOR
+MONTHLY
 MUST_CHANGE_PASSWORD
 NAME
 NAN
 NETWORK
 NEXT
 NEXTVAL
+NEVER
 NO
 NOCACHE
 NOCYCLE
@@ -362,6 +381,7 @@ NOORDER
 NOTIFICATION
 NOTIFICATION_INTEGRATION
 NOTIFICATION_PROVIDER
+NOTIFY_USERS
 NULLS
 NULL_IF
 OBJECT
@@ -385,6 +405,7 @@ OVERRIDE
 OVERWRITE
 OWNER
 OWNERSHIP
+PACKAGES
 PARALLEL
 PARAMETERS
 PARQUET
@@ -394,6 +415,7 @@ PAST
 PATTERN
 FIELD_OPTIONALITY_ENCLOSED_BY
 PER
+PERCENT
 PERMUTE
 PIPE
 PIPES
@@ -413,6 +435,7 @@ PROCEDURES
 PUBLIC
 PURGE
 PUT
+PYTHON
 QUERIES
 QUEUE
 RANGE
@@ -441,7 +464,9 @@ RESPONSE_TRANSLATOR
 RESTRICT
 RESTRICTIONS
 RESULT
+RESULTSET
 RESUME
+RETURN
 RETURNS
 RETURN_ALL_ERRORS
 RETURN_ERRORS
@@ -457,7 +482,9 @@ ROW
 RSA_PUBLIC_KEY
 RSA_PUBLIC_KEY_2
 RUNNING
+RUNTIME_VERSION
 S3
+SCALA
 SCALING_POLICY
 SCHEDULE
 SCHEMA
@@ -498,6 +525,7 @@ STAGE_COPY_OPTIONS
 STAGE_FILE_FORMAT
 STANDARD
 STARTS
+START_TIMESTAMP
 STATEMENT
 STATEMENT_QUEUED_TIMEOUT_IN_SECONDS
 STATEMENT_TIMEOUT_IN_SECONDS
@@ -514,7 +542,9 @@ STRIP_NULL_VALUES
 STRIP_OUTER_ARRAY
 STRIP_OUTER_ELEMENT
 SUBPATH
+SUPPORT
 SUSPEND
+SUSPEND_IMMEDIATE
 SUSPENDED
 SWAP
 SYSDATE
@@ -523,6 +553,7 @@ TABLES
 TABLESPACE
 TABULAR
 TAG
+TARGET_PATH
 TASK
 TASKS
 TEMP
@@ -539,6 +570,7 @@ TRANSACTION
 TRANSACTIONS
 TRANSIENT
 TRIM_SPACE
+TRIGGERS
 TRUNCATE
 TRUNCATECOLUMNS
 TYPE
@@ -572,6 +604,7 @@ WAREHOUSE
 WAREHOUSES
 WAREHOUSE_SIZE
 WAREHOUSE_TYPE
+WEEKLY
 WINDOW
 WITH
 WITHIN
@@ -581,6 +614,7 @@ WOY
 WRAPPER
 WRITE
 XML
+YEARLY
 ZONE
 ZSTD
 """
