@@ -8,12 +8,15 @@ Teradata Database SQL Data Definition Language Syntax and Examples
 
 """
 
+from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.core.parser import (
     AnyNumberOf,
     Anything,
     BaseSegment,
     Bracketed,
     CodeSegment,
+    ComparisonOperatorSegment,
+    CompositeComparisonOperatorSegment,
     Dedent,
     Delimited,
     Indent,
@@ -23,10 +26,8 @@ from sqlfluff.core.parser import (
     Ref,
     RegexLexer,
     Sequence,
-    StartsWith,
+    StringParser,
 )
-
-from sqlfluff.core.dialects import load_raw_dialect
 from sqlfluff.dialects import dialect_ansi as ansi
 
 ansi_dialect = load_raw_dialect("ansi")
@@ -39,7 +40,6 @@ teradata_dialect.patch_lexer_matchers(
             "numeric_literal",
             r"([0-9]+(\.[0-9]*)?)",
             CodeSegment,
-            segment_kwargs={"type": "numeric_literal"},
         ),
     ]
 )
@@ -61,6 +61,7 @@ teradata_dialect.sets("unreserved_keywords").update(
         "CASESPECIFIC",
         "CS",
         "DAYS",
+        "DEL",
         "DUAL",
         "ERRORCODE",
         "EXPORT",
@@ -77,15 +78,19 @@ teradata_dialect.sets("unreserved_keywords").update(
         "MAXVALUELENGTH",
         "MEETS",
         "MERGEBLOCKRATIO",
+        "NONE",
         "PERCENT",
         "PROFILE",
         "PROTECTION",
+        "QUERY_BAND",
         "QUIT",
         "RUN",
         "SAMPLE",
         "SEL",
         "SS",
         "STAT",
+        "STATS",
+        "STATISTICS",
         "SUMMARY",
         "THRESHOLD",
         "UC",
@@ -97,6 +102,41 @@ teradata_dialect.sets("reserved_keywords").update(["UNION", "TIMESTAMP"])
 
 teradata_dialect.sets("bare_functions").update(["DATE"])
 
+teradata_dialect.replace(
+    # ANSI standard comparison operators plus Teradata extensions
+    ComparisonOperatorGrammar=OneOf(
+        Ref("EqualsSegment"),
+        Ref("EqualsSegment_a"),
+        Ref("GreaterThanSegment"),
+        Ref("GreaterThanSegment_a"),
+        Ref("LessThanSegment"),
+        Ref("LessThanSegment_a"),
+        Ref("GreaterThanOrEqualToSegment"),
+        Ref("GreaterThanOrEqualToSegment_a"),
+        Ref("LessThanOrEqualToSegment"),
+        Ref("LessThanOrEqualToSegment_a"),
+        Ref("NotEqualToSegment"),
+        Ref("NotEqualToSegment_a"),
+        Ref("NotEqualToSegment_b"),
+        Ref("NotEqualToSegment_c"),
+        Ref("LikeOperatorSegment"),
+        Sequence("IS", "DISTINCT", "FROM"),
+        Sequence("IS", "NOT", "DISTINCT", "FROM"),
+    )
+)
+
+teradata_dialect.add(
+    # Add Teradata comparison operator extensions
+    EqualsSegment_a=StringParser("EQ", ComparisonOperatorSegment),
+    GreaterThanSegment_a=StringParser("GT", ComparisonOperatorSegment),
+    LessThanSegment_a=StringParser("LT", ComparisonOperatorSegment),
+    GreaterThanOrEqualToSegment_a=StringParser("GE", ComparisonOperatorSegment),
+    LessThanOrEqualToSegment_a=StringParser("LE", ComparisonOperatorSegment),
+    NotEqualToSegment_a=StringParser("NE", ComparisonOperatorSegment),
+    NotEqualToSegment_b=StringParser("NOT=", ComparisonOperatorSegment),
+    NotEqualToSegment_c=StringParser("^=", ComparisonOperatorSegment),
+)
+
 
 # BTEQ statement
 class BteqKeyWordSegment(BaseSegment):
@@ -213,7 +253,7 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
     match_grammar = Sequence(
         "COLLECT",
         Ref.keyword("SUMMARY", optional=True),
-        OneOf("STATISTICS", "STAT"),
+        OneOf("STAT", "STATS", "STATISTICS"),
         Sequence(
             "USING",
             Delimited(
@@ -243,7 +283,7 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
                 # COLUMN ...
                 Sequence(
                     "COLUMN",
-                    Bracketed(
+                    OptionallyBracketed(
                         Delimited(
                             OneOf(
                                 Ref("ColumnReferenceSegment"),
@@ -340,6 +380,7 @@ class DatatypeSegment(ansi.DatatypeSegment):
 
     match_grammar = Sequence(
         Ref("DatatypeIdentifierSegment"),
+        Ref("BracketedArguments", optional=True),
         Bracketed(
             OneOf(
                 Delimited(Ref("ExpressionSegment")),
@@ -574,6 +615,13 @@ class TdTableConstraints(BaseSegment):
         ),
         # WITH DATA
         Sequence("WITH", Sequence("NO", optional=True), "DATA"),
+        # AND STATISITCS
+        Sequence(
+            "AND",
+            Sequence("NO", optional=True),
+            OneOf("STAT", "STATS", "STATISTICS"),
+            optional=True,
+        ),
         # ON COMMIT PRESERVE ROWS
         Sequence("ON", "COMMIT", OneOf("PRESERVE", "DELETE"), "ROWS"),
     )
@@ -648,8 +696,7 @@ class FromUpdateClauseSegment(BaseSegment):
     """A `FROM` clause like in `SELECT` but terminated by SET."""
 
     type = "from_in_update_clause"
-    match_grammar = StartsWith("FROM", terminator=Ref.keyword("SET"))
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "FROM",
         Delimited(
             # Optional old school delimited joins
@@ -664,7 +711,7 @@ class StatementSegment(ansi.StatementSegment):
 
     type = "statement"
 
-    parse_grammar = ansi.StatementSegment.parse_grammar.copy(
+    match_grammar = ansi.StatementSegment.match_grammar.copy(
         insert=[
             Ref("TdCollectStatisticsStatementSegment"),
             Ref("BteqStatementSegment"),
@@ -673,46 +720,16 @@ class StatementSegment(ansi.StatementSegment):
             Ref("TdCommentStatementSegment"),
             Ref("DatabaseStatementSegment"),
             Ref("SetSessionStatementSegment"),
+            Ref("SetQueryBandStatementSegment"),
         ],
     )
 
-    match_grammar = ansi.StatementSegment.match_grammar.copy()
-
-
-teradata_dialect.add(
-    TdCastIdentifierGrammar=Sequence(
-        OneOf("DATE", "TIMESTAMP"), Ref("ExpressionSegment")
-    ),
-)
-
-teradata_dialect.replace(
-    SingleIdentifierGrammar=OneOf(
-        Ref("NakedIdentifierSegment"),
-        Ref("QuotedIdentifierSegment"),
-        Ref("TdCastIdentifierGrammar"),
-    ),
-    SelectClauseSegmentGrammar=Sequence(
-        OneOf("SELECT", "SEL"),
-        Ref("SelectClauseModifierSegment", optional=True),
-        Indent,
-        Delimited(
-            Ref("SelectClauseElementSegment"),
-            allow_trailing=True,
-        ),
-    ),
-)
-
 
 class QualifyClauseSegment(BaseSegment):
     """A `QUALIFY` clause like in `SELECT`."""
 
     type = "qualify_clause"
-    match_grammar = StartsWith(
-        "QUALIFY",
-        terminator=OneOf(Sequence("ORDER", "BY"), "LIMIT", "QUALIFY", "WINDOW"),
-        enforce_whitespace_preceding_terminator=True,
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "QUALIFY",
         Indent,
         OptionallyBracketed(Ref("ExpressionSegment")),
@@ -726,8 +743,7 @@ class SelectStatementSegment(ansi.SelectStatementSegment):
     https://dev.mysql.com/doc/refman/5.7/en/select.html
     """
 
-    match_grammar = ansi.SelectStatementSegment.match_grammar
-    parse_grammar = ansi.SelectStatementSegment.parse_grammar.copy(
+    match_grammar = ansi.SelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OrderByClauseSegment", optional=True),
     )
@@ -739,13 +755,50 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     https://dev.mysql.com/doc/refman/5.7/en/select.html
     """
 
-    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar
-    parse_grammar = ansi.UnorderedSelectStatementSegment.parse_grammar.copy(
+    match_grammar = ansi.UnorderedSelectStatementSegment.match_grammar.copy(
         insert=[Ref("QualifyClauseSegment", optional=True)],
         before=Ref("OverlapsClauseSegment", optional=True),
     )
 
 
+class SelectClauseSegment(ansi.SelectClauseSegment):
+    """A group of elements in a select target statement.
+
+    Remove OVERLAPS as a terminator as this can be part of SelectClauseModifierSegment
+    """
+
+    match_grammar = ansi.SelectClauseSegment.match_grammar.copy(
+        # Allow "SEL" as in place of just "SELECT"
+        insert=[OneOf("SELECT", "SEL")],
+        before=Ref.keyword("SELECT"),
+        remove=[Ref.keyword("SELECT")],
+        terminators=[
+            "FROM",
+            "WHERE",
+            Sequence("ORDER", "BY"),
+            "LIMIT",
+            Ref("SetOperatorSegment"),
+        ],
+        replace_terminators=True,
+    )
+
+
+class DeleteStatementSegment(BaseSegment):
+    """A `DELETE` statement.
+
+    DEL[ETE] FROM <table name> [ WHERE <search condition> ]
+    """
+
+    type = "delete_statement"
+    # match grammar. This one makes sense in the context of knowing that it's
+    # definitely a statement, we just don't know what type yet.
+    match_grammar: Matchable = Sequence(
+        OneOf("DELETE", "DEL"),
+        Ref("FromClauseSegment"),
+        Ref("WhereClauseSegment", optional=True),
+    )
+
+
 class SelectClauseModifierSegment(BaseSegment):
     """Things that come after SELECT but before the columns.
 
@@ -757,6 +810,12 @@ class SelectClauseModifierSegment(BaseSegment):
     match_grammar = OneOf(
         "DISTINCT",
         "ALL",
+        Sequence(
+            "TOP",
+            Ref("ExpressionSegment"),
+            Sequence("PERCENT", optional=True),
+            Sequence("WITH", "TIES", optional=True),
+        ),
         Sequence(
             "NORMALIZE",
             OneOf(
@@ -782,28 +841,6 @@ class SelectClauseModifierSegment(BaseSegment):
     )
 
 
-class SelectClauseSegment(ansi.SelectClauseSegment):
-    """A group of elements in a select target statement.
-
-    Remove OVERLAPS as a terminator as this can be part of SelectClauseModifierSegment
-    """
-
-    match_grammar = StartsWith(
-        Sequence(
-            OneOf("SELECT", "SEL"), Ref("WildcardExpressionSegment", optional=True)
-        ),
-        terminator=OneOf(
-            "FROM",
-            "WHERE",
-            Sequence("ORDER", "BY"),
-            "LIMIT",
-            Ref("SetOperatorSegment"),
-        ),
-        enforce_whitespace_preceding_terminator=True,
-    )
-    parse_grammar = ansi.SelectClauseSegment.parse_grammar
-
-
 class DatabaseStatementSegment(BaseSegment):
     """A `DATABASE` statement.
 
@@ -833,3 +870,46 @@ class SetSessionStatementSegment(BaseSegment):
         ),
         Ref("DatabaseStatementSegment"),
     )
+
+
+class SetQueryBandStatementSegment(BaseSegment):
+    """A `SET QUERY_BAND` statement.
+
+    SET QUERY_BAND = { 'band_specification [...]' | NONE } [ UPDATE ]
+    FOR { SESSION [VOLATILE] | TRANSACTION } [;]
+
+    https://docs.teradata.com/r/Teradata-VantageTM-SQL-Data-Definition-Language-Syntax-and-Examples/July-2021/Session-Statements/SET-QUERY_BAND
+    """
+
+    type = "set_query_band_statement"
+    match_grammar: Matchable = Sequence(
+        "SET",
+        "QUERY_BAND",
+        Ref("EqualsSegment"),
+        OneOf(Ref("QuotedLiteralSegment"), "NONE"),
+        Sequence("UPDATE", optional=True),
+        "FOR",
+        OneOf(Sequence("SESSION", Sequence("VOLATILE", optional=True)), "TRANSACTION"),
+    )
+
+
+class NotEqualToSegment_b(CompositeComparisonOperatorSegment):
+    """The comparison operator extension NOT=.
+
+    https://www.docs.teradata.com/r/Teradata-Database-SQL-Functions-Operators-Expressions-and-Predicates/March-2017/Comparison-Operators-and-Functions/Comparison-Operators/Supported-Comparison-Operators
+    """
+
+    match_grammar = Sequence(
+        Ref("NotOperatorGrammar"), Ref("RawEqualsSegment"), allow_gaps=False
+    )
+
+
+class NotEqualToSegment_c(CompositeComparisonOperatorSegment):
+    """The comparison operator extension ^=.
+
+    https://www.docs.teradata.com/r/Teradata-Database-SQL-Functions-Operators-Expressions-and-Predicates/March-2017/Comparison-Operators-and-Functions/Comparison-Operators/Supported-Comparison-Operators
+    """
+
+    match_grammar = Sequence(
+        Ref("BitwiseXorSegment"), Ref("RawEqualsSegment"), allow_gaps=False
+    )
