@@ -2,16 +2,16 @@
 
 import logging
 from dataclasses import dataclass
-
-from typing import List, Type, Tuple, cast
+from typing import List, Tuple, Type, cast
 
 from sqlfluff.core.parser import BaseSegment
 from sqlfluff.core.rules import LintFix, LintResult
-
 from sqlfluff.utils.reflow.elements import ReflowBlock, ReflowPoint, ReflowSequenceType
-from sqlfluff.utils.reflow.reindent import deduce_line_indent
-from sqlfluff.utils.reflow.helpers import fixes_from_results, pretty_segment_name
-
+from sqlfluff.utils.reflow.helpers import (
+    deduce_line_indent,
+    fixes_from_results,
+    pretty_segment_name,
+)
 
 # We're in the utils module, but users will expect reflow
 # logs to appear in the context of rules. Hence it's a subset
@@ -163,6 +163,20 @@ def identify_rebreak_spans(
             continue
         # Does the element itself have config? (The easy case)
         if elem.line_position:
+            # We should check whether this is a valid place to break based
+            # on whether it's in a templated tag. If it's not a literal, then skip
+            # it.
+            # TODO: We probably only care if the side of the element that we would
+            # break at (i.e. the start if it's `leading` or the end if it's
+            # `trailing`), but we'll go with the blunt logic for simplicity first.
+            if not elem.segments[0].pos_marker.is_literal():
+                reflow_logger.debug(
+                    "        ! Skipping rebreak span on %s because "
+                    "non-literal location.",
+                    elem.segments[0],
+                )
+                continue
+
             # Blocks should only have one segment so it's easy to pick it.
             spans.append(
                 _RebreakSpan(
@@ -182,11 +196,25 @@ def identify_rebreak_spans(
             if elem.depth_info.stack_positions[key].idx != 0:
                 continue
             # Can we find the end?
-            for end_idx in range(idx, len(element_buffer) - 2):
+            # NOTE: It's safe to look right to the end here rather than up to
+            # -2 because we're going to end up stepping back by two in the
+            # complicated cases.
+            for end_idx in range(idx, len(element_buffer)):
                 end_elem = element_buffer[end_idx]
+                final_idx = None
+
                 if not isinstance(end_elem, ReflowBlock):
                     continue
-                if end_elem.depth_info.stack_positions[key].type in ("end", "solo"):
+                elif key not in end_elem.depth_info.stack_positions:
+                    # If we get here, it means the last block was the end.
+                    # NOTE: This feels a little hacky, but it's because of a limitation
+                    # in detecting the "end" and "solo" markers effectively in larger
+                    # sections.
+                    final_idx = end_idx - 2  # pragma: no cover
+                elif end_elem.depth_info.stack_positions[key].type in ("end", "solo"):
+                    final_idx = end_idx
+
+                if final_idx is not None:
                     # Found the end. Add it to the stack.
                     # We reference the appropriate element from the parent stack.
                     target_depth = elem.depth_info.stack_hashes.index(key)
@@ -197,7 +225,7 @@ def identify_rebreak_spans(
                         _RebreakSpan(
                             target,
                             idx,
-                            end_idx,
+                            final_idx,
                             # NOTE: this isn't pretty but until it needs to be more
                             # complex, this works.
                             elem.line_position_configs[key].split(":")[0],
@@ -205,6 +233,7 @@ def identify_rebreak_spans(
                         )
                     )
                     break
+
             # If we find the start, but not the end, it's not a problem, but
             # we won't be rebreaking this span. This is important so that we
             # don't rebreak part of something without the context of what's
@@ -243,7 +272,16 @@ def rebreak_sequence(
     # to handle comments differently. There are two other important points:
     # 1. The next newline outward before code (but passing over comments).
     # 2. The point before the next _code_ segment (ditto comments).
-    locations = [_RebreakLocation.from_span(span, elem_buff) for span in spans]
+    locations = []
+    for span in spans:
+        try:
+            locations.append(_RebreakLocation.from_span(span, elem_buff))
+        # If we try and create a location from an incomplete span (i.e. one
+        # where we're unable to find the next newline effectively), then
+        # we'll get an exception. If we do - skip that one - we won't be
+        # able to effectively work with it even if we could construct it.
+        except UnboundLocalError:
+            pass
 
     # Handle each span:
     for loc in locations:
@@ -333,9 +371,24 @@ def rebreak_sequence(
                     lint_results=[],
                     anchor_on="after",
                 )
+
+                # Handle the potential case of an empty point.
+                # https://github.com/sqlfluff/sqlfluff/issues/4184
+                for i in range(loc.next.pre_code_pt_idx):
+                    if elem_buff[loc.next.pre_code_pt_idx - i].segments:
+                        create_anchor = elem_buff[
+                            loc.next.pre_code_pt_idx - i
+                        ].segments[-1]
+                        break
+                else:  # pragma: no cover
+                    # NOTE: We don't test this because we *should* always find
+                    # _something_ to anchor the creation on, even if we're
+                    # unlucky enough not to find it on the first pass.
+                    raise NotImplementedError("Could not find anchor for creation.")
+
                 fixes.append(
                     LintFix.create_after(
-                        elem_buff[loc.next.pre_code_pt_idx].segments[-1],
+                        create_anchor,
                         [loc.target],
                     )
                 )
