@@ -1,18 +1,27 @@
 """Dataclasses for reflow work."""
 
-from itertools import chain
 import logging
-from typing import Iterator, List, Optional, Sequence, Tuple, cast, Type
-from sqlfluff.core.config import FluffConfig
+from itertools import chain
+from typing import Iterator, List, Optional, Sequence, Tuple, Type, cast
 
+from sqlfluff.core.config import FluffConfig
 from sqlfluff.core.parser import BaseSegment, RawSegment
 from sqlfluff.core.rules.base import LintFix, LintResult
 from sqlfluff.utils.reflow.config import ReflowConfig
 from sqlfluff.utils.reflow.depthmap import DepthMap
-
-from sqlfluff.utils.reflow.elements import ReflowBlock, ReflowPoint, ReflowSequenceType
-from sqlfluff.utils.reflow.rebreak import rebreak_sequence
+from sqlfluff.utils.reflow.elements import (
+    ReflowBlock,
+    ReflowPoint,
+    ReflowSequenceType,
+    get_consumed_whitespace,
+)
 from sqlfluff.utils.reflow.helpers import fixes_from_results
+from sqlfluff.utils.reflow.rebreak import rebreak_sequence
+from sqlfluff.utils.reflow.reindent import (
+    construct_single_indent,
+    lint_indent_points,
+    lint_line_length,
+)
 
 # We're in the utils module, but users will expect reflow
 # logs to appear in the context of rules. Hence it's a subset
@@ -93,8 +102,11 @@ class ReflowSequence:
         return "".join(elem.raw for elem in self.elements)
 
     @staticmethod
-    def _validate_reflow_sequence(elements: ReflowSequenceType):
-        assert elements, "ReflowSequence has empty elements."
+    def _validate_reflow_sequence(elements: ReflowSequenceType) -> None:
+        # An empty set of elements _is_ allowed as an edge case.
+        if not elements:
+            # Return early if so
+            return None
         # Check odds and evens
         OddType = elements[0].__class__
         EvenType = ReflowPoint if OddType is ReflowBlock else ReflowBlock
@@ -107,6 +119,7 @@ class ReflowSequence:
             assert all(
                 isinstance(elem, EvenType) for elem in elements[1::2]
             ), f"Not all even elements are {EvenType.__name__}"
+            return None
         except AssertionError as err:  # pragma: no cover
             for elem in elements:
                 reflow_logger.error("   - %s", elem)
@@ -127,7 +140,12 @@ class ReflowSequence:
         for seg in segments:
             # NOTE: end_of_file is block-like rather than point-like.
             # This is to facilitate better evaluation of the ends of files.
-            if seg.is_type("whitespace", "newline", "indent"):
+            # NOTE: This also allows us to include literal placeholders for
+            # whitespace only strings.
+            if (
+                seg.is_type("whitespace", "newline", "indent")
+                or (get_consumed_whitespace(seg) or "").isspace()
+            ):
                 # Add to the buffer and move on.
                 seg_buff.append(seg)
                 continue
@@ -500,9 +518,6 @@ class ReflowSequence:
                 new_point = point
             # Otherwise apply the new fixes
             else:
-                reflow_logger.debug(
-                    "    Filter %r allows fixes for point: %s", filter, new_lint_results
-                )
                 lint_results = new_lint_results
 
             if pre and (not new_elements or new_elements[-1] != pre):
@@ -546,3 +561,65 @@ class ReflowSequence:
             depth_map=self.depth_map,
             lint_results=lint_results,
         )
+
+    def reindent(self) -> "ReflowSequence":
+        """Reindent lines within a sequence."""
+        if self.lint_results:
+            raise NotImplementedError(  # pragma: no cover
+                "rebreak cannot currently handle pre-existing embodied fixes."
+            )
+
+        single_indent = construct_single_indent(
+            indent_unit=self.reflow_config.indent_unit,
+            tab_space_size=self.reflow_config.tab_space_size,
+        )
+
+        reflow_logger.info("# Evaluating indents.")
+        elements, indent_results = lint_indent_points(
+            self.elements,
+            single_indent=single_indent,
+            skip_indentation_in=self.reflow_config.skip_indentation_in,
+            allow_implicit_indents=self.reflow_config.allow_implicit_indents,
+        )
+
+        return ReflowSequence(
+            elements=elements,
+            root_segment=self.root_segment,
+            reflow_config=self.reflow_config,
+            depth_map=self.depth_map,
+            lint_results=indent_results,
+        )
+
+    def break_long_lines(self):
+        """Rebreak any remaining long lines in a sequence.
+
+        This assumes that reindent() has already been applied.
+        """
+        if self.lint_results:
+            raise NotImplementedError(  # pragma: no cover
+                "break_long_lines cannot currently handle pre-existing "
+                "embodied fixes."
+            )
+
+        single_indent = construct_single_indent(
+            indent_unit=self.reflow_config.indent_unit,
+            tab_space_size=self.reflow_config.tab_space_size,
+        )
+
+        reflow_logger.info("# Evaluating line lengths.")
+        elements, length_results = lint_line_length(
+            self.elements,
+            self.root_segment,
+            single_indent=single_indent,
+            line_length_limit=self.reflow_config.max_line_length,
+            allow_implicit_indents=self.reflow_config.allow_implicit_indents,
+            trailing_comments=self.reflow_config.trailing_comments,
+        )
+
+        return ReflowSequence(
+            elements=elements,
+            root_segment=self.root_segment,
+            reflow_config=self.reflow_config,
+            depth_map=self.depth_map,
+            lint_results=length_results,
+        )
