@@ -8,7 +8,8 @@ from sqlfluff.cli.autocomplete import dialect_shell_complete
     "incomplete,expected",
     [
         ["an", ["ansi"]],
-        ["d", ["databricks", "db2"]],
+        ["d", ["databricks", "db2", "duckdb"]],
+        ["g", ["greenplum"]],
         ["s", ["snowflake", "soql", "sparksql", "sqlite"]],
         ["post", ["postgres"]],
     ],
