@@ -2,6 +2,7 @@
 
 import configparser
 import json
+import logging
 import os
 import pathlib
 import re
@@ -11,7 +12,6 @@ import subprocess
 import sys
 import tempfile
 import textwrap
-import logging
 from unittest.mock import MagicMock, patch
 
 import chardet
@@ -24,17 +24,18 @@ from click.testing import CliRunner
 # We import the library directly here to get the version
 import sqlfluff
 from sqlfluff.cli.commands import (
-    lint,
-    version,
-    rules,
-    fix,
-    parse,
+    cli_format,
     dialects,
+    fix,
     get_config,
+    lint,
+    parse,
     render,
+    rules,
+    version,
 )
+from sqlfluff.core.parser import CommentSegment
 from sqlfluff.core.rules import BaseRule, LintFix, LintResult
-from sqlfluff.core.parser.segments.raw import CommentSegment
 from sqlfluff.utils.testing.cli import invoke_assert_code
 
 re_ansi_escape = re.compile(r"\x1b[^m]*m")
@@ -73,10 +74,9 @@ def contains_ansi_escape(s: str) -> bool:
 
 
 expected_output = """== [test/fixtures/linter/indentation_error_simple.sql] FAIL
-L:   2 | P:   4 | L003 | Expected 1 indentation, found less than 1 [compared to
-                       | line 01]
-L:   5 | P:  10 | L010 | Keywords must be consistently upper case.
-L:   5 | P:  13 | L031 | Avoid aliases in from clauses and join conditions.
+L:   2 | P:   1 | LT02 | Expected indent of 4 spaces. [layout.indent]
+L:   5 | P:  10 | CP01 | Keywords must be consistently upper case.
+                       | [capitalisation.keywords]
 """
 
 
@@ -93,7 +93,7 @@ def test__cli__command_directed():
         ],
     )
     # We should get a readout of what the error was
-    check_a = "L:   2 | P:   4 | L003"
+    check_a = "L:   2 | P:   1 | LT02"
     # NB: Skip the number at the end because it's configurable
     check_b = "ndentation"
     assert check_a in result.output
@@ -251,6 +251,14 @@ def test__cli__command_lint_stdin(command):
     invoke_assert_code(args=[lint, ("--dialect=ansi",) + command], cli_input=sql)
 
 
+def test__cli__command_lint_empty_stdin():
+    """Check linting an empty file raises no exceptions.
+
+    https://github.com/sqlfluff/sqlfluff/issues/4807
+    """
+    invoke_assert_code(args=[lint, ("-d", "ansi", "-")], cli_input="")
+
+
 def test__cli__command_render_stdin():
     """Check render on a simple script using stdin."""
     with open("test/fixtures/cli/passing_a.sql") as test_file:
@@ -270,7 +278,7 @@ def test__cli__command_render_stdin():
                 "-n",
                 "test/fixtures/cli/passing_b.sql",
                 "--exclude-rules",
-                "L051",
+                "AM05",
             ],
         ),
         # Basic render
@@ -293,7 +301,7 @@ def test__cli__command_render_stdin():
                 "test/fixtures/cli/passing_b.sql",
                 "-vvvvvvvvvvv",
                 "--exclude-rules",
-                "L051",
+                "AM05",
             ],
         ),
         # Test basic linting with specific logger.
@@ -307,7 +315,7 @@ def test__cli__command_render_stdin():
                 "--logger",
                 "parser",
                 "-e",
-                "L051",
+                "AM05",
             ],
         ),
         # Check basic parsing
@@ -317,7 +325,7 @@ def test__cli__command_render_stdin():
                 "-n",
                 "test/fixtures/cli/passing_b.sql",
                 "--exclude-rules",
-                "L051",
+                "AM05",
             ],
         ),
         # Test basic parsing with very high verbosity
@@ -328,7 +336,7 @@ def test__cli__command_render_stdin():
                 "test/fixtures/cli/passing_b.sql",
                 "-vvvvvvvvvvv",
                 "-e",
-                "L051",
+                "AM05",
             ],
         ),
         # Check basic parsing, with the code only option
@@ -336,8 +344,9 @@ def test__cli__command_render_stdin():
         # Check basic parsing, with the yaml output
         (parse, ["-n", "test/fixtures/cli/passing_b.sql", "-c", "-f", "yaml"]),
         (parse, ["-n", "test/fixtures/cli/passing_b.sql", "--format", "yaml"]),
-        # Check the profiler and benching commands
-        (parse, ["-n", "test/fixtures/cli/passing_b.sql", "--profiler"]),
+        # Check parsing with no output (used mostly for testing)
+        (parse, ["-n", "test/fixtures/cli/passing_b.sql", "--format", "none"]),
+        # Check the benching commands
         (parse, ["-n", "test/fixtures/cli/passing_b.sql", "--bench"]),
         (
             lint,
@@ -346,7 +355,7 @@ def test__cli__command_render_stdin():
                 "test/fixtures/cli/passing_b.sql",
                 "--bench",
                 "--exclude-rules",
-                "L051",
+                "AM05",
             ],
         ),
         (
@@ -356,7 +365,7 @@ def test__cli__command_render_stdin():
                 "test/fixtures/cli/passing_b.sql",
                 "--bench",
                 "--exclude-rules",
-                "L051",
+                "AM05",
             ],
         ),
         # Check linting works in specifying rules
@@ -365,17 +374,31 @@ def test__cli__command_render_stdin():
             [
                 "-n",
                 "--rules",
-                "L001",
+                "CP01",
                 "test/fixtures/linter/operator_errors.sql",
             ],
         ),
+        # Check ignoring linting (multiprocess)
+        # https://github.com/sqlfluff/sqlfluff/issues/5066
+        (
+            lint,
+            [
+                "-n",
+                "--ignore",
+                "linting",
+                "-p",
+                "2",
+                "test/fixtures/linter/operator_errors.sql",
+                "test/fixtures/linter/comma_errors.sql",
+            ],
+        ),
         # Check linting works in specifying multiple rules
         (
             lint,
             [
                 "-n",
                 "--rules",
-                "L001,L002",
+                "CP01,LT02",
                 "test/fixtures/linter/operator_errors.sql",
             ],
         ),
@@ -385,9 +408,9 @@ def test__cli__command_render_stdin():
             [
                 "-n",
                 "--rules",
-                "L001,L006",
+                "CP01,LT01",
                 "--exclude-rules",
-                "L006,L031",
+                "LT01,AL07",
                 "test/fixtures/linter/operator_errors.sql",
             ],
         ),
@@ -397,7 +420,7 @@ def test__cli__command_render_stdin():
             [
                 "-n",
                 "--exclude-rules",
-                "L006,L007,L031,L039,L071",
+                "LT01,LT03,AL07",
                 "test/fixtures/linter/operator_errors.sql",
             ],
         ),
@@ -407,7 +430,7 @@ def test__cli__command_render_stdin():
             [
                 "-n",
                 "--exclude-rules",
-                "L003,L009,L031",
+                "LT02,LT12,AL07",
                 "--ignore",
                 "parsing,lexing",
                 "test/fixtures/linter/parse_lex_error.sql",
@@ -434,6 +457,9 @@ def test__cli__command_render_stdin():
         ),
         # Check timing outputs doesn't raise exceptions
         (lint, ["test/fixtures/cli/passing_a.sql", "--persist-timing", "test.csv"]),
+        # Check lint --help command doesn't raise exception.
+        # NOTE: This tests the LazySequence in action.
+        (lint, ["--help"]),
     ],
 )
 def test__cli__command_lint_parse(command):
@@ -451,7 +477,7 @@ def test__cli__command_lint_parse(command):
                 fix,
                 [
                     "--rules",
-                    "L001",
+                    "LT01",
                     "test/fixtures/cli/fail_many.sql",
                     "-vvvvvvv",
                 ],
@@ -465,7 +491,7 @@ def test__cli__command_lint_parse(command):
                 fix,
                 [
                     "--rules",
-                    "L001",
+                    "LT01",
                     "--fixed-suffix",
                     "_fix",
                     "test/fixtures/cli/fail_many.sql",
@@ -487,6 +513,46 @@ def test__cli__command_lint_parse(command):
             ),
             1,
         ),
+        # Format
+        (
+            (
+                cli_format,
+                [
+                    "--fixed-suffix",
+                    "_fix",
+                    "test/fixtures/linter/whitespace_errors.sql",
+                ],
+            ),
+            0,
+        ),
+        # Format with --persist-timing
+        (
+            (
+                cli_format,
+                [
+                    "--fixed-suffix",
+                    "_fix",
+                    "test/fixtures/linter/whitespace_errors.sql",
+                    "--persist-timing",
+                    "test.csv",
+                ],
+            ),
+            0,
+        ),
+        # Format (specifying rules)
+        (
+            (
+                cli_format,
+                [
+                    "--rules",
+                    "LT01",
+                    "--fixed-suffix",
+                    "_fix",
+                    "test/fixtures/linter/whitespace_errors.sql",
+                ],
+            ),
+            2,
+        ),
         # Template syntax error in macro file
         (
             (
@@ -495,6 +561,29 @@ def test__cli__command_lint_parse(command):
             ),
             1,
         ),
+        # Test overriding library path when it doesn't cause an issue
+        (
+            (
+                lint,
+                ["test/fixtures/cli/passing_a.sql", "--library-path", "none"],
+            ),
+            0,
+        ),
+        # Test overriding library path when it DOES cause an issue
+        # (because macros won't be found).
+        (
+            (
+                # Render because that's the step where the issue will
+                # occur.
+                render,
+                [
+                    "test/fixtures/templater/jinja_r_library_in_macro/jinja.sql",
+                    "--library-path",
+                    "none",
+                ],
+            ),
+            1,
+        ),
         # Test render fail
         (
             (
@@ -503,6 +592,30 @@ def test__cli__command_lint_parse(command):
             ),
             1,
         ),
+        # Test a longer lint fail with --bench
+        # This tests the threshold rules clause
+        (
+            (
+                lint,
+                [
+                    "test/fixtures/linter/autofix/bigquery/004_templating/before.sql",
+                    "--bench",
+                ],
+            ),
+            1,
+        ),
+        # Test that setting --quiet with --verbose raises an error.
+        (
+            (
+                fix,
+                [
+                    "--quiet",
+                    "--verbose",
+                    "test/fixtures/cli/fail_many.sql",
+                ],
+            ),
+            2,
+        ),
     ],
 )
 def test__cli__command_lint_parse_with_retcode(command, ret_code):
@@ -534,14 +647,14 @@ def test__cli__command_lint_skip_ignore_files():
         ],
     )
     assert result.exit_code == 1
-    assert "L009" in result.output.strip()
+    assert "LT12" in result.output.strip()
 
 
 def test__cli__command_lint_ignore_local_config():
     """Test that --ignore-local_config ignores .sqlfluff file as expected."""
     runner = CliRunner()
     # First we test that not including the --ignore-local-config includes
-    # .sqlfluff file, and therefore the lint doesn't raise L012
+    # .sqlfluff file, and therefore the lint doesn't raise AL02
     result = runner.invoke(
         lint,
         [
@@ -549,9 +662,9 @@ def test__cli__command_lint_ignore_local_config():
         ],
     )
     assert result.exit_code == 0
-    assert "L012" not in result.output.strip()
+    assert "AL02" not in result.output.strip()
     # Then repeat the same lint but this time ignoring the .sqlfluff file.
-    # We should see L012 raised.
+    # We should see AL02 raised.
     result = runner.invoke(
         lint,
         [
@@ -561,7 +674,7 @@ def test__cli__command_lint_ignore_local_config():
         ],
     )
     assert result.exit_code == 1
-    assert "L012" in result.output.strip()
+    assert "AL02" in result.output.strip()
 
 
 def test__cli__command_lint_warning():
@@ -586,7 +699,7 @@ def test__cli__command_lint_warning():
     # But should also contain the warnings.
     # NOTE: Not including the whole description because it's too long.
     assert (
-        "L:   4 | P:   9 | L006 | WARNING: Expected single whitespace"
+        "L:   4 | P:   9 | LT01 | WARNING: Expected single whitespace"
         in result.output.strip()
     )
 
@@ -693,11 +806,11 @@ def generic_roundtrip_test(
 @pytest.mark.parametrize(
     "rule,fname",
     [
-        ("L001", "test/fixtures/linter/indentation_errors.sql"),
-        ("L008", "test/fixtures/linter/whitespace_errors.sql"),
-        ("L008", "test/fixtures/linter/indentation_errors.sql"),
+        ("LT01", "test/fixtures/linter/indentation_errors.sql"),
+        ("LT01", "test/fixtures/linter/whitespace_errors.sql"),
+        ("LT01", "test/fixtures/linter/indentation_errors.sql"),
         # Really stretching the ability of the fixer to re-indent a file
-        ("L003", "test/fixtures/linter/indentation_error_hard.sql"),
+        ("LT02", "test/fixtures/linter/indentation_error_hard.sql"),
     ],
 )
 def test__cli__command__fix(rule, fname):
@@ -717,7 +830,7 @@ def test__cli__command__fix(rule, fname):
             FROM my_schema.my_table
             where processdate ! 3
             """,
-            ["--force", "--fixed-suffix", "FIXED", "--rules", "L010"],
+            ["--force", "--fixed-suffix", "FIXED", "--rules", "CP01"],
             None,
             1,
         ),
@@ -730,7 +843,7 @@ def test__cli__command__fix(rule, fname):
             where processdate {{ condition }}
             """,
             # Test the short versions of the options.
-            ["--force", "-x", "FIXED", "-r", "L010"],
+            ["--force", "-x", "FIXED", "-r", "CP01"],
             None,
             1,
         ),
@@ -744,7 +857,7 @@ def test__cli__command__fix(rule, fname):
             where processdate ! 3  -- noqa: PRS
             """,
             # Test the short versions of the options.
-            ["--force", "-x", "FIXED", "-r", "L010"],
+            ["--force", "-x", "FIXED", "-r", "CP01"],
             None,
             1,
         ),
@@ -756,7 +869,7 @@ def test__cli__command__fix(rule, fname):
             FROM my_schema.my_table
             WHERE processdate ! 3
             """,
-            ["--force", "--fixed-suffix", "FIXED", "--rules", "L010"],
+            ["--force", "--fixed-suffix", "FIXED", "--rules", "CP01"],
             None,
             1,
         ),
@@ -768,7 +881,7 @@ def test__cli__command__fix(rule, fname):
             FROM my_schema.my_table
             WHERE processdate ! 3  --noqa: PRS
             """,
-            ["--force", "--fixed-suffix", "FIXED", "--rules", "L010"],
+            ["--force", "--fixed-suffix", "FIXED", "--rules", "CP01"],
             None,
             0,
         ),
@@ -786,7 +899,7 @@ def test__cli__command__fix(rule, fname):
                 "--fixed-suffix",
                 "FIXED",
                 "--rules",
-                "L010",
+                "CP01",
                 "--FIX-EVEN-UNPARSABLE",
             ],
             """
@@ -816,7 +929,7 @@ def test__cli__command__fix(rule, fname):
                 FROM my_schema.my_table
                 where processdate != 3""",
             ],
-            ["--force", "--fixed-suffix", "FIXED", "--rules", "L010"],
+            ["--force", "--fixed-suffix", "FIXED", "--rules", "CP01"],
             [
                 None,
                 """SELECT my_col
@@ -845,7 +958,7 @@ def test__cli__fix_error_handling_behavior(sql, fix_args, fixed, exit_code, tmpd
     assert len(sql) == len(fixed)
     tmp_path = pathlib.Path(str(tmpdir))
     for idx, this_sql in enumerate(sql):
-        filepath = tmp_path / f"testing{idx+1}.sql"
+        filepath = tmp_path / f"testing{idx + 1}.sql"
         filepath.write_text(textwrap.dedent(this_sql))
     with tmpdir.as_cwd():
         with pytest.raises(SystemExit) as e:
@@ -860,7 +973,7 @@ def test__cli__fix_error_handling_behavior(sql, fix_args, fixed, exit_code, tmpd
             )
         assert exit_code == e.value.code
     for idx, this_fixed in enumerate(fixed):
-        fixed_path = tmp_path / f"testing{idx+1}FIXED.sql"
+        fixed_path = tmp_path / f"testing{idx + 1}FIXED.sql"
         if this_fixed is not None:
             assert textwrap.dedent(this_fixed) == fixed_path.read_text()
         else:
@@ -964,10 +1077,10 @@ def _mock_eval(rule, context):
         ("-- noqa: disable=all\n-- Comment A\nSELECT 1 FROM foo", 0),
     ],
 )
-@patch("sqlfluff.rules.L001.Rule_L001._eval", _mock_eval)
+@patch("sqlfluff.rules.layout.LT01.Rule_LT01._eval", _mock_eval)
 def test__cli__fix_loop_limit_behavior(sql, exit_code, tmpdir):
     """Tests how "fix" behaves when the loop limit is exceeded."""
-    fix_args = ["--force", "--fixed-suffix", "FIXED", "--rules", "L001"]
+    fix_args = ["--force", "--fixed-suffix", "FIXED", "--rules", "LT01"]
     tmp_path = pathlib.Path(str(tmpdir))
     filepath = tmp_path / "testing.sql"
     filepath.write_text(textwrap.dedent(sql))
@@ -989,39 +1102,15 @@ def test__cli__fix_loop_limit_behavior(sql, exit_code, tmpdir):
     assert not fixed_path.is_file()
 
 
-# Test case disabled because there isn't a good example of where to test this.
-# This *should* test the case where a rule DOES have a proposed fix, but for
-# some reason when we try to apply it, there's a failure.
-# @pytest.mark.parametrize('rule,fname', [
-#     # NB: L004 currently has no fix routine.
-#     ('L004', 'test/fixtures/linter/indentation_errors.sql')
-# ])
-# def test__cli__command__fix_fail(rule, fname):
-#     """Test the round trip of detecting, fixing and then still detecting the rule."""
-#     with open(fname, mode='r') as test_file:
-#         generic_roundtrip_test(test_file, rule, fix_exit_code=1, final_exit_code=65)
-
-
 @pytest.mark.parametrize(
     "stdin,rules,stdout",
     [
-        ("select * from t", "L003", "select * from t"),  # no change
+        ("select * from t", "LT02", "select * from t"),  # no change
         (
             " select * from t",
-            "L003",
+            "LT02",
             "select * from t",
         ),  # fix preceding whitespace
-        # L031 fix aliases in joins
-        (
-            "SELECT u.id, c.first_name, c.last_name, COUNT(o.user_id) "
-            "FROM users as u JOIN customers as c on u.id = c.user_id JOIN orders as o "
-            "on u.id = o.user_id;",
-            "L031",
-            "SELECT users.id, customers.first_name, customers.last_name, "
-            "COUNT(orders.user_id) "
-            "FROM users JOIN customers on users.id = customers.user_id JOIN orders on "
-            "users.id = orders.user_id;",
-        ),
     ],
 )
 def test__cli__command_fix_stdin(stdin, rules, stdout):
@@ -1036,6 +1125,28 @@ def test__cli__command_fix_stdin(stdin, rules, stdout):
     assert result.output == stdout
 
 
+@pytest.mark.parametrize(
+    "stdin,stdout",
+    [
+        ("select * from t\n", "select * from t\n"),  # no change
+        (
+            "   select    *    FRoM     t    ",
+            "select * from t\n",
+        ),
+    ],
+)
+def test__cli__command_format_stdin(stdin, stdout):
+    """Check stdin input for fix works."""
+    result = invoke_assert_code(
+        args=[
+            cli_format,
+            ("-", "--disable-progress-bar", "--dialect=ansi"),
+        ],
+        cli_input=stdin,
+    )
+    assert result.output == stdout
+
+
 def test__cli__command_fix_stdin_logging_to_stderr(monkeypatch):
     """Check that logging goes to stderr when stdin is passed to fix."""
     perfect_sql = "select col from table"
@@ -1048,7 +1159,7 @@ def test__cli__command_fix_stdin_logging_to_stderr(monkeypatch):
 
     monkeypatch.setattr(sqlfluff.cli.commands, "Linter", MockLinter)
     result = invoke_assert_code(
-        args=[fix, ("-", "--rules=L003", "--dialect=ansi")],
+        args=[fix, ("-", "--rules=LT02", "--dialect=ansi")],
         cli_input=perfect_sql,
         mix_stderr=False,
     )
@@ -1110,8 +1221,8 @@ def test__cli__command_fix_stdin_error_exit_code(
 @pytest.mark.parametrize(
     "rule,fname,prompt,exit_code,fix_exit_code",
     [
-        ("L001", "test/fixtures/linter/indentation_errors.sql", "y", 0, 0),
-        ("L001", "test/fixtures/linter/indentation_errors.sql", "n", 1, 1),
+        ("LT01", "test/fixtures/linter/indentation_errors.sql", "y", 0, 0),
+        ("LT01", "test/fixtures/linter/indentation_errors.sql", "n", 1, 1),
     ],
 )
 def test__cli__command__fix_no_force(rule, fname, prompt, exit_code, fix_exit_code):
@@ -1163,7 +1274,7 @@ def test__cli__command_parse_serialize_from_stdin(serialize, write_file, tmp_pat
     assert result["filepath"] == "stdin"
 
 
-@pytest.mark.parametrize("serialize", ["yaml", "json"])
+@pytest.mark.parametrize("serialize", ["yaml", "json", "none"])
 @pytest.mark.parametrize(
     "sql,expected,exit_code",
     [
@@ -1175,16 +1286,18 @@ def test__cli__command_parse_serialize_from_stdin(serialize, write_file, tmp_pat
                     "filepath": "stdin",
                     "violations": [
                         {
-                            "code": "L010",
+                            "code": "CP01",
                             "line_no": 1,
                             "line_pos": 1,
                             "description": "Keywords must be consistently upper case.",
+                            "name": "capitalisation.keywords",
                         },
                         {
-                            "code": "L010",
+                            "code": "CP01",
                             "line_no": 1,
                             "line_pos": 10,
                             "description": "Keywords must be consistently upper case.",
+                            "name": "capitalisation.keywords",
                         },
                     ],
                 }
@@ -1201,7 +1314,7 @@ def test__cli__command_lint_serialize_from_stdin(serialize, sql, expected, exit_
             (
                 "-",
                 "--rules",
-                "L010",
+                "CP01",
                 "--format",
                 serialize,
                 "--disable-progress-bar",
@@ -1216,6 +1329,8 @@ def test__cli__command_lint_serialize_from_stdin(serialize, sql, expected, exit_
         assert json.loads(result.output) == expected
     elif serialize == "yaml":
         assert yaml.safe_load(result.output) == expected
+    elif serialize == "none":
+        assert result.output == ""
     else:
         raise Exception
 
@@ -1268,7 +1383,7 @@ def test__cli__command_lint_nocolor(isatty, should_strip_ansi, capsys, tmpdir):
 
 @pytest.mark.parametrize(
     "serialize",
-    ["human", "yaml", "json", "github-annotation", "github-annotation-native"],
+    ["human", "yaml", "json", "github-annotation", "github-annotation-native", "none"],
 )
 @pytest.mark.parametrize("write_file", [None, "outfile"])
 def test__cli__command_lint_serialize_multiple_files(serialize, write_file, tmp_path):
@@ -1301,7 +1416,8 @@ def test__cli__command_lint_serialize_multiple_files(serialize, write_file, tmp_
         ret_code=1,
     )
 
-    if write_file:
+    # NOTE: The "none" serializer doesn't write a file even if specified.
+    if write_file and serialize != "none":
         with open(target_file, "r") as payload_file:
             result_payload = payload_file.read()
     else:
@@ -1309,12 +1425,15 @@ def test__cli__command_lint_serialize_multiple_files(serialize, write_file, tmp_
 
     # Print for debugging.
     payload_length = len(result_payload.split("\n"))
-    print(f"## Payload (length {payload_length}):")
+    print("=== BEGIN RESULT OUTPUT")
     print(result_payload)
-    print("## End Payload")
+    print("=== END RESULT OUTPUT")
+    print("Result length:", payload_length)
 
     if serialize == "human":
-        assert payload_length == 26 if write_file else 32
+        assert payload_length == 25 if write_file else 34
+    elif serialize == "none":
+        assert payload_length == 1  # There will be a single newline.
     elif serialize == "json":
         result = json.loads(result_payload)
         assert len(result) == 2
@@ -1330,7 +1449,7 @@ def test__cli__command_lint_serialize_multiple_files(serialize, write_file, tmp_
         # SQLFluff produces trailing newline
         if result[-1] == "":
             del result[-1]
-        assert len(result) == 17
+        assert len(result) == 12
     else:
         raise Exception
 
@@ -1354,19 +1473,6 @@ def test__cli__command_lint_serialize_github_annotation():
     )
     result = json.loads(result.output)
     assert result == [
-        {
-            "annotation_level": "warning",
-            # Normalise paths to control for OS variance
-            "file": os.path.normpath(
-                "test/fixtures/linter/identifier_capitalisation.sql"
-            ),
-            "line": 1,
-            "message": "L036: Select targets should be on a new line unless there is "
-            "only one select target.",
-            "start_column": 1,
-            "end_column": 1,
-            "title": "SQLFluff",
-        },
         {
             "annotation_level": "warning",
             # Normalise paths to control for OS variance
@@ -1374,7 +1480,7 @@ def test__cli__command_lint_serialize_github_annotation():
                 "test/fixtures/linter/identifier_capitalisation.sql"
             ),
             "line": 2,
-            "message": "L027: Unqualified reference 'foo' found in select with more "
+            "message": "RF02: Unqualified reference 'foo' found in select with more "
             "than one referenced table/view.",
             "start_column": 5,
             "end_column": 5,
@@ -1387,7 +1493,19 @@ def test__cli__command_lint_serialize_github_annotation():
                 "test/fixtures/linter/identifier_capitalisation.sql"
             ),
             "line": 3,
-            "message": "L012: Implicit/explicit aliasing of columns.",
+            "message": "LT02: Expected indent of 8 spaces.",
+            "start_column": 1,
+            "end_column": 1,
+            "title": "SQLFluff",
+        },
+        {
+            "annotation_level": "warning",
+            # Normalise paths to control for OS variance
+            "file": os.path.normpath(
+                "test/fixtures/linter/identifier_capitalisation.sql"
+            ),
+            "line": 3,
+            "message": "AL02: Implicit/explicit aliasing of columns.",
             "start_column": 5,
             "end_column": 5,
             "title": "SQLFluff",
@@ -1399,7 +1517,7 @@ def test__cli__command_lint_serialize_github_annotation():
                 "test/fixtures/linter/identifier_capitalisation.sql"
             ),
             "line": 3,
-            "message": "L014: Unquoted identifiers must be consistently lower case.",
+            "message": "CP02: Unquoted identifiers must be consistently lower case.",
             "start_column": 5,
             "end_column": 5,
             "title": "SQLFluff",
@@ -1411,7 +1529,7 @@ def test__cli__command_lint_serialize_github_annotation():
                 "test/fixtures/linter/identifier_capitalisation.sql"
             ),
             "line": 4,
-            "message": "L010: Keywords must be consistently lower case.",
+            "message": "CP01: Keywords must be consistently lower case.",
             "start_column": 1,
             "end_column": 1,
             "title": "SQLFluff",
@@ -1423,7 +1541,7 @@ def test__cli__command_lint_serialize_github_annotation():
                 "test/fixtures/linter/identifier_capitalisation.sql"
             ),
             "line": 4,
-            "message": "L014: Unquoted identifiers must be consistently lower case.",
+            "message": "CP02: Unquoted identifiers must be consistently lower case.",
             "start_column": 12,
             "end_column": 12,
             "title": "SQLFluff",
@@ -1435,7 +1553,7 @@ def test__cli__command_lint_serialize_github_annotation():
                 "test/fixtures/linter/identifier_capitalisation.sql"
             ),
             "line": 4,
-            "message": "L014: Unquoted identifiers must be consistently lower case.",
+            "message": "CP02: Unquoted identifiers must be consistently lower case.",
             "start_column": 18,
             "end_column": 18,
             "title": "SQLFluff",
@@ -1466,22 +1584,25 @@ def test__cli__command_lint_serialize_github_annotation_native():
 
     assert result.output == "\n".join(
         [
-            f"::error title=SQLFluff,file={fpath_normalised},line=1,col=1::"
-            "L036: Select targets should be on a new line unless there is only one "
-            "select target.",
             f"::error title=SQLFluff,file={fpath_normalised},line=2,col=5::"
-            "L027: Unqualified reference 'foo' found in select with more than one "
-            "referenced table/view.",
+            "RF02: Unqualified reference 'foo' found in select with more than one "
+            "referenced table/view. [references.qualification]",
+            f"::error title=SQLFluff,file={fpath_normalised},line=3,col=1::"
+            "LT02: Expected indent of 8 spaces. [layout.indent]",
             f"::error title=SQLFluff,file={fpath_normalised},line=3,col=5::"
-            "L012: Implicit/explicit aliasing of columns.",
+            "AL02: Implicit/explicit aliasing of columns. [aliasing.column]",
             f"::error title=SQLFluff,file={fpath_normalised},line=3,col=5::"
-            "L014: Unquoted identifiers must be consistently lower case.",
+            "CP02: Unquoted identifiers must be consistently lower case. "
+            "[capitalisation.identifiers]",
             f"::error title=SQLFluff,file={fpath_normalised},line=4,col=1::"
-            "L010: Keywords must be consistently lower case.",
+            "CP01: Keywords must be consistently lower case. "
+            "[capitalisation.keywords]",
             f"::error title=SQLFluff,file={fpath_normalised},line=4,col=12::"
-            "L014: Unquoted identifiers must be consistently lower case.",
+            "CP02: Unquoted identifiers must be consistently lower case. "
+            "[capitalisation.identifiers]",
             f"::error title=SQLFluff,file={fpath_normalised},line=4,col=18::"
-            "L014: Unquoted identifiers must be consistently lower case.",
+            "CP02: Unquoted identifiers must be consistently lower case. "
+            "[capitalisation.identifiers]",
             "",  # SQLFluff produces trailing newline
         ]
     )
@@ -1547,7 +1668,7 @@ def test_encoding(encoding_in, encoding_out):
     with open("test/fixtures/linter/indentation_errors.sql", "r") as testfile:
         generic_roundtrip_test(
             testfile,
-            "L001",
+            "LT01",
             input_file_encoding=encoding_in,
             output_file_encoding=encoding_out,
         )
@@ -1590,7 +1711,7 @@ def test_cli_encoding(encoding, method, expect_success, tmpdir):
 
 
 def test_cli_no_disable_noqa_flag():
-    """Test that unset --disable_noqa flag respects inline noqa comments."""
+    """Test that unset --disable-noqa flag respects inline noqa comments."""
     invoke_assert_code(
         ret_code=0,
         args=[
@@ -1601,7 +1722,7 @@ def test_cli_no_disable_noqa_flag():
 
 
 def test_cli_disable_noqa_flag():
-    """Test that --disable_noqa flag ignores inline noqa comments."""
+    """Test that --disable-noqa flag ignores inline noqa comments."""
     result = invoke_assert_code(
         ret_code=1,
         args=[
@@ -1615,7 +1736,26 @@ def test_cli_disable_noqa_flag():
     raw_output = repr(result.output)
 
     # Linting error is raised even though it is inline ignored.
-    assert r"L:   5 | P:  11 | L010 |" in raw_output
+    assert r"L:   6 | P:  11 | CP01 |" in raw_output
+
+
+def test_cli_warn_unused_noqa_flag():
+    """Test that --warn-unused-ignores flag works."""
+    result = invoke_assert_code(
+        # Return value should still be success.
+        ret_code=0,
+        args=[
+            lint,
+            [
+                "test/fixtures/cli/disable_noqa_test.sql",
+                "--warn-unused-ignores",
+            ],
+        ],
+    )
+    raw_output = repr(result.output)
+
+    # Warning shown.
+    assert r"L:   5 | P:  18 | NOQA | WARNING: Unused noqa: 'noqa: CP01'" in raw_output
 
 
 def test_cli_get_default_config():
@@ -1703,8 +1843,8 @@ class TestProgressBars:
         raw_output = repr(result.output)
 
         assert r"\rlint by rules:" in raw_output
-        assert r"\rrule L001:" in raw_output
-        assert r"\rrule L049:" in raw_output
+        assert r"\rrule LT01:" in raw_output
+        assert r"\rrule CV05:" in raw_output
 
     def test_cli_lint_enabled_progress_bar_multiple_paths(
         self, mock_disable_progress_bar: MagicMock
@@ -1733,8 +1873,8 @@ class TestProgressBars:
             in raw_output
         )
         assert r"\rlint by rules:" in raw_output
-        assert r"\rrule L001:" in raw_output
-        assert r"\rrule L049:" in raw_output
+        assert r"\rrule LT01:" in raw_output
+        assert r"\rrule CV05:" in raw_output
 
     def test_cli_lint_enabled_progress_bar_multiple_files(
         self, mock_disable_progress_bar: MagicMock
@@ -1772,8 +1912,8 @@ class TestProgressBars:
             in raw_output
         )
         assert r"\rlint by rules:" in raw_output
-        assert r"\rrule L001:" in raw_output
-        assert r"\rrule L049:" in raw_output
+        assert r"\rrule LT01:" in raw_output
+        assert r"\rrule CV05:" in raw_output
 
     def test_cli_fix_disabled_progress_bar(
         self, mock_disable_progress_bar: MagicMock
@@ -1818,9 +1958,11 @@ class TestProgressBars:
 
 multiple_expected_output = """==== finding fixable violations ====
 == [test/fixtures/linter/multiple_sql_errors.sql] FAIL
-L:  12 | P:   1 | L003 | Expected 1 indentation, found 0 [compared to line 10]
+L:  12 | P:   1 | LT02 | Expected indent of 4 spaces. [layout.indent]
+L:  40 | P:  10 | ST09 | Joins should list the table referenced earlier first.
+                       | [structure.join_condition_order]
 ==== fixing violations ====
-1 fixable linting violations found
+2 fixable linting violations found
 Are you sure you wish to attempt to fix these? [Y/n] ...
 Invalid input, please enter 'Y' or 'N'
 Aborting...
@@ -1829,7 +1971,7 @@ Aborting...
 
 
 def test__cli__fix_multiple_errors_no_show_errors():
-    """Basic checking of lint functionality."""
+    """Test the fix output."""
     result = invoke_assert_code(
         ret_code=1,
         args=[
@@ -1848,8 +1990,57 @@ def test__cli__fix_multiple_errors_no_show_errors():
     assert result.output.replace("\\", "/").startswith(multiple_expected_output)
 
 
+def test__cli__fix_multiple_errors_quiet_force():
+    """Test the fix --quiet option with --force."""
+    result = invoke_assert_code(
+        ret_code=0,
+        args=[
+            fix,
+            [
+                "--disable-progress-bar",
+                "test/fixtures/linter/multiple_sql_errors.sql",
+                "--force",
+                "--quiet",
+                "-x",
+                "_fix",
+            ],
+        ],
+    )
+    normalised_output = result.output.replace("\\", "/")
+    assert normalised_output.startswith(
+        """== [test/fixtures/linter/multiple_sql_errors.sql] FIXED
+2 fixable linting violations found"""
+    )
+
+
+def test__cli__fix_multiple_errors_quiet_no_force():
+    """Test the fix --quiet option without --force."""
+    result = invoke_assert_code(
+        ret_code=0,
+        args=[
+            fix,
+            [
+                "--disable-progress-bar",
+                "test/fixtures/linter/multiple_sql_errors.sql",
+                "--quiet",
+                "-x",
+                "_fix",
+            ],
+            # Test with the confirmation step.
+            "y",
+        ],
+    )
+    normalised_output = result.output.replace("\\", "/")
+    assert normalised_output.startswith(
+        """2 fixable linting violations found
+Are you sure you wish to attempt to fix these? [Y/n] ...
+== [test/fixtures/linter/multiple_sql_errors.sql] FIXED
+All Finished"""
+    )
+
+
 def test__cli__fix_multiple_errors_show_errors():
-    """Basic checking of lint functionality."""
+    """Test the fix --show-lint-violations option."""
     result = invoke_assert_code(
         ret_code=1,
         args=[
@@ -1866,24 +2057,21 @@ def test__cli__fix_multiple_errors_show_errors():
     assert check_a in result.output
     # Finally check the WHOLE output to make sure that unexpected newlines are not
     # added. The replace command just accounts for cross platform testing.
+    assert "L:  12 | P:   1 | LT02 | Expected indent of 4 spaces." in result.output
     assert (
-        "L:  12 | P:   1 | L003 | Expected 1 indentation, found 0 [compared to line 10]"
-        in result.output
-    )
-    assert (
-        "L:  36 | P:   9 | L027 | Unqualified reference 'package_id' found in "
+        "L:  36 | P:   9 | RF02 | Unqualified reference 'package_id' found in "
         "select with more than" in result.output
     )
     assert (
-        "L:  45 | P:  17 | L027 | Unqualified reference 'owner_type' found in "
+        "L:  45 | P:  17 | RF02 | Unqualified reference 'owner_type' found in "
         "select with more than" in result.output
     )
     assert (
-        "L:  45 | P:  50 | L027 | Unqualified reference 'app_key' found in "
+        "L:  45 | P:  50 | RF02 | Unqualified reference 'app_key' found in "
         "select with more than one" in result.output
     )
     assert (
-        "L:  42 | P:  45 | L027 | Unqualified reference 'owner_id' found in "
+        "L:  42 | P:  45 | RF02 | Unqualified reference 'owner_id' found in "
         "select with more than" in result.output
     )
 
