@@ -1,41 +1,53 @@
 """Tests for the configuration routines."""
 
+import logging
 import os
 import sys
-
-from sqlfluff.core import config, Linter, FluffConfig
-from sqlfluff.core.config import (
-    REMOVED_CONFIGS,
-    ConfigLoader,
-    nested_combine,
-    dict_diff,
-)
-from sqlfluff.core.errors import SQLFluffUserError
-from sqlfluff.core.templaters import (
-    RawTemplater,
-    PythonTemplater,
-    JinjaTemplater,
-    PlaceholderTemplater,
-)
-
 from pathlib import Path
-from unittest.mock import patch, call
+from unittest.mock import call, patch
+
 import appdirs
 import pytest
 
+from sqlfluff.core import FluffConfig, Linter
+from sqlfluff.core.config import (
+    REMOVED_CONFIGS,
+    ConfigLoader,
+)
+from sqlfluff.core.errors import SQLFluffUserError
+from sqlfluff.core.templaters import (
+    JinjaTemplater,
+    PlaceholderTemplater,
+    PythonTemplater,
+    RawTemplater,
+)
+from sqlfluff.utils.testing.logging import fluff_log_catcher
 
 config_a = {
-    "core": {"testing_val": "foobar", "testing_int": 4},
+    "core": {"testing_val": "foobar", "testing_int": 4, "dialect": "mysql"},
     "bar": {"foo": "barbar"},
 }
 
 config_b = {
-    "core": {"rules": "L007", "dialect": "ansi"},
+    "core": {"rules": "LT03", "dialect": "ansi"},
     "layout": {
         "type": {"comma": {"line_position": "trailing", "spacing_before": "touch"}}
     },
 }
 
+config_c = {
+    "core": {"rules": "LT03", "dialect": "ansi"},
+    # NOTE:
+    # - NOT_A_RULE doesn't match anything.
+    # - L001 is an alias, but no longer a rule.
+    # - layout is a group and but doesn't match any individual rule.
+    "rules": {
+        "NOT_A_RULE": {"foo": "bar"},
+        "L001": {"foo": "bar"},
+        "layout": {"foo": "bar"},
+    },
+}
+
 
 @pytest.fixture
 def mock_xdg_home(monkeypatch):
@@ -43,28 +55,6 @@ def mock_xdg_home(monkeypatch):
     monkeypatch.setenv("XDG_CONFIG_HOME", "~/.config/my/special/path")
 
 
-def test__config__nested_combine():
-    """Test combination of two config dicts."""
-    a = {"a": {"b": {"c": 123, "d": 456}}}
-    b = {"b": {"b": {"c": 123, "d": 456}}}
-    c = {"a": {"b": {"c": 234, "e": 456}}}
-    r = nested_combine(a, b, c)
-    assert r == {
-        "a": {"b": {"c": 234, "e": 456, "d": 456}},
-        "b": {"b": {"c": 123, "d": 456}},
-    }
-
-
-def test__config__dict_diff():
-    """Test diffs between two config dicts."""
-    a = {"a": {"b": {"c": 123, "d": 456, "f": 6}}}
-    b = {"b": {"b": {"c": 123, "d": 456}}}
-    c = {"a": {"b": {"c": 234, "e": 456, "f": 6}}}
-    assert dict_diff(a, b) == a
-    assert dict_diff(a, c) == {"a": {"b": {"c": 123, "d": 456}}}
-    assert dict_diff(c, a) == {"a": {"b": {"c": 234, "e": 456}}}
-
-
 def test__config__load_file_dir():
     """Test loading config from a directory path."""
     c = ConfigLoader()
@@ -74,6 +64,31 @@ def test__config__load_file_dir():
     assert cfg == config_a
 
 
+def test__config__load_from_string():
+    """Test loading config from a string."""
+    c = ConfigLoader()
+    # Load a string
+    with open(
+        os.path.join("test", "fixtures", "config", "inheritance_a", ".sqlfluff")
+    ) as f:
+        config_string = f.read()
+    cfg = c.load_config_string(config_string)
+    assert cfg == config_a
+
+
+def test__config__from_strings():
+    """Test loading config from multiple strings."""
+    strings = [
+        "[sqlfluff]\ndialect=mysql\ntesting_val=foobar",
+        "[sqlfluff]\ndialect=postgres\ntesting_val2=bar",
+        "[sqlfluff]\ndialect=mysql\ntesting_val=foo",
+    ]
+    cfg = FluffConfig.from_strings(*strings)
+    assert cfg.get("dialect") == "mysql"
+    assert cfg.get("testing_val2") == "bar"
+    assert cfg.get("testing_val") == "foo"
+
+
 def test__config__load_file_f():
     """Test loading config from a file path."""
     c = ConfigLoader()
@@ -93,7 +108,7 @@ def test__config__load_nested():
     )
     assert cfg == {
         "core": {
-            "dialect": "ansi",
+            "dialect": "mysql",
             "testing_val": "foobar",
             "testing_int": 1,
             "testing_bar": 7.698,
@@ -130,10 +145,12 @@ def test__config__load_toml():
             "testing_bar": 7.698,
             "testing_bool": False,
             "testing_arr": ["a", "b", "c"],
+            "rules": ["LT03", "LT09"],
             "testing_inline_table": {"x": 1},
         },
         "bar": {"foo": "foobar"},
         "fnarr": {"fnarr": {"foo": "foobar"}},
+        "rules": {"capitalisation.keywords": {"capitalisation_policy": "upper"}},
     }
 
 
@@ -198,19 +215,33 @@ def test__config__nested_config_tests():
     test.
     """
     lntr = Linter(
-        config=FluffConfig(overrides=dict(exclude_rules="L002", dialect="ansi"))
+        # Exclude CP02 in overrides (similar to cli --exclude-rules)
+        config=FluffConfig(overrides=dict(exclude_rules="CP02", dialect="ansi"))
     )
     lnt = lntr.lint_path("test/fixtures/config/inheritance_b")
     violations = lnt.check_tuples(by_path=True)
     for k in violations:
         if k.endswith("nested\\example.sql"):
-            assert ("L003", 1, 4) in violations[k]
-            assert ("L009", 1, 12) in violations[k]
-            assert "L002" not in [c[0] for c in violations[k]]
+            # CP01 is enabled in the .sqlfluff file and not excluded.
+            assert ("CP01", 1, 4) in violations[k]
+            # LT02 is enabled in the .sqlfluff file and not excluded.
+            assert ("LT02", 1, 1) in violations[k]
+            # CP02 is enabled in the .sqlfluff file but excluded by the
+            # override above.
+            assert "CP02" not in [c[0] for c in violations[k]]
         elif k.endswith("inheritance_b\\example.sql"):
-            assert ("L003", 1, 4) in violations[k]
-            assert "L002" not in [c[0] for c in violations[k]]
-            assert "L009" not in [c[0] for c in violations[k]]
+            # CP01 is enabled because while disabled in the tox.ini file,
+            # the exclude-rules option is overridden by the override above
+            # which effectively sets the exclude to CP02 and in effect
+            # re-enables CP01.
+            # This may seem counter-intuitive but is in line with current
+            # documentation on how to use `rules` and `exclude-rules`.
+            # https://docs.sqlfluff.com/en/latest/configuration.html#enabling-and-disabling-rules
+            assert ("CP01", 1, 4) in violations[k]
+            # CP02 is disabled because of the override above.
+            assert "CP02" not in [c[0] for c in violations[k]]
+            # LT02 is disabled because it is not in the `rules` of tox.ini
+            assert "LT02" not in [c[0] for c in violations[k]]
 
 
 @patch("os.path.exists")
@@ -247,18 +278,6 @@ def test__config__load_user_appdir_config(
     )
 
 
-@pytest.mark.parametrize(
-    "raw_str, expected",
-    [
-        ("L011,L022,L031", ["L011", "L022", "L031"]),
-        ("\nL011,\nL022,\nL031,", ["L011", "L022", "L031"]),
-    ],
-)
-def test__config__split_comma_separated_string(raw_str, expected):
-    """Tests that comma separated string config is handled correctly."""
-    assert config._split_comma_separated_string(raw_str) == expected
-
-
 def test__config__templater_selection():
     """Test template selection by name."""
     cfg = FluffConfig(overrides={"dialect": "ansi"})
@@ -282,11 +301,11 @@ def test__config__glob_exclude_config_tests():
     lnt = lntr.lint_path("test/fixtures/config/glob_exclude/test.sql")
     violations = lnt.check_tuples(by_path=True)
     for k in violations:
-        assert ("L044", 10, 1) in violations[k]
-        assert "L027" not in [c[0] for c in violations[k]]
-        assert "L050" not in [c[0] for c in violations[k]]
-        assert "L051" not in [c[0] for c in violations[k]]
-        assert "L052" not in [c[0] for c in violations[k]]
+        assert ("AM04", 12, 1) in violations[k]
+        assert "RF02" not in [c[0] for c in violations[k]]
+        assert "LT13" not in [c[0] for c in violations[k]]
+        assert "AM05" not in [c[0] for c in violations[k]]
+        assert "CV06" not in [c[0] for c in violations[k]]
 
 
 def test__config__glob_include_config_tests():
@@ -299,11 +318,11 @@ def test__config__glob_include_config_tests():
     lnt = lntr.lint_path("test/fixtures/config/glob_include/test.sql")
     violations = lnt.check_tuples(by_path=True)
     for k in violations:
-        assert ("L050", 1, 1) in violations[k]
-        assert ("L051", 12, 1) in violations[k]
-        assert ("L052", 12, 9) in violations[k]
-        assert ("L027", 10, 8) in violations[k]
-        assert "L044" not in [c[0] for c in violations[k]]
+        assert ("LT13", 1, 1) in violations[k]
+        assert ("AM05", 14, 1) in violations[k]
+        assert ("CV06", 14, 9) in violations[k]
+        assert ("RF02", 12, 8) in violations[k]
+        assert "AM04" not in [c[0] for c in violations[k]]
 
 
 def test__config__rules_set_to_none():
@@ -317,9 +336,9 @@ def test__config__rules_set_to_none():
     lnt = lntr.lint_path("test/fixtures/config/rules_set_to_none/test.sql")
     violations = lnt.check_tuples(by_path=True)
     for k in violations:
-        assert ("L050", 1, 1) in violations[k]
-        assert ("L044", 12, 1) in violations[k]
-        assert ("L010", 12, 10) in violations[k]
+        assert ("LT13", 1, 1) in violations[k]
+        assert ("AM04", 12, 1) in violations[k]
+        assert ("CP01", 12, 10) in violations[k]
 
 
 def test__config__rules_group_with_exclude():
@@ -330,15 +349,15 @@ def test__config__rules_group_with_exclude():
     lnt = lntr.lint_path("test/fixtures/config/rules_group_with_exclude/test.sql")
     violations = lnt.check_tuples(by_path=True)
     for k in violations:
-        assert ("L010", 15, 1) in violations[k]
-        assert "L019" not in [c[0] for c in violations[k]]
+        assert ("CP01", 15, 1) in violations[k]
+        assert "LT04" not in [c[0] for c in violations[k]]
 
 
 def test__config__get_section():
     """Test FluffConfig.get_section method."""
     cfg = FluffConfig(config_b)
 
-    assert cfg.get_section("core").get("rules", None) == "L007"
+    assert cfg.get_section("core").get("rules", None) == "LT03"
     assert cfg.get_section(["layout", "type", "comma"]) == {
         "line_position": "trailing",
         "spacing_before": "touch",
@@ -350,7 +369,7 @@ def test__config__get():
     """Test FluffConfig.get method."""
     cfg = FluffConfig(config_b)
 
-    assert cfg.get("rules") == "L007"
+    assert cfg.get("rules") == "LT03"
     assert cfg.get("rulez") is None
     assert cfg.get("rulez", section="core", default=123) == 123
     assert (
@@ -368,14 +387,26 @@ def test__config__from_kwargs():
     # Instantiate config object.
     cfg = FluffConfig.from_kwargs(
         dialect="snowflake",
-        rules=["L001", "L002"],
-        exclude_rules=["L010", "L011"],
+        rules=["LT01", "LT02"],
+        exclude_rules=["CP01", "AL01"],
     )
 
     # Verify we can later retrieve the config values.
     assert cfg.get("dialect") == "snowflake"
-    assert cfg.get("rules") == "L001,L002"
-    assert cfg.get("exclude_rules") == "L010,L011"
+    assert cfg.get("rules") == "LT01,LT02"
+    assert cfg.get("exclude_rules") == "CP01,AL01"
+
+
+def test__config__from_string():
+    """Test from_string method of FluffConfig."""
+    with open(
+        os.path.join("test", "fixtures", "config", "inheritance_a", ".sqlfluff")
+    ) as f:
+        config_string = f.read()
+    cfg = FluffConfig.from_string(config_string)
+    # Verify we can later retrieve the config values.
+    assert cfg.get("testing_val") == "foobar"
+    assert cfg.get("dialect") == "mysql"
 
 
 def test__config_missing_dialect():
@@ -420,10 +451,54 @@ def test__config__validate_configs_indirect():
         )
 
 
+@pytest.mark.parametrize(
+    "raw_sql",
+    [
+        (
+            # "types" not "type"
+            "-- sqlfluff:layout:types:comma:line_position:leading\n"
+            "SELECT 1"
+        ),
+        (
+            # Unsupported layout config length
+            "-- sqlfluff:layout:foo\n"
+            "SELECT 1"
+        ),
+        (
+            # Unsupported layout config length
+            "-- sqlfluff:layout:type:comma:bar\n"
+            "SELECT 1"
+        ),
+        (
+            # Unsupported layout config key ("foo")
+            "-- sqlfluff:layout:type:comma:foo:bar\n"
+            "SELECT 1"
+        ),
+        (
+            # Unsupported layout config key ("foo") [no space]
+            "--sqlfluff:layout:type:comma:foo:bar\n"
+            "SELECT 1"
+        ),
+    ],
+)
+def test__config__validate_configs_inline_layout(raw_sql):
+    """Test _validate_configs method of FluffConfig when used on a file.
+
+    This test covers both the validation of inline config
+    directives but also the validation of layout configs.
+    """
+    # Instantiate config object.
+    cfg = FluffConfig(configs={"core": {"dialect": "ansi"}})
+
+    # Try to process an invalid inline config. Make sure we get an error.
+    with pytest.raises(SQLFluffUserError):
+        cfg.process_raw_file_for_config(raw_sql, "test.sql")
+
+
 def test__config__validate_configs_precedence_same_file():
     """Test _validate_configs method of FluffConfig where there's a conflict."""
     # Check with a known conflicted value
-    old_key = ("rules", "L007", "operator_new_lines")
+    old_key = ("rules", "LT03", "operator_new_lines")
     new_key = ("layout", "type", "binary_operator", "line_position")
     # Check it's still conflicted.
     assert any(
@@ -439,3 +514,73 @@ def test__config__validate_configs_precedence_same_file():
     assert len(res) == 1
     # Check that the old key isn't there.
     assert not any(k == old_key for k, _ in res)
+
+
+def test__config__toml_list_config():
+    """Test Parsing TOML list of values."""
+    c = ConfigLoader()
+    loaded_config = c.load_config_file(
+        os.path.join("test", "fixtures", "config", "toml"),
+        "pyproject.toml",
+    )
+    loaded_config["core"]["dialect"] = "ansi"
+    cfg = FluffConfig(loaded_config)
+
+    # Verify we can later retrieve the config values.
+    assert cfg.get("dialect") == "ansi"
+    assert cfg.get("rules") == ["LT03", "LT09"]
+
+
+def test__config__warn_unknown_rule():
+    """Test warnings when rules are unknown."""
+    lntr = Linter(config=FluffConfig(config_c))
+
+    with fluff_log_catcher(logging.WARNING, "sqlfluff.rules") as caplog:
+        lntr.get_rulepack()
+
+    # Check we get a warning on the unrecognised rule.
+    assert (
+        "Rule configuration contain a section for unexpected rule 'NOT_A_RULE'."
+    ) in caplog.text
+    # Check we get a warning for the deprecated rule.
+    assert (
+        "Rule configuration contain a section for unexpected rule 'L001'."
+    ) in caplog.text
+    # Check we get a hint for the matched rule.
+    assert "match for rule LT01 with name 'layout.spacing'" in caplog.text
+    # Check we get a warning for the group name.
+    assert (
+        "Rule configuration contain a section for unexpected rule 'layout'."
+    ) in caplog.text
+    # Check we get a hint for the matched rule group.
+    # NOTE: We don't check the set explicitly because we can't assume ordering.
+    assert ("The reference was found as a match for multiple rules: {") in caplog.text
+    assert ("LT01") in caplog.text
+    assert ("LT02") in caplog.text
+
+
+def test__process_inline_config():
+    """Test the processing of inline in-file configuration directives."""
+    cfg = FluffConfig(config_b)
+    assert cfg.get("rules") == "LT03"
+
+    cfg.process_inline_config("-- sqlfluff:rules:LT02", "test.sql")
+    assert cfg.get("rules") == "LT02"
+
+    assert cfg.get("tab_space_size", section="indentation") == 4
+    cfg.process_inline_config("-- sqlfluff:indentation:tab_space_size:20", "test.sql")
+    assert cfg.get("tab_space_size", section="indentation") == 20
+
+    assert cfg.get("dialect") == "ansi"
+    assert cfg.get("dialect_obj").name == "ansi"
+    cfg.process_inline_config("-- sqlfluff:dialect:postgres", "test.sql")
+    assert cfg.get("dialect") == "postgres"
+    assert cfg.get("dialect_obj").name == "postgres"
+
+    assert cfg.get("rulez") is None
+    cfg.process_inline_config("-- sqlfluff:rulez:LT06", "test.sql")
+    assert cfg.get("rulez") == "LT06"
+
+    # Check that Windows paths don't get mangled
+    cfg.process_inline_config("-- sqlfluff:jinja:my_path:c:\\foo", "test.sql")
+    assert cfg.get("my_path", section="jinja") == "c:\\foo"
