@@ -6,25 +6,36 @@ sense to test in the context of a templater which supports
 loops and placeholders.
 """
 
-from collections import defaultdict
 import logging
+from collections import defaultdict
+from pathlib import Path
 from typing import List, NamedTuple
 
 import pytest
 from jinja2.exceptions import UndefinedError
 
-from sqlfluff.core.errors import SQLFluffSkipFile, SQLTemplaterError
+from sqlfluff.core import FluffConfig, Linter
+from sqlfluff.core.errors import SQLFluffSkipFile, SQLFluffUserError, SQLTemplaterError
 from sqlfluff.core.templaters import JinjaTemplater
 from sqlfluff.core.templaters.base import RawFileSlice, TemplatedFile
 from sqlfluff.core.templaters.jinja import DummyUndefined, JinjaAnalyzer
-from sqlfluff.core import Linter, FluffConfig
-
 
 JINJA_STRING = (
     "SELECT * FROM {% for c in blah %}{{c}}{% if not loop.last %}, "
     "{% endif %}{% endfor %} WHERE {{condition}}\n\n"
 )
 
+JINJA_MACRO_CALL_SQL = (
+    "{% macro render_name(title) %}\n"
+    "  '{{ title }}. foo' as {{ caller() }}\n"
+    "{% endmacro %}\n"
+    "SELECT\n"
+    "    {% call render_name('Sir') %}\n"
+    "        bar\n"
+    "    {% endcall %}\n"
+    "FROM baz\n"
+)
+
 
 @pytest.mark.parametrize(
     "instr, expected_outstr",
@@ -143,6 +154,82 @@ class RawTemplatedTestCase(NamedTuple):
                 "SELECT 1, 2\n",
             ],
         ),
+        RawTemplatedTestCase(
+            name="strip_and_templated_whitespace",
+            instr="SELECT {{- '  ' -}} 1{{ ' , 2' -}}\n",
+            templated_str="SELECT  1 , 2",
+            expected_templated_sliced__source_list=[
+                "SELECT",
+                " ",
+                "{{- '  ' -}}",
+                " ",
+                "1",
+                "{{ ' , 2' -}}",
+                "\n",
+            ],
+            expected_templated_sliced__templated_list=[
+                "SELECT",
+                "",  # Placeholder for consumed whitespace
+                "  ",  # Placeholder for templated whitespace
+                "",  # Placeholder for consumed whitespace
+                "1",
+                " , 2",
+                "",  # Placeholder for consumed newline
+            ],
+            expected_raw_sliced__source_list=[
+                "SELECT",
+                " ",
+                "{{- '  ' -}}",
+                " ",
+                "1",
+                "{{ ' , 2' -}}",
+                "\n",
+            ],
+        ),
+        RawTemplatedTestCase(
+            name="strip_both_block_hard",
+            instr="SELECT {%- set x = 42 %} 1 {%- if true -%} , 2{% endif -%}\n",
+            templated_str="SELECT 1, 2",
+            expected_templated_sliced__source_list=[
+                "SELECT",
+                # NB: Even though the jinja tag consumes whitespace, we still
+                # get it here as a placeholder.
+                " ",
+                "{%- set x = 42 %}",
+                " 1",
+                # This whitespace is a seperate from the 1 because it's consumed.
+                " ",
+                "{%- if true -%}",
+                " ",
+                ", 2",
+                "{% endif -%}",
+                "\n",
+            ],
+            expected_templated_sliced__templated_list=[
+                "SELECT",
+                "",  # Consumed whitespace placeholder
+                "",  # Jinja block placeholder
+                " 1",
+                "",  # Consumed whitespace
+                "",  # Jinja block placeholder
+                "",  # More consumed whitespace
+                ", 2",
+                "",  # Jinja block
+                "",  # Consumed final newline.
+            ],
+            expected_raw_sliced__source_list=[
+                "SELECT",
+                " ",
+                "{%- set x = 42 %}",
+                " 1",
+                " ",
+                "{%- if true -%}",
+                " ",
+                ", 2",
+                "{% endif -%}",
+                "\n",
+            ],
+        ),
         RawTemplatedTestCase(
             name="basic_data",
             instr="""select
@@ -407,7 +494,7 @@ def test__templater_jinja_slices(case: RawTemplatedTestCase):
         fname="test",
         config=FluffConfig(overrides={"dialect": "ansi"}),
     )
-    assert templated_file
+    assert templated_file is not None
     assert templated_file.source_str == case.instr
     assert templated_file.templated_str == case.templated_str
     # Build and check the list of source strings referenced by "sliced_file".
@@ -527,7 +614,7 @@ def test__templater_jinja_error_catastrophic():
 def test__templater_jinja_error_macro_path_does_not_exist():
     """Tests that an error is raised if macro path doesn't exist."""
     with pytest.raises(ValueError) as e:
-        JinjaTemplater().template_builder(
+        JinjaTemplater().construct_render_func(
             config=FluffConfig.from_path(
                 "test/fixtures/templater/jinja_macro_path_does_not_exist"
             )
@@ -535,16 +622,36 @@ def test__templater_jinja_error_macro_path_does_not_exist():
     assert str(e.value).startswith("Path does not exist")
 
 
+def test__templater_jinja_error_macro_invalid():
+    """Tests that an error is raised if a macro is invalid."""
+    invalid_macro_config_string = (
+        "[sqlfluff]\n"
+        "templater = jinja\n"
+        "dialect = ansi\n"
+        "[sqlfluff:templater:jinja:macros]\n"
+        "a_macro_def = {% macro pkg.my_macro() %}pass{% endmacro %}\n"
+    )
+    config = FluffConfig.from_string(invalid_macro_config_string)
+    with pytest.raises(SQLFluffUserError) as e:
+        JinjaTemplater().construct_render_func(config=config)
+    error_string = str(e.value)
+    assert error_string.startswith("Error loading user provided macro")
+    assert "{% macro pkg.my_macro() %}pass{% endmacro %}" in error_string
+
+
 def test__templater_jinja_lint_empty():
     """Check that parsing a file which renders to an empty string.
 
-    No exception should be raised, but the parsed tree should be None.
+    No exception should be raised, and we should get a single templated element.
     """
     lntr = Linter(dialect="ansi")
     parsed = lntr.parse_string(in_str='{{ "" }}')
     assert parsed.templated_file.source_str == '{{ "" }}'
     assert parsed.templated_file.templated_str == ""
-    assert parsed.tree is None
+    # Get the types of the segments
+    print(f"Segments: {parsed.tree.raw_segments}")
+    seg_types = [seg.get_type() for seg in parsed.tree.raw_segments]
+    assert seg_types == ["placeholder", "end_of_file"]
 
 
 def assert_structure(yaml_loader, path, code_only=True, include_meta=False):
@@ -579,6 +686,7 @@ def assert_structure(yaml_loader, path, code_only=True, include_meta=False):
         ("jinja_c_dbt/dbt_builtins_source", True, False),
         ("jinja_c_dbt/dbt_builtins_this", True, False),
         ("jinja_c_dbt/dbt_builtins_var_default", True, False),
+        ("jinja_c_dbt/dbt_builtins_test", True, False),
         # do directive
         ("jinja_e/jinja", True, False),
         # case sensitivity and python literals
@@ -600,6 +708,11 @@ def assert_structure(yaml_loader, path, code_only=True, include_meta=False):
         ("jinja_l_metas/004", False, True),
         ("jinja_l_metas/005", False, True),
         ("jinja_l_metas/006", False, True),
+        ("jinja_l_metas/007", False, True),
+        ("jinja_l_metas/008", False, True),
+        ("jinja_l_metas/009", False, True),
+        ("jinja_l_metas/010", False, True),
+        ("jinja_l_metas/011", False, True),
         # Library Loading from a folder when library is module
         ("jinja_m_libraries_module/jinja", True, False),
         ("jinja_n_nested_macros/jinja", True, False),
@@ -608,6 +721,7 @@ def assert_structure(yaml_loader, path, code_only=True, include_meta=False):
         ("jinja_p_disable_dbt_builtins/disable_dbt_builtins", True, False),
         # Load all the macros
         ("jinja_q_multiple_path_macros/jinja", True, False),
+        ("jinja_s_filters_in_library/jinja", True, False),
     ],
 )
 def test__templater_full(subpath, code_only, include_meta, yaml_loader, caplog):
@@ -636,7 +750,11 @@ def test__templater_jinja_block_matching(caplog):
     template_segments = [
         seg
         for seg in parsed.raw_segments
-        if seg.is_type("template_loop", "placeholder")
+        if seg.is_type("template_loop")
+        or (
+            seg.is_type("placeholder")
+            and seg.block_type in ("block_start", "block_end", "block_mid")
+        )
     ]
 
     # Group them together by block UUID
@@ -654,9 +772,18 @@ def test__templater_jinja_block_matching(caplog):
     groups = {
         "for actions clause 1": [(6, 5), (9, 5), (12, 5), (15, 5)],
         "for actions clause 2": [(17, 5), (21, 5), (29, 5), (37, 5)],
-        "if loop.first 1": [(18, 9), (20, 9)],
-        "if loop.first 2": [(22, 9), (28, 9)],
-        "if loop.first 3": [(30, 9), (36, 9)],
+        # NOTE: all the if loop clauses are grouped together.
+        "if loop.first": [
+            (18, 9),
+            (20, 9),
+            (20, 9),
+            (22, 9),
+            (22, 9),
+            (28, 9),
+            (30, 9),
+            (30, 9),
+            (36, 9),
+        ],
     }
 
     # Check all are accounted for:
@@ -693,22 +820,22 @@ def test__templater_jinja_block_matching(caplog):
                 (" ", "literal", 22),
                 ("{{field}}", "templated", 23),
                 (" ", "literal", 32),
-                ("{% for i in [1, 3]%}", "block_start", 33),
-                (", fld_", "literal", 53),
-                ("{{i}}", "templated", 59),
-                ("{% endfor %}", "block_end", 64),
-                (" FROM my_schema.", "literal", 76),
-                ("{{my_table}}", "templated", 92),
-                (" ", "literal", 104),
+                ("{% for i in [1, 3]%}", "block_start", 33, 1, "for"),
+                (", fld_", "literal", 53, 1),
+                ("{{i}}", "templated", 59, 1),
+                ("{% endfor %}", "block_end", 64, 1, "endfor"),
+                (" FROM my_schema.", "literal", 76, 2),
+                ("{{my_table}}", "templated", 92, 2),
+                (" ", "literal", 104, 2),
             ],
         ),
         (
             "{% set thing %}FOO{% endset %} BAR",
             [
-                ("{% set thing %}", "block_start", 0),
-                ("FOO", "literal", 15),
-                ("{% endset %}", "block_end", 18),
-                (" BAR", "literal", 30),
+                ("{% set thing %}", "block_start", 0, 1, "set"),
+                ("FOO", "literal", 15, 1),
+                ("{% endset %}", "block_end", 18, 1, "endset"),
+                (" BAR", "literal", 30, 2),
             ],
         ),
         (
@@ -720,14 +847,59 @@ select 1 from foobarfoobarfoobarfoobar_{{ "dev" }}
 {{ my_query }}
 """,
             [
-                ("{% set my_query %}", "block_start", 0),
-                ("\nselect 1 from foobarfoobarfoobarfoobar_", "literal", 18),
-                ('{{ "dev" }}', "templated", 58),
-                ("\n", "literal", 69),
-                ("{% endset %}", "block_end", 70),
-                ("\n", "literal", 82),
-                ("{{ my_query }}", "templated", 83),
-                ("\n", "literal", 97),
+                ("{% set my_query %}", "block_start", 0, 1, "set"),
+                ("\nselect 1 from foobarfoobarfoobarfoobar_", "literal", 18, 1),
+                ('{{ "dev" }}', "templated", 58, 1),
+                ("\n", "literal", 69, 1),
+                ("{% endset %}", "block_end", 70, 1, "endset"),
+                ("\n", "literal", 82, 2),
+                ("{{ my_query }}", "templated", 83, 2),
+                ("\n", "literal", 97, 2),
+            ],
+        ),
+        # Tests for jinja blocks that consume whitespace.
+        (
+            """SELECT 1 FROM {%+if true-%} {{ref('foo')}} {%-endif%}""",
+            [
+                ("SELECT 1 FROM ", "literal", 0),
+                ("{%+if true-%}", "block_start", 14, 1, "if"),
+                (" ", "literal", 27, 1),
+                ("{{ref('foo')}}", "templated", 28, 1),
+                (" ", "literal", 42, 1),
+                ("{%-endif%}", "block_end", 43, 1, "endif"),
+            ],
+        ),
+        (
+            """{% for item in some_list -%}
+    SELECT *
+    FROM some_table
+{{ "UNION ALL\n" if not loop.last }}
+{%- endfor %}""",
+            [
+                ("{% for item in some_list -%}", "block_start", 0, 1, "for"),
+                # This gets consumed in the templated file, but it's still here.
+                ("\n    ", "literal", 28, 1),
+                ("SELECT *\n    FROM some_table\n", "literal", 33, 1),
+                ('{{ "UNION ALL\n" if not loop.last }}', "templated", 62, 1),
+                ("\n", "literal", 97, 1),
+                ("{%- endfor %}", "block_end", 98, 1, "endfor"),
+            ],
+        ),
+        (
+            JINJA_MACRO_CALL_SQL,
+            [
+                ("{% macro render_name(title) %}", "block_start", 0, 1, "macro"),
+                ("\n" "  '", "literal", 30, 1),
+                ("{{ title }}", "templated", 34, 1),
+                (". foo' as ", "literal", 45, 1),
+                ("{{ caller() }}", "templated", 55, 1),
+                ("\n", "literal", 69, 1),
+                ("{% endmacro %}", "block_end", 70, 1, "endmacro"),
+                ("\n" "SELECT\n" "    ", "literal", 84, 2),
+                ("{% call render_name('Sir') %}", "block_start", 96, 3, "call"),
+                ("\n" "        bar\n" "    ", "literal", 125, 3),
+                ("{% endcall %}", "block_end", 142, 3, "endcall"),
+                ("\n" "FROM baz\n", "literal", 155, 4),
             ],
         ),
     ],
@@ -735,9 +907,10 @@ select 1 from foobarfoobarfoobarfoobar_{{ "dev" }}
 def test__templater_jinja_slice_template(test, result):
     """Test _slice_template."""
     templater = JinjaTemplater()
-    env, live_context, make_template = templater.template_builder()
+    env, _, render_func = templater.construct_render_func()
+
     analyzer = JinjaAnalyzer(test, env)
-    analyzer.analyze(make_template)
+    analyzer.analyze(render_func=render_func)
     resp = analyzer.raw_sliced
     # check contiguous (unless there's a comment in it)
     if "{#" not in test:
@@ -748,14 +921,12 @@ def test__templater_jinja_slice_template(test, result):
             assert raw_slice.source_idx == idx
             idx += len(raw_slice.raw)
     # Check total result
-    assert [
-        (raw_slice.raw, raw_slice.slice_type, raw_slice.source_idx)
-        for raw_slice in resp
-    ] == result
+    assert resp == [RawFileSlice(*args) for args in result]
 
 
 def _statement(*args, **kwargs):
-    return "_statement"
+    # NOTE: The standard dbt statement() call returns nothing.
+    return ""
 
 
 def _load_result(*args, **kwargs):
@@ -908,6 +1079,18 @@ SELECT 1
                 ("literal", slice(42, 53, None), slice(18, 29, None)),
             ],
         ),
+        (
+            # Tests Jinja "import" directive.
+            """{% import 'echo.sql' as echo %}
+
+SELECT 1
+""",
+            None,
+            [
+                ("templated", slice(0, 31, None), slice(0, 0, None)),
+                ("literal", slice(31, 42, None), slice(0, 11, None)),
+            ],
+        ),
         (
             # Tests Jinja "from import" directive..
             """{% from 'echo.sql' import echo %}
@@ -919,9 +1102,9 @@ SELECT
 """,
             None,
             [
-                ("block_start", slice(0, 33, None), slice(0, 0, None)),
+                ("templated", slice(0, 33, None), slice(0, 0, None)),
                 ("literal", slice(33, 34, None), slice(0, 1, None)),
-                ("block_start", slice(34, 75, None), slice(1, 1, None)),
+                ("templated", slice(34, 75, None), slice(1, 1, None)),
                 ("literal", slice(75, 88, None), slice(1, 14, None)),
                 ("templated", slice(88, 105, None), slice(14, 19, None)),
                 ("literal", slice(105, 111, None), slice(19, 25, None)),
@@ -929,6 +1112,23 @@ SELECT
                 ("literal", slice(132, 133, None), slice(34, 35, None)),
             ],
         ),
+        (
+            # Tests Jinja "do" directive. Should be treated as a
+            # templated instead of block - issue 4603.
+            """{% do true %}
+
+{% if true %}
+    select 1
+{% endif %}""",
+            None,
+            [
+                ("templated", slice(0, 13, None), slice(0, 0, None)),
+                ("literal", slice(13, 15, None), slice(0, 2, None)),
+                ("block_start", slice(15, 28, None), slice(2, 2, None)),
+                ("literal", slice(28, 42, None), slice(2, 16, None)),
+                ("block_end", slice(42, 53, None), slice(16, 16, None)),
+            ],
+        ),
         (
             # Tests issue 2541, a bug where the {%- endfor %} was causing
             # IndexError: list index out of range.
@@ -1188,30 +1388,52 @@ FROM {{ j }}{{ self.table_name() }}
             ],
         ),
         (
-            """{{ statement('variables', fetch_result=true) }}
-""",
+            "{{ statement('variables', fetch_result=true) }}\n",
             dict(
                 statement=_statement,
                 load_result=_load_result,
             ),
             [
-                ("templated", slice(0, 47, None), slice(0, 10, None)),
-                ("literal", slice(47, 48, None), slice(10, 11, None)),
+                ("templated", slice(0, 47, None), slice(0, 0, None)),
+                ("literal", slice(47, 48, None), slice(0, 1, None)),
             ],
         ),
         (
-            "{% call statement('variables', fetch_result=true) %}"
-            "select 1 as test"
-            "{% endcall %}\n",
+            "{% call statement('variables', fetch_result=true) %}\n"
+            "select 1 as test\n"
+            "{% endcall %}\n"
+            "select 2 as foo\n",
             dict(
                 statement=_statement,
                 load_result=_load_result,
             ),
             [
-                ("templated", slice(0, 52, None), slice(0, 10, None)),
-                ("literal", slice(52, 68, None), slice(10, 10, None)),
-                ("block_end", slice(68, 81, None), slice(10, 10, None)),
-                ("literal", slice(81, 82, None), slice(10, 11, None)),
+                ("block_start", slice(0, 52, None), slice(0, 0, None)),
+                ("literal", slice(52, 70, None), slice(0, 0, None)),
+                ("block_end", slice(70, 83, None), slice(0, 0, None)),
+                ("literal", slice(83, 100, None), slice(0, 17, None)),
+            ],
+        ),
+        (
+            JINJA_MACRO_CALL_SQL,
+            None,
+            [
+                # First all of this is the call block.
+                ("block_start", slice(0, 30, None), slice(0, 0, None)),
+                ("literal", slice(30, 34, None), slice(0, 0, None)),
+                ("templated", slice(34, 45, None), slice(0, 0, None)),
+                ("literal", slice(45, 55, None), slice(0, 0, None)),
+                ("templated", slice(55, 69, None), slice(0, 0, None)),
+                ("literal", slice(69, 70, None), slice(0, 0, None)),
+                ("block_end", slice(70, 84, None), slice(0, 0, None)),
+                # Then the actual query.
+                ("literal", slice(84, 96, None), slice(0, 12, None)),
+                # The block_start (call) contains the actual content.
+                ("block_start", slice(96, 125, None), slice(12, 47, None)),
+                # The middle and end of the call, have zero length in the template
+                ("literal", slice(125, 142, None), slice(47, 47, None)),
+                ("block_end", slice(142, 155, None), slice(47, 47, None)),
+                ("literal", slice(155, 165, None), slice(47, 57, None)),
             ],
         ),
     ],
@@ -1219,16 +1441,15 @@ FROM {{ j }}{{ self.table_name() }}
 def test__templater_jinja_slice_file(raw_file, override_context, result, caplog):
     """Test slice_file."""
     templater = JinjaTemplater(override_context=override_context)
-    env, live_context, make_template = templater.template_builder(
+    _, _, render_func = templater.construct_render_func(
         config=FluffConfig.from_path(
             "test/fixtures/templater/jinja_slice_template_macros"
         )
     )
 
-    templated_file = make_template(raw_file).render()
     with caplog.at_level(logging.DEBUG, logger="sqlfluff.templater"):
         raw_sliced, sliced_file, templated_str = templater.slice_file(
-            raw_file, templated_file, make_template=make_template
+            raw_file, render_func=render_func
         )
     # Create a TemplatedFile from the results. This runs some useful sanity
     # checks.
@@ -1368,3 +1589,80 @@ def test_undefined_magic_methods():
     assert ud > ud
 
     assert ud + ud is ud
+
+
+@pytest.mark.parametrize(
+    "sql_path, expected_renderings",
+    [
+        pytest.param(
+            "simple_if_true.sql",
+            [
+                "\nSELECT 1\n\n",
+                "\nSELECT 2\n\n",
+            ],
+            id="simple_if_true",
+        ),
+        pytest.param(
+            "simple_if_false.sql",
+            [
+                "\nSELECT 2\n\n",
+                "\nSELECT 1\n\n",
+            ],
+            id="simple_if_false",
+        ),
+        pytest.param(
+            "if_elif_else.sql",
+            [
+                "\nSELECT 1\n\n",
+                "\nSELECT 2\n\n",
+                "\nSELECT 3\n\n",
+            ],
+            id="if_elif_else",
+        ),
+        pytest.param(
+            "if_else_if_nested.sql",
+            [
+                "\nSELECT 1\n\n",
+                "\n\nSELECT 2\n\n\n",
+                "\n\nSELECT 3\n\n\n",
+            ],
+            id="if_else_if_nested",
+        ),
+        # This test case exercises the scoring function. Generates up to 10
+        # variants, but only the top 5 are returned.
+        pytest.param(
+            "if_elif_else_chain_scoring.sql",
+            [
+                "\nSELECT 1\n\n",
+                "\nSELECT 100000000\n\n",
+                "\nSELECT 10000000\n\n",
+                "\nSELECT 1000000\n\n",
+                "\nSELECT 100000\n\n",
+                "\nSELECT 10000\n\n",
+            ],
+            id="if_elif_else_chain_scoring",
+        ),
+        # This test case results in a TypeError executing the variant. This
+        # should be ignored, and only the primary should be returned.
+        pytest.param(
+            "if_true_elif_type_error_else.sql",
+            [
+                "\nSELECT 1\n\n",
+                "\nSELECT 2\n\n",
+            ],
+            id="if_true_elif_type_error_else",
+        ),
+    ],
+)
+def test__templater_lint_unreached_code(sql_path: str, expected_renderings):
+    """Test that Jinja templater slices raw and templated file correctly."""
+    test_dir = Path("test/fixtures/templater/jinja_lint_unreached_code")
+    t = JinjaTemplater()
+    renderings = []
+    for templated_file, _ in t.process_with_variants(
+        in_str=(test_dir / sql_path).read_text(),
+        fname=str(sql_path),
+        config=FluffConfig.from_path(str(test_dir)),
+    ):
+        renderings.append(templated_file.templated_str)
+    assert renderings == expected_renderings
