@@ -1,16 +1,15 @@
 """Tests for templaters."""
 
-import pytest
 import logging
+
+import pytest
+
+from sqlfluff.core import FluffConfig, SQLTemplaterError
 from sqlfluff.core.errors import SQLFluffSkipFile
-
 from sqlfluff.core.templaters import PythonTemplater
-from sqlfluff.core import SQLTemplaterError, FluffConfig
-
 from sqlfluff.core.templaters.base import RawFileSlice, TemplatedFileSlice
 from sqlfluff.core.templaters.python import IntermediateFileSlice
 
-
 PYTHON_STRING = "SELECT * FROM {blah}"
 
 
@@ -459,7 +458,9 @@ def test__templater_python_slice_file(raw_file, templated_file, unwrap_wrapped,
     """Test slice_file."""
     _, resp, _ = PythonTemplater().slice_file(
         raw_file,
-        templated_file,
+        # For the render_func we just use a function which just returns the
+        # templated file from the test case.
+        (lambda x: templated_file),
         config=FluffConfig(
             configs={"templater": {"unwrap_wrapped_queries": unwrap_wrapped}},
             overrides={"dialect": "ansi"},
