@@ -13,6 +13,10 @@ select as struct
   '1' as bb,
   2 as aa;
 
+select distinct as struct
+    '1' as bb,
+    2 as aa;
+
 -- Example of explicitly building a struct in a select clause.
 select
   struct(
@@ -46,3 +50,8 @@ SELECT
     some_other_field
   ) AS col
 FROM table;
+
+-- Empty STRUCT within TO_JSON
+SELECT
+  TO_JSON(STRUCT()) AS col
+FROM table;
