@@ -9,6 +9,8 @@ ALTER TABLE `user` RENAME AS `users`;
 
 ALTER TABLE `users` RENAME `user`;
 
+ALTER TABLE `users` RENAME COLUMN `col_1` TO `del_col_1`;
+
 ALTER TABLE `users`
 CHANGE COLUMN `birthday` `date_of_birth` INT(11) NULL DEFAULT NULL;
 
@@ -46,4 +48,16 @@ KEY_BLOCK_SIZE 8;
 ALTER TABLE `foo`.`bar` ADD INDEX `index_name`(`col_1`, `col_2`, `col_3`)
 KEY_BLOCK_SIZE 8 COMMENT 'index for col_1, col_2, col_3';
 
-ALTER TABLE `foo`.`bar` DROP INDEX `index_name`
+ALTER TABLE `foo`.`bar` DROP INDEX `index_name`;
+
+ALTER TABLE `foo`.`bar` RENAME INDEX `index_name` to `new_index_name`;
+
+ALTER TABLE `foo`.`bar` RENAME KEY `key_name` to `new_key_name`;
+
+ALTER TABLE `x` ADD CONSTRAINT FOREIGN KEY(`xk`) REFERENCES `y`(`yk`);
+
+ALTER TABLE `users`
+    ADD COLUMN `active` tinyint(1) DEFAULT '0';
+
+ALTER TABLE `users`
+    ADD COLUMN IF NOT EXISTS `active` tinyint(1) DEFAULT '0';
