@@ -18,6 +18,7 @@ CREATE TABLE `foo` (
   ts10 TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
   ts11 TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP(),
   ts12 TIMESTAMP NULL DEFAULT '0000-00-00 00:00:00',
-  ts13 TIMESTAMP NULL DEFAULT NOW,
-  ts14 TIMESTAMP NULL DEFAULT NOW()
+  ts13 TIMESTAMP NULL DEFAULT NOW ON UPDATE NOW,
+  ts14 TIMESTAMP NULL DEFAULT NOW() ON UPDATE NOW(),
+  ts15 TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
 )
