@@ -39,3 +39,13 @@ CREATE TABLE student (id INT, student_name STRING, age INT)
 USING CSV
 PARTITIONED BY (age)
 CLUSTERED BY (id) INTO 4 BUCKETS;
+
+CREATE EXTERNAL TABLE IF NOT EXISTS student (id INT, student_name STRING, age INT)
+USING iceberg
+PARTITIONED BY (age);
+
+CREATE TABLE student (id INT, student_name STRING, age INT)
+USING CSV
+COMMENT "this is a comment"
+PARTITIONED BY (age)
+STORED AS PARQUET;
