@@ -71,3 +71,15 @@ FROM person
         SUM(age) AS a, AVG(class) AS c
         FOR (name, age) IN ('John' AS c1, ('Mike', 40) AS c2)
     );
+
+
+SELECT * FROM (
+  some_table
+) PIVOT (
+  min(timestamp_ns) / 1e9 as min_timestamp_s -- this is the offending line
+
+  FOR run_id in (
+    test_run_id as test,
+    ctrl_run_id as ctrl
+  )
+);
