@@ -37,3 +37,17 @@ WHERE EXISTS (
         SELECT 1 FROM person
         WHERE person.id = person.id AND person.age IS NULL
     );
+
+SELECT 
+    name,
+    age
+FROM person
+WHERE person.id is distinct from person.age;
+
+SELECT 
+    name,
+    age
+FROM person
+WHERE person.id is not distinct from person.age
+
+
